@@ -18,10 +18,12 @@
 #include "autoconfig.h"
 
 #include <assert.h>
+#include <stdlib.h>
 
 #include <iostream>
 #include <string>
-#include <set>
+#include <algorithm>
+#include <tr1/unordered_set>
 #include <cstring>
 
 #include "textsplit.h"
@@ -30,9 +32,9 @@
 #include "utf8iter.h"
 #include "uproplist.h"
 
-#ifndef NO_NAMESPACES
+using std::tr1::unordered_set;
 using namespace std;
-#endif /* NO_NAMESPACES */
+
 
 /**
  * Splitting a text into words. The code in this file works with utf-8
@@ -50,15 +52,17 @@ using namespace std;
 // because it makes some tests in the code simpler.
 const unsigned int charclasses_size = 256;
 enum CharClass {LETTER=256, SPACE=257, DIGIT=258, WILD=259, 
-                A_ULETTER=260, A_LLETTER=261};
+                A_ULETTER=260, A_LLETTER=261, SKIP=262};
 static int charclasses[charclasses_size];
 
 // Real UTF-8 characters are handled with sets holding all characters
 // with interesting properties. This is far from full-blown management
 // of Unicode properties, but seems to do the job well enough in most
 // common cases
-static set<unsigned int> unicign;
-static set<unsigned int> visiblewhite;
+static vector<unsigned int> vpuncblocks;
+static unordered_set<unsigned int> spunc;
+static unordered_set<unsigned int> visiblewhite;
+static unordered_set<unsigned int> sskip;
 
 class CharClassInit {
 public:
@@ -86,18 +90,26 @@ public:
 	for (i = 0; i  < strlen(wild); i++)
 	    charclasses[int(wild[i])] = WILD;
 
-	char special[] = ".@+-,#'_\n\r";
+	char special[] = ".@+-#'_\n\r\f";
 	for (i = 0; i  < strlen(special); i++)
 	    charclasses[int(special[i])] = special[i];
 
-	for (i = 0; i < sizeof(uniign) / sizeof(int); i++) {
-	    unicign.insert(uniign[i]);
+	for (i = 0; i < sizeof(unipunc) / sizeof(int); i++) {
+	    spunc.insert(unipunc[i]);
 	}
-	unicign.insert((unsigned int)-1);
+	spunc.insert((unsigned int)-1);
+
+	for (i = 0; i < sizeof(unipuncblocks) / sizeof(int); i++) {
+	    vpuncblocks.push_back(unipuncblocks[i]);
+	}
+	assert((vpuncblocks.size() % 2) == 0);
 
 	for (i = 0; i < sizeof(avsbwht) / sizeof(int); i++) {
 	    visiblewhite.insert(avsbwht[i]);
 	}
+	for (i = 0; i < sizeof(uniskip) / sizeof(int); i++) {
+	    sskip.insert(uniskip[i]);
+	}
     }
 };
 static const CharClassInit charClassInitInstance;
@@ -107,13 +119,37 @@ static inline int whatcc(unsigned int c)
     if (c <= 127) {
 	return charclasses[c]; 
     } else {
-	if (unicign.find(c) != unicign.end())
+	if (sskip.find(c) != sskip.end()) {
+	    return SKIP;
+	} else if (spunc.find(c) != spunc.end()) {
 	    return SPACE;
-	else
-	    return LETTER;
+	} else {
+	    vector<unsigned int>::iterator it = 
+		lower_bound(vpuncblocks.begin(), vpuncblocks.end(), c);
+	    if (c == *it)
+		return SPACE;
+	    if ((it - vpuncblocks.begin()) % 2 == 1) {
+		return SPACE;
+	    } else {
+		return LETTER;
+	    }
+	} 
     }
 }
 
+// testing whatcc...
+#if 0
+  unsigned int testvalues[] = {'a', '0', 0x80, 0xbf, 0xc0, 0x05c3, 0x1000, 
+			       0x2000, 0x2001, 0x206e, 0x206f, 0x20d0, 0x2399, 
+			       0x2400, 0x2401, 0x243f, 0x2440, 0xff65};
+  int ntest = sizeof(testvalues) / sizeof(int);
+  for (int i = 0; i < ntest; i++) {
+      int ret = whatcc(testvalues[i]);
+      printf("Tested value 0x%x, returned value %d %s\n",
+	     testvalues[i], ret, ret == LETTER ? "LETTER" : 
+	     ret == SPACE ? "SPACE" : "OTHER");
+  }
+#endif
 
 // CJK Unicode character detection:
 //
@@ -169,14 +205,24 @@ inline bool TextSplit::emitterm(bool isspan, string &w, int pos,
     LOGDEB3(("TextSplit::emitterm: [%s] pos %d\n", w.c_str(), pos));
 
     unsigned int l = w.length();
+
+#ifdef TEXTSPLIT_STATS
+    // Update word length statistics. Do this before we filter out
+    // long words because stats are used to detect bad text
+    if (!isspan || m_wordLen == m_span.length())
+	m_stats.newsamp(m_wordChars);
+#endif
+
     if (l > 0 && l < (unsigned)m_maxWordLength) {
 	// 1 byte word: we index single ascii letters and digits, but
 	// nothing else. We might want to turn this into a test for a
 	// single utf8 character instead ?
 	if (l == 1) {
-	    int c = (int)w[0];
+	    unsigned int c = ((unsigned int)w[0]) & 0xff;
 	    if (charclasses[c] != A_ULETTER && charclasses[c] != A_LLETTER && 
-                charclasses[c] != DIGIT) {
+                charclasses[c] != DIGIT &&
+		(!(m_flags & TXTS_KEEPWILD) || charclasses[c] != WILD)
+		) {
 		//cerr << "ERASING single letter term " << c << endl;
 		return true;
 	    }
@@ -278,7 +324,7 @@ inline bool TextSplit::doemit(bool spanerase, int bp, bool spanemit)
     // Adjust state
     if (m_wordLen) {
 	m_wordpos++;
-	m_wordLen = 0;
+	m_wordLen = m_wordChars = 0;
     }
     if (spanerase) {
 	discardspan();
@@ -294,9 +340,26 @@ void TextSplit::discardspan()
     m_span.erase();
     m_spanpos = m_wordpos;
     m_wordStart = 0;
-    m_wordLen = 0;
+    m_wordLen = m_wordChars = 0;
 }
 
+static inline bool isalphanum(int what, unsigned int flgs)
+{
+    return what == A_LLETTER || what == A_ULETTER ||
+	what == DIGIT || what == LETTER ||
+	((flgs & TextSplit::TXTS_KEEPWILD) && what == WILD);
+}
+static inline bool isdigit(int what, unsigned int flgs)
+{
+    return what == DIGIT || ((flgs & TextSplit::TXTS_KEEPWILD) && what == WILD);
+}
+
+#ifdef TEXTSPLIT_STATS
+#define INC_WORDCHARS ++m_wordChars
+#else
+#define INC_WORDCHARS
+#endif
+
 /** 
  * Splitting a text into terms to be indexed.
  * We basically emit a word every time we see a separator, but some chars are
@@ -312,10 +375,16 @@ bool TextSplit::text_to_words(const string &in)
 	     m_flags & TXTS_KEEPWILD ? " keepwild" : "",
 	     in.substr(0,50).c_str()));
 
+    if (in.empty())
+	return true;
+
     m_span.erase();
     m_inNumber = false;
-    m_wordStart = m_wordLen = m_prevpos = m_prevlen = m_wordpos = m_spanpos = 0;
+    m_wordStart = m_wordLen = m_wordChars = m_prevpos = m_prevlen = m_wordpos 
+	= m_spanpos = 0;
     int curspanglue = 0;
+    bool pagepending = false;
+    bool softhyphenpending = false;
 
     // Running count of non-alphanum chars. Reset when we see one;
     int nonalnumcnt = 0;
@@ -353,10 +422,23 @@ bool TextSplit::text_to_words(const string &in)
 
 	int cc = whatcc(c);
 	switch (cc) {
+	case SKIP:
+	    // Special-case soft-hyphen. To work, this depends on the
+	    // fact that only SKIP calls "continue" inside the
+	    // switch. All the others will do the softhyphenpending
+	    // reset after the switch
+	    if (c == 0xad) {
+		softhyphenpending = true;
+	    } else {
+		softhyphenpending = false;
+	    }
+	    // Skips the softhyphenpending reset
+	    continue;
 	case DIGIT:
 	    if (m_wordLen == 0)
 		m_inNumber = true;
 	    m_wordLen += it.appendchartostring(m_span);
+	    INC_WORDCHARS;
 	    nonalnumcnt = 0;
 	    break;
 
@@ -369,6 +451,10 @@ bool TextSplit::text_to_words(const string &in)
 		    return false;
 		m_inNumber = false;
 	    }
+	    if (pagepending) {
+		pagepending = false;
+		newpage(m_wordpos);
+	    }
 	    break;
 
 	case WILD:
@@ -384,39 +470,37 @@ bool TextSplit::text_to_words(const string &in)
 	    if (m_wordLen == 0) {
 		// + or - don't start a term except if this looks like
 		// it's going to be to be a number
-		if (whatcc(it[it.getCpos()+1]) == DIGIT) {
+		if (isdigit(whatcc(it[it.getCpos()+1]), m_flags)) {
 		    // -10
 		    m_inNumber = true;
 		    m_wordLen += it.appendchartostring(m_span);
+		    INC_WORDCHARS;
 		} else {
 		    goto SPACE;
 		} 
 	    } else if (m_inNumber && (m_span[m_span.length() - 1] == 'e' ||
 				      m_span[m_span.length() - 1] == 'E')) {
-		if (whatcc(it[it.getCpos()+1]) == DIGIT) {
+		if (isdigit(whatcc(it[it.getCpos()+1]), m_flags)) {
 		    m_wordLen += it.appendchartostring(m_span);
+		    INC_WORDCHARS;
 		} else {
 		    goto SPACE;
 		}
 	    } else {
-		if (!doemit(false, it.getBpos()))
-		    return false;
-		m_inNumber = false;
-		m_wordStart += it.appendchartostring(m_span);
+		goto SPACE;
 	    }
 	    break;
 
 	case '.':
-	case ',':
 	{
 	    // Need a little lookahead here. At worse this gets the end null
 	    int nextc = it[it.getCpos()+1];
 	    int nextwhat = whatcc(nextc);
 	    if (m_inNumber) {
-		// we're eliminating 132.jpg here. Good idea ?
-		if (nextwhat != DIGIT && nextc != 'e' && nextc != 'E')
+		if (!isdigit(nextwhat, m_flags))
 		    goto SPACE;
 		m_wordLen += it.appendchartostring(m_span);
+		INC_WORDCHARS;
 		curspanglue = cc;
 		break;
 	    } else {
@@ -428,15 +512,15 @@ bool TextSplit::text_to_words(const string &in)
 		// A final comma in a word will be removed by doemit
 
 		// Only letters and digits make sense after
-		if (nextwhat != A_LLETTER && nextwhat != A_ULETTER && 
-		    nextwhat != DIGIT && nextwhat != LETTER)
+		if (!isalphanum(nextwhat, m_flags))
 		    goto SPACE;
 
 		if (cc == '.') {
                     // Check for number like .1
-                    if (m_span.length() == 0 && nextwhat == DIGIT) {
+                    if (m_span.length() == 0 && isdigit(nextwhat, m_flags)) {
                         m_inNumber = true;
                         m_wordLen += it.appendchartostring(m_span);
+			INC_WORDCHARS;
                         curspanglue = cc;
                         break;
                     }
@@ -472,8 +556,10 @@ bool TextSplit::text_to_words(const string &in)
 		    return false;
 		curspanglue = cc;
 		m_inNumber = false;
+		m_wordStart += it.appendchartostring(m_span);
+	    } else {
+		goto SPACE;
 	    }
-	    m_wordStart += it.appendchartostring(m_span);
 	    break;
 	case '_':
 	    if (m_wordLen) {
@@ -501,6 +587,7 @@ bool TextSplit::text_to_words(const string &in)
 		int w = whatcc(it[it.getCpos()+1]);
 		if (w == SPACE || w == '\n' || w == '\r') {
 		    m_wordLen += it.appendchartostring(m_span);
+		    INC_WORDCHARS;
 		    break;
 		}
 	    }
@@ -508,20 +595,24 @@ bool TextSplit::text_to_words(const string &in)
 	    break;
 	case '\n':
 	case '\r':
-	    if (m_span.length() && m_span[m_span.length() - 1] == '-') {
+	    if ((m_span.length() && m_span[m_span.length() - 1] == '-') ||
+		softhyphenpending) {
 		// if '-' is the last char before end of line, just
 		// ignore the line change. This is the right thing to
 		// do almost always. We'd then need a way to check if
 		// the - was added as part of the word hyphenation, or was 
 		// there in the first place, but this would need a dictionary.
-		// Also we'd need to check for a soft-hyphen and remove it,
-		// but this would require more utf-8 magic
+		// Don't reset soft-hyphen
+		continue;
 	    } else {
 		// Handle like a normal separator
 		goto SPACE;
 	    }
 	    break;
-
+	case '\f':
+	    pagepending = true;
+	    goto SPACE;
+	    break;
 #ifdef RCL_SPLIT_CAMELCASE
             // Camelcase handling. 
             // If we get uppercase ascii after lowercase ascii, emit word.
@@ -569,9 +660,11 @@ bool TextSplit::text_to_words(const string &in)
                 m_inNumber = false;
             }
 	    m_wordLen += it.appendchartostring(m_span);
+	    INC_WORDCHARS;
 	    nonalnumcnt = 0;
 	    break;
 	}
+	softhyphenpending = false;
     }
     if (m_wordLen || m_span.length()) {
 	if (!doemit(true, it.getBpos()))
@@ -667,7 +760,7 @@ bool TextSplit::cjk_to_words(Utf8Iter *itp, unsigned int *cp)
 
     m_span.erase();
     m_inNumber = false;
-    m_wordStart = m_wordLen = m_prevpos = m_prevlen = 0;
+    m_wordStart = m_wordLen = m_wordChars = m_prevpos = m_prevlen = 0;
     m_spanpos = m_wordpos;
     *cp = c;
     return true;
@@ -780,17 +873,20 @@ template <class T> bool u8stringToStrings(const string &s, T &tokens)
     return true;
 }
 
-bool TextSplit::stringToStrings(const string &s, list<string> &tokens)
+bool TextSplit::stringToStrings(const string &s, vector<string> &tokens)
 {
-    return u8stringToStrings<list<string> >(s, tokens);
+    return u8stringToStrings<vector<string> >(s, tokens);
 }
 
 #else  // TEST driver ->
 
+#include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <stdio.h>
+#include <string.h>
+#include <math.h>
 
 #include <iostream>
 
@@ -807,7 +903,7 @@ class myTermProc : public Rcl::TermProc {
     int first;
     bool nooutput;
 public:
-    myTermProc() : TermProc(0), first(1), nooutput(false)  {}
+    myTermProc() : TermProc(0), first(1), nooutput(false) {}
     void setNoOut(bool val) {nooutput = val;}
     virtual bool takeword(const string &term, int pos, int bs, int be)
     {
@@ -839,6 +935,10 @@ static string teststring =
 	    " ,able,test-domain "
 	    " -wl,--export-dynamic "
 	    " ~/.xsession-errors "
+    "soft\xc2\xadhyphen "
+    "soft\xc2\xad\nhyphen "
+    "soft\xc2\xad\n\rhyphen "
+    "hard-\nhyphen "
 ;
 
 static string teststring1 = " nouvel-an ";
@@ -981,7 +1081,16 @@ int main(int argc, char **argv)
             printproc.setNoOut(true);
 
 	splitter.text_to_words(data);
-
+#ifdef TEXTSPLIT_STATS
+	TextSplit::Stats::Values v = splitter.getStats();
+	cout << "Average length: " 
+	     <<  v.avglen
+	     << " Standard deviation: " 
+	     << v.sigma
+	     << " Coef of variation "
+	     << v.sigma / v.avglen
+	     << endl;
+#endif
     }    
 }
 #endif // TEST
