@@ -76,12 +76,6 @@ class FsIndexer : public FsTreeWalkerCB {
     /** Make signature for file up to date checks */
     static void makesig(const struct stat *stp, string& out);
 
-    /* Hold the description for an external metadata-gathering command */
-    struct MDReaper {
-	string fieldname;
-	vector<string> cmdv;
-    };
-
  private:
 
     class PurgeCandidateRecorder {
@@ -131,16 +125,13 @@ class FsIndexer : public FsTreeWalkerCB {
     // all files in a file system area. Ie: set "rclaptg = thunderbird"
     // inside ~/.thunderbird. The boolean is set at init to avoid
     // further wasteful processing if no local fields are set.
+    // This should probably moved to internfile so that the
+    // localfields get exactly the same processing as those generated by the
+    // filters (as was done for metadatacmds fields)
     bool         m_havelocalfields;
     string       m_slocalfields;
     map<string, string>  m_localfields;
 
-    // Same idea with the metadata-gathering external commands,
-    // (e.g. used to reap tagging info: "tmsu tags %f")
-    bool m_havemdreapers;
-    string m_smdreapers;
-    vector<MDReaper> m_mdreapers;
-
 #ifdef IDX_THREADS
     friend void *FsIndexerDbUpdWorker(void*);
     friend void *FsIndexerInternfileWorker(void*);
@@ -154,15 +145,11 @@ class FsIndexer : public FsTreeWalkerCB {
 
     bool init();
     void localfieldsfromconf();
-    void mdreapersfromconf();
     void setlocalfields(const map<string, string>& flds, Rcl::Doc& doc);
-    void reapmetadata(const vector<MDReaper>& reapers, const string &fn,
-		      Rcl::Doc& doc);
     string getDbDir() {return m_config->getDbDir();}
     FsTreeWalker::Status 
     processonefile(RclConfig *config, const string &fn, 
-		   const struct stat *, const map<string,string>& localfields,
-		   const vector<MDReaper>& mdreapers);
+		   const struct stat *, const map<string,string>& localfields);
 };
 
 #endif /* _fsindexer_h_included_ */
