@@ -33,17 +33,27 @@
  * to FAM or inotify and place events on the event queue.
  */
 
-/** A small virtual interface for monitors. Probably suitable to let
-    either of fam/gamin or raw imonitor hide behind */
+/** A small virtual interface for monitors. Lets
+ *  either fam/gamin or raw imonitor hide behind 
+ */
 class RclMonitor {
 public:
-    RclMonitor(){}
-    virtual ~RclMonitor() {}
+    RclMonitor()
+	: saved_errno(0)
+    {
+    }
+
+    virtual ~RclMonitor() 
+    {
+    }
     virtual bool addWatch(const string& path, bool isDir) = 0;
     virtual bool getEvent(RclMonEvent& ev, int msecs = -1) = 0;
     virtual bool ok() const = 0;
     // Does this monitor generate 'exist' events at startup?
     virtual bool generatesExist() const = 0; 
+
+    // Save significant errno after monitor calls
+    int saved_errno;
 };
 
 // Monitor factory. We only have one compiled-in kind at a time, no
@@ -94,8 +104,14 @@ public:
 		    break;
 		}
 	    }
-	    if (!m_mon || !m_mon->ok() || !m_mon->addWatch(fn, true))
+	    if (!m_mon || !m_mon->ok())
 		return FsTreeWalker::FtwError;
+	    // We do nothing special if addWatch fails for a reasonable reason
+	    if (!m_mon->addWatch(fn, true)) {
+		if (m_mon->saved_errno != EACCES && 
+		    m_mon->saved_errno != ENOENT)
+		    return FsTreeWalker::FtwError;
+	    }
 	} else if (!m_mon->generatesExist() && 
 		   flg == FsTreeWalker::FtwRegular) {
 	    // Have to synthetize events for regular files existence
@@ -156,7 +172,7 @@ void *rclMonRcvRun(void *q)
 
 
     // Get top directories from config 
-    list<string> tdl = lconfig.getTopdirs();
+    vector<string> tdl = lconfig.getTopdirs();
     if (tdl.empty()) {
 	LOGERR(("rclMonRcvRun:: top directory list (topdirs param.) not"
 		"found in config or Directory list parse error"));
@@ -168,7 +184,7 @@ void *rclMonRcvRun(void *q)
     FsTreeWalker walker;
     walker.setSkippedPaths(lconfig.getDaemSkippedPaths());
     WalkCB walkcb(&lconfig, mon, queue, walker);
-    for (list<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
+    for (vector<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
 	lconfig.setKeyDir(*it);
 	// Adjust the follow symlinks options
 	bool follow;
@@ -190,15 +206,14 @@ void *rclMonRcvRun(void *q)
     }
 
     {
-	bool dobeagle = false;
-	lconfig.getConfParam("processbeaglequeue", &dobeagle);
-	if (dobeagle) {
-	    string beaglequeuedir;
-	    if (!lconfig.getConfParam("beaglequeuedir", beaglequeuedir))
-		beaglequeuedir = path_tildexpand("~/.beagle/ToIndex/");
-	    if (!mon->addWatch(beaglequeuedir, true)) {
-		LOGERR(("rclMonRcvRun: addwatch (beaglequeuedit) failed\n"));
-		goto terminate;
+	bool doweb = false;
+	lconfig.getConfParam("processwebqueue", &doweb);
+	if (doweb) {
+	    string webqueuedir = lconfig.getWebQueueDir();
+	    if (!mon->addWatch(webqueuedir, true)) {
+		LOGERR(("rclMonRcvRun: addwatch (webqueuedir) failed\n"));
+		if (mon->saved_errno != EACCES && mon->saved_errno != ENOENT)
+		    goto terminate;
 	    }
 	}
     }
@@ -552,22 +567,22 @@ const char *RclIntf::event_name(int code)
     code &= ~(IN_ISDIR|IN_ONESHOT);
     switch (code) {
     case IN_ACCESS: return "IN_ACCESS";
-    case IN_MODIFY: return "IN_MODIFY";
     case IN_ATTRIB: return "IN_ATTRIB";
-    case IN_CLOSE_WRITE: return "IN_CLOSE_WRITE";
-    case IN_CLOSE_NOWRITE: return "IN_CLOSE_NOWRITE";
     case IN_CLOSE: return "IN_CLOSE";
-    case IN_OPEN: return "IN_OPEN";
-    case IN_MOVED_FROM: return "IN_MOVED_FROM";
-    case IN_MOVED_TO: return "IN_MOVED_TO";
-    case IN_MOVE: return "IN_MOVE";
+    case IN_CLOSE_NOWRITE: return "IN_CLOSE_NOWRITE";
+    case IN_CLOSE_WRITE: return "IN_CLOSE_WRITE";
     case IN_CREATE: return "IN_CREATE";
     case IN_DELETE: return "IN_DELETE";
     case IN_DELETE_SELF: return "IN_DELETE_SELF";
-    case IN_MOVE_SELF: return "IN_MOVE_SELF";
-    case IN_UNMOUNT: return "IN_UNMOUNT";
-    case IN_Q_OVERFLOW: return "IN_Q_OVERFLOW";
     case IN_IGNORED: return "IN_IGNORED";
+    case IN_MODIFY: return "IN_MODIFY";
+    case IN_MOVE: return "IN_MOVE";
+    case IN_MOVED_FROM: return "IN_MOVED_FROM";
+    case IN_MOVED_TO: return "IN_MOVED_TO";
+    case IN_MOVE_SELF: return "IN_MOVE_SELF";
+    case IN_OPEN: return "IN_OPEN";
+    case IN_Q_OVERFLOW: return "IN_Q_OVERFLOW";
+    case IN_UNMOUNT: return "IN_UNMOUNT";
     default: {
 	static char msg[50];
 	sprintf(msg, "Unknown event 0x%x", code);
@@ -584,6 +599,12 @@ bool RclIntf::addWatch(const string& path, bool)
     // CLOSE_WRITE is covered through MODIFY. CREATE is needed for mkdirs
     uint32_t mask = IN_MODIFY | IN_CREATE
         | IN_MOVED_FROM | IN_MOVED_TO | IN_DELETE
+#ifdef RCL_USE_XATTR
+	// It seems that IN_ATTRIB is not needed to receive extattr
+	// modification events, which is a bit weird because only ctime is
+	// set. 
+	// | IN_ATTRIB
+#endif
 #ifdef IN_DONT_FOLLOW
 	| IN_DONT_FOLLOW
 #endif
@@ -593,7 +614,9 @@ bool RclIntf::addWatch(const string& path, bool)
 	;
     int wd;
     if ((wd = inotify_add_watch(m_fd, path.c_str(), mask)) < 0) {
-        LOGERR(("RclIntf::addWatch: inotify_add_watch failed\n"));
+	saved_errno = errno;
+        LOGERR(("RclIntf::addWatch: inotify_add_watch failed. errno %d\n",
+		   saved_errno));
 	return false;
     }
     m_idtopath[wd] = path;
@@ -675,7 +698,7 @@ bool RclIntf::getEvent(RclMonEvent& ev, int msecs)
 	eraseWatchSubTree(m_idtopath, ev.m_path);
     }
 
-
+    // IN_ATTRIB apparently not needed, see comment above
     if (evp->mask & (IN_MODIFY)) {
 	ev.m_etyp = RclMonEvent::RCLEVT_MODIFY;
     } else if (evp->mask & (IN_DELETE | IN_MOVED_FROM)) {
