@@ -65,6 +65,9 @@ static int     op_flags;
 #define OPT_b     0x2000
 #define OPT_f     0x4000
 #define OPT_C     0x8000
+#define OPT_Z     0x10000
+#define OPT_n     0x20000
+#define OPT_r     0x40000
 
 ReExec *o_reexec;
 
@@ -86,7 +89,7 @@ int stopindexing;
 // should subsequently orderly terminate what it is doing.
 class MyUpdater : public DbIxStatusUpdater {
  public:
-    MyUpdater(RclConfig *config) 
+    MyUpdater(const RclConfig *config) 
 	: m_prevphase(DbIxStatus::DBIXS_NONE)
     {
 	m_fd = open(config->getIdxStatusFile().c_str(), 
@@ -110,7 +113,10 @@ class MyUpdater : public DbIxStatusUpdater {
 	    fprintf(fp, "filesdone = %d\n", status.filesdone);
 	    fprintf(fp, "dbtotdocs = %d\n", status.dbtotdocs);
 	    fprintf(fp, "fn = %s\n", status.fn.c_str());
-	    ftruncate(m_fd, off_t(ftell(fp)));
+	    if (ftruncate(m_fd, off_t(ftell(fp))) < 0) {
+		// ? kill compiler warning about ignoring ftruncate return
+		LOGDEB(("Status update: ftruncate failed\n"));
+	    }
             // Flush data and closes fd1. m_fd still valid
 	    fclose(fp); 
 	}
@@ -147,18 +153,20 @@ static void sigcleanup(int sig)
     stopindexing = 1;
 }
 
-static bool makeIndexer(RclConfig *config)
+static void makeIndexerOrExit(RclConfig *config, bool inPlaceReset)
 {
-    if (!confindexer)
+    if (!confindexer) {
 	confindexer = new ConfIndexer(config, updater);
+	if (inPlaceReset)
+	    confindexer->setInPlaceReset();
+    }
     if (!confindexer) {
         cerr << "Cannot create indexer" << endl;
         exit(1);
     }
-    return true;
 }
 
-void rclIxIonice(RclConfig *config)
+void rclIxIonice(const RclConfig *config)
 {
     string clss, classdata;
     if (!config->getConfParam("monioniceclass", clss) || clss.empty())
@@ -167,19 +175,45 @@ void rclIxIonice(RclConfig *config)
     rclionice(clss, classdata);
 }
 
-// Index a list of files. We just check that they belong to one of the
-// topdirs subtrees, and call the indexer method. 
+class MakeListWalkerCB : public FsTreeWalkerCB {
+public:
+    MakeListWalkerCB(list<string>& files)
+	: m_files(files)
+    {
+    }
+    virtual FsTreeWalker::Status 
+    processone(const string & fn, const struct stat *, FsTreeWalker::CbFlag flg) 
+    {
+	if (flg == FsTreeWalker::FtwDirEnter || flg == FsTreeWalker::FtwRegular)
+	    m_files.push_back(fn);
+	return FsTreeWalker::FtwOk;
+    }
+    list<string>& m_files;
+};
+
+// Build a list of things to index and call indexfiles.
+bool recursive_index(RclConfig *config, const string& top)
+{
+    list<string> files;
+    MakeListWalkerCB cb(files);
+    FsTreeWalker walker;
+    walker.walk(top, cb);
+    return indexfiles(config, files);
+}
+
+// Index a list of files. We just call the top indexer method, which
+// will sort out what belongs to the indexed trees and call the
+// appropriate indexers.
 //
 // This is called either from the command line or from the monitor. In
 // this case we're called repeatedly in the same process, and the
-// confindexer is only created once by makeIndexer (but the db closed and
+// confindexer is only created once by makeIndexerOrExit (but the db closed and
 // flushed every time)
 bool indexfiles(RclConfig *config, list<string> &filenames)
 {
     if (filenames.empty())
 	return true;
-    if (!makeIndexer(config))
-	return false;
+    makeIndexerOrExit(config, (op_flags & OPT_Z) != 0);
     return confindexer->indexFiles(filenames, (op_flags&OPT_f) ? 
 				   ConfIndexer::IxFIgnoreSkip : 
 				   ConfIndexer::IxFNone);
@@ -190,16 +224,14 @@ bool purgefiles(RclConfig *config, list<string> &filenames)
 {
     if (filenames.empty())
 	return true;
-    if (!makeIndexer(config))
-	return false;
+    makeIndexerOrExit(config, (op_flags & OPT_Z) != 0);
     return confindexer->purgeFiles(filenames);
 }
 
 // Create stemming and spelling databases
 bool createAuxDbs(RclConfig *config)
 {
-    if (!makeIndexer(config))
-	return false;
+    makeIndexerOrExit(config, false);
 
     if (!confindexer->createStemmingDatabases())
 	return false;
@@ -213,8 +245,7 @@ bool createAuxDbs(RclConfig *config)
 // Create additional stem database 
 static bool createstemdb(RclConfig *config, const string &lang)
 {
-    if (!makeIndexer(config))
-        return false;
+    makeIndexerOrExit(config, false);
     return confindexer->createStemDb(lang);
 }
 
@@ -224,29 +255,36 @@ static const char usage [] =
 "\n"
 "recollindex [-h] \n"
 "    Print help\n"
-"recollindex [-z] \n"
+"recollindex [-z|-Z] \n"
 "    Index everything according to configuration file\n"
 "    -z : reset database before starting indexing\n"
+"    -Z : in place reset: consider all documents as changed. Can also\n"
+"         be combined with -i or -r but not -m\n"
 #ifdef RCL_MONITOR
 "recollindex -m [-w <secs>] -x [-D] [-C]\n"
 "    Perform real time indexing. Don't become a daemon if -D is set.\n"
 "    -w sets number of seconds to wait before starting.\n"
 "    -C disables monitoring config for changes/reexecuting.\n"
+"    -n disables initial incremental indexing (!and purge!).\n"
 #ifndef DISABLE_X11MON
 "    -x disables exit on end of x11 session\n"
 #endif /* DISABLE_X11MON */
 #endif /* RCL_MONITOR */
 "recollindex -e <filename [filename ...]>\n"
 "    Purge data for individual files. No stem database updates\n"
-"recollindex -i [-f] <filename [filename ...]>\n"
+"recollindex -i [-f] [-Z] <filename [filename ...]>\n"
 "    Index individual files. No database purge or stem database updates\n"
 "    -f : ignore skippedPaths and skippedNames while doing this\n"
+"recollindex -r [-f] [-Z] <top> \n"
+"   Recursive partial reindex\n"
 "recollindex -l\n"
 "    List available stemming languages\n"
 "recollindex -s <lang>\n"
 "    Build stem database for additional language <lang>\n"
+#ifdef FUTURE_IMPROVEMENT
 "recollindex -b\n"
 "    Process the Beagle queue\n"
+#endif
 #ifdef RCL_USE_ASPELL
 "recollindex -S\n"
 "    Build aspell spelling dictionary.>\n"
@@ -274,12 +312,21 @@ void lockorexit(Pidfile *pidfile)
 	    ". Return (other pid?): " << pid << endl;
 	exit(1);
     }
+    if (pidfile->write_pid() != 0) {
+	cerr << "Can't become exclusive indexer: " << pidfile->getreason() <<
+	    endl;
+	exit(1);
+    }
 }
 
 int main(int argc, char **argv)
 {
     string a_config;
     int sleepsecs = 60;
+
+    // The reexec struct is used by the daemon to shed memory after
+    // the initial indexing pass and to restart when the configuration
+    // changes
     o_reexec = new ReExec;
     o_reexec->init(argc, argv);
 
@@ -306,6 +353,8 @@ int main(int argc, char **argv)
 	    case 'i': op_flags |= OPT_i; break;
 	    case 'l': op_flags |= OPT_l; break;
 	    case 'm': op_flags |= OPT_m; break;
+	    case 'n': op_flags |= OPT_n; break;
+	    case 'r': op_flags |= OPT_r; break;
 	    case 's': op_flags |= OPT_s; break;
 #ifdef RCL_USE_ASPELL
 	    case 'S': op_flags |= OPT_S; break;
@@ -315,6 +364,7 @@ int main(int argc, char **argv)
 		    Usage(); 
 		argc--; goto b1;
 	    case 'x': op_flags |= OPT_x; break;
+	    case 'Z': op_flags |= OPT_Z; break;
 	    case 'z': op_flags |= OPT_z; break;
 	    default: Usage(); break;
 	    }
@@ -330,7 +380,9 @@ int main(int argc, char **argv)
     }
 #endif
 
-    if ((op_flags & OPT_z) && (op_flags & (OPT_i|OPT_e)))
+    if ((op_flags & OPT_z) && (op_flags & (OPT_i|OPT_e|OPT_r)))
+	Usage();
+    if ((op_flags & OPT_Z) && (op_flags & (OPT_m)))
 	Usage();
 
     string reason;
@@ -343,10 +395,33 @@ int main(int argc, char **argv)
     }
     o_reexec->atexit(cleanup);
 
-    bool rezero(op_flags & OPT_z);
+    string rundir;
+    config->getConfParam("idxrundir", rundir);
+    if (!rundir.compare("tmp")) {
+	LOGINFO(("recollindex: changing current directory to [%s]\n",
+		tmplocation().c_str()));
+	if (chdir(tmplocation().c_str()) < 0) {
+	    LOGERR(("chdir(%s) failed, errno %d\n", 
+		    tmplocation().c_str(), errno));
+	}
+    } else if (!rundir.empty()) {
+	LOGINFO(("recollindex: changing current directory to [%s]\n",
+		 rundir.c_str()));
+	if (chdir(rundir.c_str()) < 0) {
+	    LOGERR(("chdir(%s) failed, errno %d\n", 
+		    rundir.c_str(), errno));
+	}
+    }
+
+    bool rezero((op_flags & OPT_z) != 0);
+    bool inPlaceReset((op_flags & OPT_Z) != 0);
     Pidfile pidfile(config->getPidfile());
     updater = new MyUpdater(config);
 
+    // Log something at LOGINFO to reset the trace file. Else at level
+    // 3 it's not even truncated if all docs are up to date.
+    LOGINFO(("recollindex: starting up\n"));
+
     if (setpriority(PRIO_PROCESS, 0, 20) != 0) {
         LOGINFO(("recollindex: can't setpriority(), errno %d\n", errno));
     }
@@ -355,7 +430,6 @@ int main(int argc, char **argv)
 
     if (op_flags & (OPT_i|OPT_e)) {
 	lockorexit(&pidfile);
-	pidfile.write_pid();
 
 	list<string> filenames;
 
@@ -377,14 +451,22 @@ int main(int argc, char **argv)
 	    status = indexfiles(config, filenames);
 	else 
 	    status = purgefiles(config, filenames);
-        if (!confindexer->getReason().empty())
+        if (confindexer && !confindexer->getReason().empty())
+            cerr << confindexer->getReason() << endl;
+        exit(status ? 0 : 1);
+    } else if (op_flags & OPT_r) {
+	if (argc != 1) 
+	    Usage();
+	string top = *argv++; argc--;
+	bool status = recursive_index(config, top);
+        if (confindexer && !confindexer->getReason().empty())
             cerr << confindexer->getReason() << endl;
         exit(status ? 0 : 1);
     } else if (op_flags & OPT_l) {
 	if (argc != 0) 
 	    Usage();
-	list<string> stemmers = ConfIndexer::getStemmerNames();
-	for (list<string>::const_iterator it = stemmers.begin(); 
+	vector<string> stemmers = ConfIndexer::getStemmerNames();
+	for (vector<string>::const_iterator it = stemmers.begin(); 
 	     it != stemmers.end(); it++) {
 	    cout << *it << endl;
 	}
@@ -394,6 +476,11 @@ int main(int argc, char **argv)
 	    Usage();
 	string lang = *argv++; argc--;
 	exit(!createstemdb(config, lang));
+#ifdef RCL_USE_ASPELL
+    } else if (op_flags & OPT_S) {
+	makeIndexerOrExit(config, false);
+        exit(!confindexer->createAspellDict());
+#endif // ASPELL
 
 #ifdef RCL_MONITOR
     } else if (op_flags & OPT_m) {
@@ -408,6 +495,7 @@ int main(int argc, char **argv)
 	      exit(1);
 	    }
 	}
+	// Need to rewrite pid, it changed
 	pidfile.write_pid();
 
         // Not too sure if I have to redo the nice thing after daemon(),
@@ -429,17 +517,29 @@ int main(int argc, char **argv)
 	      }
 	    }
 	}
-	confindexer = new ConfIndexer(config, updater);
-	if (!confindexer->index(rezero, ConfIndexer::IxTAll) || stopindexing) {
-	  LOGERR(("recollindex, initial indexing pass failed, not going into monitor mode\n"));
-	  exit(1);
+	if (!(op_flags & OPT_n)) {
+	    makeIndexerOrExit(config, inPlaceReset);
+	    LOGDEB(("Recollindex: initial indexing pass before monitoring\n"));
+	    if (!confindexer->index(rezero, ConfIndexer::IxTAll) || 
+		stopindexing) {
+		LOGERR(("recollindex, initial indexing pass failed, "
+			"not going into monitor mode\n"));
+		exit(1);
+	    }
+	    deleteZ(confindexer);
+	    o_reexec->insertArgs(vector<string>(1, "-n"));
+	    LOGINFO(("recollindex: reexecuting with -n after initial full pass\n"));
+	    // Note that -n will be inside the reexec when we come
+	    // back, but the monitor will explicitely strip it before
+	    // starting a config change exec to ensure that we do a
+	    // purging pass in this case.
+	    o_reexec->reexec();
 	}
         if (updater) {
 	    updater->status.phase = DbIxStatus::DBIXS_MONITOR;
 	    updater->status.fn.clear();
 	    updater->update();
 	}
-	deleteZ(confindexer);
 	int opts = RCLMON_NONE;
 	if (op_flags & OPT_D)
 	    opts |= RCLMON_NOFORK;
@@ -452,19 +552,12 @@ int main(int argc, char **argv)
 	exit(monret == false);
 #endif // MONITOR
 
-#ifdef RCL_USE_ASPELL
-    } else if (op_flags & OPT_S) {
-	if (!makeIndexer(config))
-            exit(1);
-        exit(!confindexer->createAspellDict());
-#endif // ASPELL
     } else if (op_flags & OPT_b) {
         cerr << "Not yet" << endl;
         return 1;
     } else {
 	lockorexit(&pidfile);
-	pidfile.write_pid();
-	confindexer = new ConfIndexer(config, updater);
+	makeIndexerOrExit(config, inPlaceReset);
 	bool status = confindexer->index(rezero, ConfIndexer::IxTAll);
 	if (!status) 
 	    cerr << "Indexing failed" << endl;
