@@ -118,6 +118,28 @@ void FileInterner::reapXAttrs(const string& path)
 }
 #endif // RCL_USE_XATTR
 
+void FileInterner::reapCmdMetadata(const string& fn)
+{
+    const vector<MDReaper>& reapers = m_cfg->getMDReapers();
+    if (reapers.empty())
+	return;
+    map<char,string> smap = create_map<char, string>('f', fn);
+    for (vector<MDReaper>::const_iterator rp = reapers.begin();
+	 rp != reapers.end(); rp++) {
+	vector<string> cmd;
+	for (vector<string>::const_iterator it = rp->cmdv.begin();
+	     it != rp->cmdv.end(); it++) {
+	    string s;
+	    pcSubst(*it, s, smap);
+	    cmd.push_back(s);
+	}
+	string output;
+	if (ExecCmd::backtick(cmd, output)) {
+	    m_cmdFields[rp->fieldname] =  output;
+	}
+    }
+}
+
 // This is used when the user wants to retrieve a search result doc's parent
 // (ie message having a given attachment)
 bool FileInterner::getEnclosingUDI(const Rcl::Doc &doc, string& udi)
@@ -265,20 +287,22 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
     if (!df or df->is_unknown()) {
 	// No real handler for this type, for now :( 
 	LOGDEB(("FileInterner:: unprocessed mime: [%s] [%s]\n", 
-		 l_mime.c_str(), f.c_str()));
+		l_mime.c_str(), f.c_str()));
 	if (!df)
 	    return;
     }
     df->set_property(Dijon::Filter::OPERATING_MODE, 
-			    m_forPreview ? "view" : "index");
+		     m_forPreview ? "view" : "index");
     df->set_property(Dijon::Filter::DJF_UDI, udi);
 
 #ifdef RCL_USE_XATTR
     // Get fields computed from extended attributes. We use the
     // original file, not the m_fn which may be the uncompressed temp
     // file
-    reapXAttrs(f);
+    if (!m_noxattrs)
+	reapXAttrs(f);
 #endif //RCL_USE_XATTR
+    reapCmdMetadata(f);
 
     df->set_docsize(docsize);
     if (!df->set_document_file(l_mime, m_fn)) {
@@ -356,6 +380,7 @@ void FileInterner::initcommon(RclConfig *cnf, int flags)
     for (unsigned int i = 0; i < MAXHANDLERS; i++)
 	m_tmpflgs[i] = false;
     m_targetMType = cstr_textplain;
+    m_cfg->getConfParam("noxattrfields", &m_noxattrs);
 }
 
 FileInterner::FileInterner(const Rcl::Doc& idoc, RclConfig *cnf, int flags)
@@ -594,6 +619,19 @@ bool FileInterner::dijontorcl(Rcl::Doc& doc)
     return true;
 }
 
+static void docfieldfrommeta(RclConfig* cfg, const string& name, 
+			     const string &value, Rcl::Doc& doc)
+{
+    string fieldname = cfg->fieldCanon(name);
+    LOGDEB0(("Internfile:: setting [%s] from cmd value [%s]\n",
+	     fieldname.c_str(), value.c_str()));
+    if (fieldname == cstr_dj_keymd) {
+	doc.dmtime = value;
+    } else {
+	doc.meta[fieldname] = value;
+    }
+}
+
 // Collect the ipath from the current path in the document tree.
 // While we're at it, we also set the mimetype and filename,
 // which are special properties: we want to get them from the topmost
@@ -615,16 +653,45 @@ void FileInterner::collectIpathAndMT(Rcl::Doc& doc) const
     bool hasipath = false;
 
 #ifdef RCL_USE_XATTR
-    // Set fields from extended file attributes.
-    // These can be later augmented by values from inside the file
-    for (map<string,string>::const_iterator it = m_XAttrsFields.begin(); 
-	 it != m_XAttrsFields.end(); it++) {
-	LOGDEB1(("Internfile:: setting [%s] from xattrs value [%s]\n",
-		 m_cfg->fieldCanon(it->first).c_str(), it->second.c_str()));
-	doc.meta[m_cfg->fieldCanon(it->first)] = it->second;
+    if (!m_noxattrs) {
+	// Set fields from extended file attributes.
+	// These can be later augmented by values from inside the file
+	for (map<string,string>::const_iterator it = m_XAttrsFields.begin(); 
+	     it != m_XAttrsFields.end(); it++) {
+	    LOGDEB1(("Internfile:: setting [%s] from xattrs value [%s]\n",
+		     m_cfg->fieldCanon(it->first).c_str(), it->second.c_str()));
+	    doc.meta[m_cfg->fieldCanon(it->first)] = it->second;
+	}
     }
 #endif //RCL_USE_XATTR
 
+    // Set fields from external commands
+    // These override those from xattrs and can be later augmented by
+    // values from inside the file.
+    //
+    // This is a bit atrocious because some entry names are special:
+    // "modificationdate" will set mtime instead of an ordinary field,
+    // and the output from anything beginning with "rclmulti" will be
+    // interpreted as multiple fields in configuration file format...
+    for (map<string,string>::const_iterator it = m_cmdFields.begin(); 
+	 it != m_cmdFields.end(); it++) {
+	if (!it->first.compare(0, 8, "rclmulti")) {
+	    ConfSimple simple(it->second);
+	    if (simple.ok()) {
+		vector<string> names = simple.getNames("");
+		for (vector<string>::const_iterator nm = names.begin(); 
+		     nm != names.end(); nm++) {
+		    string value;
+		    if (simple.get(*nm, value)) {
+			docfieldfrommeta(m_cfg, *nm, value, doc);
+		    }
+		}
+	    }
+	} else {
+	    docfieldfrommeta(m_cfg, it->first, it->second, doc);
+	}
+    }
+
     // If there is no ipath stack, the mimetype is the one from the file
     doc.mimetype = m_mimetype;
 
