@@ -18,6 +18,7 @@
 
 #include <iostream>
 #include <sstream>
+using namespace std;
 
 #include "cstr.h"
 #include "mh_execm.h"
@@ -33,10 +34,6 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#ifndef NO_NAMESPACES
-using namespace std;
-#endif /* NO_NAMESPACES */
-
 bool MimeHandlerExecMultiple::startCmd()
 {
     LOGDEB(("MimeHandlerExecMultiple::startCmd\n"));
@@ -50,13 +47,19 @@ bool MimeHandlerExecMultiple::startCmd()
     // Command name
     string cmd = params.front();
     
-    // Build parameter list: delete cmd name
-    list<string>::iterator it = params.begin();
-    list<string>myparams(++it, params.end());
+    m_maxmemberkb = 50000;
+    m_config->getConfParam("membermaxkbs", &m_maxmemberkb);
+    ostringstream oss;
+    oss << "RECOLL_FILTER_MAXMEMBERKB=" << m_maxmemberkb;
+    m_cmd.putenv(oss.str());
 
-    // Start filter
+    m_cmd.putenv("RECOLL_CONFDIR", m_config->getConfDir());
     m_cmd.putenv(m_forPreview ? "RECOLL_FILTER_FORPREVIEW=yes" :
 		"RECOLL_FILTER_FORPREVIEW=no");
+
+    // Build parameter list: delete cmd name
+    vector<string>myparams(params.begin() + 1, params.end());
+
     if (m_cmd.startExec(cmd, myparams, 1, 1) < 0) {
         m_reason = string("RECFILTERROR HELPERNOTFOUND ") + cmd;
         missingHelper = true;
@@ -115,8 +118,12 @@ bool MimeHandlerExecMultiple::readDataElement(string& name, string &data)
                 ibuf.c_str()));
         return false;
     }
-    LOGDEB1(("MHExecMultiple: got name [%s] len: %d\n", name.c_str(), len));
 
+    if (len / 1024 > m_maxmemberkb) {
+        LOGERR(("MHExecMultiple: data len > maxmemberkb\n"));
+        return false;
+    }
+    
     // Hack: check for 'Document:' and read directly the document data
     // to m_metaData[cstr_dj_keycontent] to avoid an extra copy of the bulky
     // piece
@@ -134,6 +141,9 @@ bool MimeHandlerExecMultiple::readDataElement(string& name, string &data)
                 len, datap->length()));
         return false;
     }
+    LOGDEB1(("MHExecMe:rdDtElt got: name [%s] len %d value [%s]\n", 
+	     name.c_str(), len, datap->size() > 100 ? 
+	     (datap->substr(0, 100) + " ...").c_str() : datap->c_str()));
     return true;
 }
 
@@ -243,11 +253,11 @@ bool MimeHandlerExecMultiple::next_document()
                 m_fn.c_str(), ipath.c_str()));
     }
 
-    // If this has an ipath, it is an internal doc from a
-    // multi-document file. In this case, either the filter supplies the 
-    // mimetype, or the ipath MUST be a filename-like string which we can use
-    // to compute a mime type
     if (!ipath.empty()) {
+	// If this has an ipath, it is an internal doc from a
+	// multi-document file. In this case, either the filter
+	// supplies the mimetype, or the ipath MUST be a filename-like
+	// string which we can use to compute a mime type
         m_metaData[cstr_dj_keyipath] = ipath;
         if (mtype.empty()) {
 	    LOGDEB0(("MHExecMultiple: no mime type from filter, "
@@ -266,43 +276,35 @@ bool MimeHandlerExecMultiple::next_document()
             }
         }
         m_metaData[cstr_dj_keymt] = mtype;
-        string md5, xmd5;
-        MD5String(m_metaData[cstr_dj_keycontent], md5);
-        m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
+	if (!m_forPreview) {
+	    string md5, xmd5;
+	    MD5String(m_metaData[cstr_dj_keycontent], md5);
+	    m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
+	}
     } else {
+	// "Self" document.
         m_metaData[cstr_dj_keymt] = mtype.empty() ? "text/html" : mtype;
         m_metaData.erase(cstr_dj_keyipath);
-        string md5, xmd5, reason;
-        if (MD5File(m_fn, md5, &reason)) {
-            m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
-        } else {
-            LOGERR(("MimeHandlerExecM: cant compute md5 for [%s]: %s\n",
-                    m_fn.c_str(), reason.c_str()));
-        }
-    }
-
-    // Charset. For many document types it doesn't matter. For text
-    // and html it does. We supply a default from the configuration. 
-    if (charset.empty()) {
-	charset = cfgFilterOutputCharset.empty() ? "utf-8" : 
-	    cfgFilterOutputCharset;
-	if (!stringlowercmp("default", charset)) {
-	    charset = m_dfltInputCharset;
+	if (!m_forPreview) {
+	    string md5, xmd5, reason;
+	    if (MD5File(m_fn, md5, &reason)) {
+		m_metaData[cstr_dj_keymd5] = MD5HexPrint(md5, xmd5);
+	    } else {
+		LOGERR(("MimeHandlerExecM: cant compute md5 for [%s]: %s\n",
+			m_fn.c_str(), reason.c_str()));
+	    }
 	}
     }
-    m_metaData[cstr_dj_keyorigcharset] = charset;
-    m_metaData[cstr_dj_keycharset] = charset;
 
-    if (!m_metaData[cstr_dj_keymt].compare(cstr_textplain)) {
-	(void)txtdcode("mh_execm");
-    }
-    
-    
+    handle_cs(m_metaData[cstr_dj_keymt], charset);
+
     if (eofnext_received)
         m_havedoc = false;
 
     LOGDEB0(("MHExecMultiple: returning %d bytes of content,"
-	    " mtype [%s] charset [%s]\n", m_metaData[cstr_dj_keycontent].size(), 
-     m_metaData[cstr_dj_keymt].c_str(), m_metaData[cstr_dj_keycharset].c_str()));
+	    " mtype [%s] charset [%s]\n", 
+	     m_metaData[cstr_dj_keycontent].size(), 
+	     m_metaData[cstr_dj_keymt].c_str(), 
+	     m_metaData[cstr_dj_keycharset].c_str()));
     return true;
 }
