@@ -41,6 +41,10 @@
 #include "ptmutex.h"
 
 using namespace std;
+
+// Define maximum message size for safety. 100MB would seem reasonable
+static const unsigned int max_mbox_member_size = 100 * 1024 * 1024;
+
 class FpKeeper { 
 public:
     FpKeeper(FILE **fpp) : m_fpp(fpp) {}
@@ -247,10 +251,10 @@ void MimeHandlerMbox::clear()
     RecollFilter::clear();
 }
 
-bool MimeHandlerMbox::set_document_file(const string &fn)
+bool MimeHandlerMbox::set_document_file(const string& mt, const string &fn)
 {
     LOGDEB(("MimeHandlerMbox::set_document_file(%s)\n", fn.c_str()));
-    RecollFilter::set_document_file(fn);
+    RecollFilter::set_document_file(mt, fn);
     m_fn = fn;
     if (m_vfp) {
 	fclose((FILE *)m_vfp);
@@ -279,6 +283,14 @@ bool MimeHandlerMbox::set_document_file(const string &fn)
 	}
     }
 
+    // And double check for thunderbird 
+    string tbirdmsf = fn + ".msf";
+    if ((m_quirks&MBOXQUIRK_TBIRD) == 0 && access(tbirdmsf.c_str(), 0) == 0) {
+	LOGDEB(("MimeHandlerMbox: detected unconfigured tbird mbox in %s\n",
+		fn.c_str()));
+	m_quirks |= MBOXQUIRK_TBIRD;
+    }
+
     return true;
 }
 
@@ -491,6 +503,12 @@ bool MimeHandlerMbox::next_document()
 	    line[ll] = '\n';
 	    line[ll+1] = 0;
 	    msgtxt += line;
+	    if (msgtxt.size() > max_mbox_member_size) {
+		LOGERR(("mh_mbox: huge message (more than %u MB) inside %s,"
+			" giving up\n", max_mbox_member_size/(1024*1024),
+			m_fn.c_str()));
+		return false;
+	    }
 	}
     }
     LOGDEB2(("Message text length %d\n", msgtxt.size()));
@@ -580,8 +598,8 @@ int main(int argc, char **argv)
 	exit(1);
     }
     config->setKeyDir(path_getfather(filename));
-    MimeHandlerMbox mh(config, "text/x-mail");
-    if (!mh.set_document_file(filename)) {
+    MimeHandlerMbox mh(config, "some_id");
+    if (!mh.set_document_file("text/x-mail", filename)) {
 	cerr << "set_document_file failed" << endl;
 	exit(1);
     }
