@@ -176,14 +176,15 @@ static NamedEntsInitializer namedEntsInitializerInstance;
 MyHtmlParser::MyHtmlParser()
     : in_script_tag(false),
       in_style_tag(false),
-      in_body_tag(false),
       in_pre_tag(false),
+      in_title_tag(false),
       pending_space(false),
       indexing_allowed(true)
 {
     // The default html document charset is iso-8859-1. We'll update
-    // this value from the encoding tag if found.
-    charset = "iso-8859-1";
+    // this value from the encoding tag if found. Actually use cp1252 which
+    // is a superset
+    charset = "CP1252";
 }
 
 void MyHtmlParser::decode_entities(string &s)
@@ -256,12 +257,20 @@ void MyHtmlParser::decode_entities(string &s)
 void
 MyHtmlParser::process_text(const string &text)
 {
-    LOGDEB2(("process_text: pending_space %d txt [%s]\n", pending_space,
-	    text.c_str()));
+    LOGDEB2(("process_text: title %d script %d style %d pre %d "
+	     "pending_space %d txt [%s]\n", 
+	     in_title_tag,
+	     in_script_tag,
+	     in_style_tag,
+	     in_pre_tag,
+	     pending_space,
+	     text.c_str()));
     CancelCheck::instance().checkCancel();
 
     if (!in_script_tag && !in_style_tag) {
-	if (!in_pre_tag) {
+	if (in_title_tag) {
+	    titledump += text;
+	} else if (!in_pre_tag) {
 	    string::size_type b = 0;
 	    bool only_space = true;
 	    while ((b = text.find_first_not_of(WHITESPACE, b)) != string::npos) {
@@ -308,11 +317,11 @@ MyHtmlParser::opening_tag(const string &tag)
 	    if (tag == "address") pending_space = true;
 	    break;
 	case 'b':
-	    if (tag == "body") {
-		dump.resize(0);
-		in_body_tag = true;
-		break;
-	    }
+	    // body: some bad docs have several opening body tags and
+	    // even text before the body is displayed by Opera and
+	    // Firefox.  We used to reset the dump each time we saw a
+	    // body tag, but I can't see any reason to do so.
+
 	    if (tag == "blockquote" || tag == "br") {
 		dump += '\n';
 		pending_space = true;
@@ -361,9 +370,7 @@ MyHtmlParser::opening_tag(const string &tag)
 		    if (get_parameter("name", name)) {
 			lowercase_term(name);
 			if (name == "date") {
-			    // Yes this doesnt exist. It's output by filters
-			    // And the format isn't even standard http/html
-			    // FIXME
+			    // Specific to Recoll filters.
 			    decode_entities(content);
 			    struct tm tm;
 			    if (strptime(content.c_str(), 
@@ -377,10 +384,22 @@ MyHtmlParser::opening_tag(const string &tag)
 			    }
 			} else if (name == "robots") {
 			} else {
+			    string markup;
+			    bool ishtml = false;
+			    if (get_parameter("markup", markup)) {
+				if (!stringlowercmp("html", markup)) {
+				    ishtml = true;
+				}
+			    }
 			    if (!meta[name].empty())
 				meta[name] += ' ';
 			    decode_entities(content);
 			    meta[name] += content;
+			    if (ishtml && 
+				meta[name].compare(0, cstr_fldhtm.size(), 
+						   cstr_fldhtm)) {
+				meta[name].insert(0, cstr_fldhtm);
+			    }
 			}
 		    } 
 		    string hdr;
@@ -393,7 +412,8 @@ MyHtmlParser::opening_tag(const string &tag)
 			    if ((k = p.params.find(cstr_html_charset)) != 
 				p.params.end()) {
 				charset = k->second;
-				if (!samecharset(charset, fromcharset)) {
+				if (!charset.empty() && 
+				    !samecharset(charset, fromcharset)) {
 				    LOGDEB1(("Doc http-equiv charset '%s' "
 					    "differs from dir deflt '%s'\n",
 					    charset.c_str(), 
@@ -409,7 +429,8 @@ MyHtmlParser::opening_tag(const string &tag)
 		    // HTML5 added: <meta charset="...">
 		    lowercase_term(newcharset);
 		    charset = newcharset;
-		    if (!samecharset(charset, fromcharset)) {
+		    if (!charset.empty() && 
+			!samecharset(charset, fromcharset)) {
 			LOGDEB1(("Doc html5 charset '%s' "
 				 "differs from dir deflt '%s'\n",
 				 charset.c_str(), 
@@ -418,8 +439,7 @@ MyHtmlParser::opening_tag(const string &tag)
 		    }
 		}
 		break;
-	    }
-	    if (tag == "marquee" || tag == "menu" || tag == "multicol")
+	    } else if (tag == "marquee" || tag == "menu" || tag == "multicol")
 		pending_space = true;
 	    break;
 	case 'o':
@@ -442,16 +462,19 @@ MyHtmlParser::opening_tag(const string &tag)
 	    if (tag == "style") {
 		in_style_tag = true;
 		break;
-	    }
-	    if (tag == "script") {
+	    } else if (tag == "script") {
 		in_script_tag = true;
 		break;
-	    }
-	    if (tag == "select") pending_space = true;
+	    } else if (tag == "select") 
+		pending_space = true;
 	    break;
 	case 't':
 	    if (tag == "table" || tag == "td" || tag == "textarea" ||
-		tag == "th") pending_space = true;
+		tag == "th") {
+		pending_space = true;
+	    } else if (tag == "title") {
+		in_title_tag = true;
+	    }
 	    break;
 	case 'u':
 	    if (tag == "ul") pending_space = true;
@@ -473,11 +496,11 @@ MyHtmlParser::closing_tag(const string &tag)
 	    if (tag == "address") pending_space = true;
 	    break;
 	case 'b':
-	    if (tag == "body") {
-		LOGDEB1(("Myhtmlparse: body close tag found\n"));
-		in_body_tag = false;
-		return false;
-	    }
+	    // body: We used to signal and end of doc here by returning
+	    // false but the browsers just ignore body and html
+	    // closing tags if there is further text, so it seems right
+	    // to do the same
+
 	    if (tag == "blockquote" || tag == "br") pending_space = true;
 	    break;
 	case 'c':
@@ -532,9 +555,10 @@ MyHtmlParser::closing_tag(const string &tag)
 	    break;
 	case 't':
 	    if (tag == "title") {
+		in_title_tag = false;
 		if (meta.find("title") == meta.end()|| meta["title"].empty()) {
-		    meta["title"] = dump;
-		    dump.clear();
+		    meta["title"] = titledump;
+		    titledump.clear();
 		}
 		break;
 	    }
@@ -562,6 +586,4 @@ MyHtmlParser::closing_tag(const string &tag)
 void
 MyHtmlParser::do_eof()
 {
-    //    if (!in_body_tag)
-    //	throw(false);
 }
