@@ -82,7 +82,6 @@ void ConfIndexW::acceptChanges()
     delete m_conf;
     m_conf = 0;
     m_rclconf->updateMainConfig();
-    snapshotConfig();
 
     hide();
 }
@@ -114,7 +113,11 @@ void ConfIndexW::reloadPanels()
 
     w = new ConfBeaglePanelW(this, m_conf);
     m_widgets.push_back(w);
-    tabWidget->addTab(w, QObject::tr("Beagle web history"));
+    tabWidget->addTab(w, QObject::tr("Web history"));
+
+    w = new ConfSearchPanelW(this, m_conf);
+    m_widgets.push_back(w);
+    tabWidget->addTab(w, QObject::tr("Search parameters"));
 }
 
 ConfBeaglePanelW::ConfBeaglePanelW(QWidget *parent, ConfNull *config)
@@ -124,12 +127,11 @@ ConfBeaglePanelW::ConfBeaglePanelW(QWidget *parent, ConfNull *config)
     vboxLayout->setSpacing(spacing);
     vboxLayout->setMargin(margin);
 
-    ConfLink lnk1(new ConfLinkRclRep(config, "processbeaglequeue"));
+    ConfLink lnk1(new ConfLinkRclRep(config, "processwebqueue"));
     ConfParamBoolW* cp1 = 
-        new ConfParamBoolW(this, lnk1, tr("Steal Beagle indexing queue"),
-                           tr("Beagle MUST NOT be running. Enables processing "
-                          "the beagle queue to index Firefox web history.<br>"
-                          "(you should also install the Firefox Beagle plugin)"
+        new ConfParamBoolW(this, lnk1, tr("Process the WEB history queue"),
+                           tr("Enables indexing Firefox visited pages.<br>"
+			     "(you need also install the Firefox Recoll plugin)"
 			  ));
     vboxLayout->addWidget(cp1);
 
@@ -155,6 +157,71 @@ ConfBeaglePanelW::ConfBeaglePanelW(QWidget *parent, ConfNull *config)
     vboxLayout->insertStretch(-1);
 }
 
+ConfSearchPanelW::ConfSearchPanelW(QWidget *parent, ConfNull *config)
+    : QWidget(parent)
+{
+    QVBoxLayout *vboxLayout = new QVBoxLayout(this);
+    vboxLayout->setSpacing(spacing);
+    vboxLayout->setMargin(margin);
+
+    if (!o_index_stripchars) {
+	ConfLink lnk1(new ConfLinkRclRep(config, "autodiacsens"));
+	ConfParamBoolW* cp1 = 
+	    new ConfParamBoolW(this, lnk1, tr("Automatic diacritics sensitivity"),
+			       tr("<p>Automatically trigger diacritics sensitivity "
+				  "if the search term has accented characters "
+				  "(not in unac_except_trans). Else you need to "
+				  "use the query language and the <i>D</i> "
+				  "modifier to specify "
+				  "diacritics sensitivity."
+				   ));
+	vboxLayout->addWidget(cp1);
+
+	ConfLink lnk2(new ConfLinkRclRep(config, "autocasesens"));
+	ConfParamBoolW* cp2 = 
+	    new ConfParamBoolW(this, lnk2, 
+			       tr("Automatic character case sensitivity"),
+			       tr("<p>Automatically trigger character case "
+				  "sensitivity if the entry has upper-case "
+				  "characters in any but the first position. "
+				  "Else you need to use the query language and "
+				  "the <i>C</i> modifier to specify character-case "
+				  "sensitivity."
+				   ));
+	vboxLayout->addWidget(cp2);
+    }
+
+    ConfLink lnk3(new ConfLinkRclRep(config, "maxTermExpand"));
+    ConfParamIntW* cp3 = 
+        new ConfParamIntW(this, lnk3, 
+			   tr("Maximum term expansion count"),
+			   tr("<p>Maximum expansion count for a single term "
+			      "(e.g.: when using wildcards). The default "
+			      "of 10 000 is reasonable and will avoid "
+			      "queries that appear frozen while the engine is "
+			      "walking the term list."
+			       ));
+    vboxLayout->addWidget(cp3);
+
+
+    ConfLink lnk4(new ConfLinkRclRep(config, "maxXapianClauses"));
+    ConfParamIntW* cp4 = 
+        new ConfParamIntW(this, lnk4, 
+			   tr("Maximum Xapian clauses count"),
+			   tr("<p>Maximum number of elementary clauses we "
+			      "add to a single Xapian query. In some cases, "
+			      "the result of term expansion can be "
+			      "multiplicative, and we want to avoid using "
+			      "excessive memory. The default of 100 000 "
+			      "should be both high enough in most cases "
+			      "and compatible with current typical hardware "
+			      "configurations."
+			       ));
+    vboxLayout->addWidget(cp4);
+
+    vboxLayout->insertStretch(-1);
+}
+
 ConfTopPanelW::ConfTopPanelW(QWidget *parent, ConfNull *config)
     : QWidget(parent)
 {
@@ -184,9 +251,9 @@ ConfTopPanelW::ConfTopPanelW(QWidget *parent, ConfNull *config)
     setSzPol(eskp, QSizePolicy::Preferred, QSizePolicy::Preferred, 1, 3);
     gl1->addWidget(eskp, 1, 0, 1, 2);
 
-    list<string> cstemlangs = Rcl::Db::getStemmerNames();
+    vector<string> cstemlangs = Rcl::Db::getStemmerNames();
     QStringList stemlangs;
-    for (list<string>::const_iterator it = cstemlangs.begin(); 
+    for (vector<string>::const_iterator it = cstemlangs.begin(); 
 	 it != cstemlangs.end(); it++) {
 	stemlangs.push_back(QString::fromUtf8(it->c_str()));
     }
@@ -231,12 +298,6 @@ ConfTopPanelW::ConfTopPanelW(QWidget *parent, ConfNull *config)
 			 "0 means no limit (this is the default)."), 0, 100);
     gl1->addWidget(w, 5, 0);
 
-    ConfLink lnkusfc(new ConfLinkRclRep(config, "usesystemfilecommand"));
-    w = new ConfParamBoolW(this, lnkusfc, tr("Use system's 'file' command"),
-		       tr("Use the system's 'file' command if internal<br>"
-			  "mime type identification fails."));
-    gl1->addWidget(w, 5, 1);
-    
     ConfLink lnknaspl(new ConfLinkRclRep(config, "noaspell"));
     ConfParamBoolW* cpasp =
     new ConfParamBoolW(this, lnknaspl, tr("No aspell usage"),
@@ -251,7 +312,7 @@ ConfTopPanelW::ConfTopPanelW(QWidget *parent, ConfNull *config)
 		      tr("The language for the aspell dictionary. "
 			 "This should look like 'en' or 'fr' ...<br>"
 			 "If this value is not set, the NLS environment "
-			 "will be used to compute it, which usually works."
+			 "will be used to compute it, which usually works. "
 			 "To get an idea of what is installed on your system, "
 			 "type 'aspell config' and look for .dat files inside "
 			 "the 'data-dir' directory. "));
@@ -264,9 +325,23 @@ ConfTopPanelW::ConfTopPanelW(QWidget *parent, ConfNull *config)
 	ConfParamFNW(this, lnkdbd, tr("Database directory name"),
 		     tr("The name for a directory where to store the index<br>"
 			"A non-absolute path is taken relative to the "
-			" configuration directory. The default is 'xapiandb'."
+			"configuration directory. The default is 'xapiandb'."
 			), true);
     gl1->addWidget(edbd, 7, 0, 1, 2);
+    
+    ConfLink lnkuexc(new ConfLinkRclRep(config, "unac_except_trans"));
+    ConfParamStrW *euexc = new 
+	ConfParamStrW(this, lnkuexc, tr("Unac exceptions"),
+		     tr("<p>These are exceptions to the unac mechanism "
+			"which, by default, removes all diacritics, "
+			"and performs canonic decomposition. You can override "
+			"unaccenting for some characters, depending on your "
+			"language, and specify additional decompositions, "
+			"e.g. for ligatures. In each space-separated entry, "
+			"the first character is the source one, and the rest "
+			"is the translation."
+			));
+    gl1->addWidget(euexc, 8, 0, 1, 2);
 }
 
 ConfSubPanelW::ConfSubPanelW(QWidget *parent, ConfNull *config)
@@ -296,10 +371,10 @@ ConfSubPanelW::ConfSubPanelW(QWidget *parent, ConfNull *config)
     // customized in the system config like .thunderbird or
     // .purple. This doesn't prevent them to add and customize them
     // further.
-    list<string> allkeydirs = config->getSubKeys(true); 
+    vector<string> allkeydirs = config->getSubKeys(true); 
 
     QStringList qls;
-    for (list<string>::const_iterator it = allkeydirs.begin(); 
+    for (vector<string>::const_iterator it = allkeydirs.begin(); 
 	 it != allkeydirs.end(); it++) {
 	qls.push_back(QString::fromUtf8(it->c_str()));
     }
@@ -340,7 +415,7 @@ ConfSubPanelW::ConfSubPanelW(QWidget *parent, ConfNull *config)
     m_widgets.push_back(eskn);
     gl1->addWidget(eskn, 0, 0, 1, 2);
 
-    list<string> args;
+    vector<string> args;
     args.push_back("-l");
     ExecCmd ex;
     string icout;
@@ -433,7 +508,7 @@ ConfSubPanelW::ConfSubPanelW(QWidget *parent, ConfNull *config)
 		      tr("Max. filter exec. time (S)"),
 		      tr("External filters working longer than this will be "
                          "aborted. This is for the rare case (ie: postscript) "
-                         "where a document could cause a filter to loop"
+                         "where a document could cause a filter to loop. "
 			 "Set to -1 for no limit.\n"),
 		      -1, 10000);
     m_widgets.push_back(efiltmaxsecs);
