@@ -25,36 +25,26 @@
 #include "guiutils.h"
 #include "pathut.h"
 #include "base64.h"
+#include "advshist.h"
 
 #include <QSettings>
 #include <QStringList>
 #include <QFileDialog>
 
 RclDynConf *g_dynconf;
+AdvSearchHist *g_advshistory;
+RclConfig *theconfig;
 
 // The table should not be necessary, but I found no css way to get
 // qt 4.6 qtextedit to clear the margins after the float img without 
 // introducing blank space.
 const char *PrefsPack::dfltResListFormat = 
-#if 1
-	"<table><tr><td><img src='%I' width='64'></td>"
-	"<td>%R %S %L &nbsp;&nbsp;<b>%T</b><br>"
-	"%M&nbsp;%D&nbsp;&nbsp;&nbsp;<i>%U</i>&nbsp;%i<br>"
+	"<table><tr><td><a href='%U'><img src='%I' width='64'></a></td>"
+	"<td>%L &nbsp;<i>%S</i> &nbsp;&nbsp;<b>%T</b><br>"
+	"<span style='white-space:nowrap'><i>%M</i>&nbsp;%D</span>&nbsp;&nbsp;&nbsp; <i>%U</i>&nbsp;%i<br>"
 	"%A %K</td></tr></table>"
-#elif 0
-	"<img src='%I' align='left'>"
-	"%R %S %L &nbsp;&nbsp;<b>%T</b><br>"
-	"%M&nbsp;%D&nbsp;&nbsp;&nbsp;<i>%U</i><br>"
-	"%A %K"
-#endif
     ;
 
-const char* v162reslistformat="<table><tr><td><img src='%I'></td>"
-    "<td>%R %S %L &nbsp;&nbsp;<b>%T</b><br>"
-    "%M&nbsp;%D&nbsp;&nbsp;&nbsp;<i>%U</i><br>"
-    "%A %K</td></tr></table>";
-
-
 // The global preferences structure
 PrefsPack prefs;
 
@@ -108,8 +98,12 @@ void rwSettings(bool writing)
 	}
     }
 
-    SETTING_RW(prefs.autoSearchOnWS, "/Recoll/prefs/reslist/autoSearchOnWS", 
+    SETTING_RW(prefs.ssearchOnWS, "/Recoll/prefs/reslist/autoSearchOnWS", 
 	       Bool, false);
+    SETTING_RW(prefs.ssearchNoComplete, 
+	       "/Recoll/prefs/ssearch/noComplete", Bool, false);
+    SETTING_RW(prefs.ssearchAsYouType, 
+	       "/Recoll/prefs/ssearch/asYouType", Bool, false);
     SETTING_RW(prefs.catgToolBar, "/Recoll/prefs/catgToolBar", Bool, false);
     SETTING_RW(prefs.ssearchAutoPhrase, 
 	       "/Recoll/prefs/ssearchAutoPhrase", Bool, true);
@@ -122,10 +116,8 @@ void rwSettings(bool writing)
 	       "/Recoll/prefs/showResultsAsTable", Bool, false);
     SETTING_RW(prefs.maxhltextmbs, "/Recoll/prefs/preview/maxhltextmbs", Int, 3);
 
-    // The default is true because I find it more often useful to keep
-    // indentation than to fold lines. Mileage may vary. 
     SETTING_RW(prefs.previewPlainPre, 
-	       "/Recoll/prefs/preview/plainPre", Bool, true);
+	       "/Recoll/prefs/preview/plainPre", Int, PrefsPack::PP_PREWRAP);
     SETTING_RW(prefs.qtermcolor, "/Recoll/prefs/qtermcolor", String, "blue");
     if (!writing && prefs.qtermcolor == "")
 	prefs.qtermcolor = "blue";
@@ -140,29 +132,33 @@ void rwSettings(bool writing)
 	prefs.reslistdateformat = "&nbsp;%Y-%m-%d&nbsp;%H:%M:%S&nbsp;%z";
     prefs.creslistdateformat = (const char*)prefs.reslistdateformat.toUtf8();
 
-    SETTING_RW(prefs.reslistfontfamily, "/Recoll/prefs/reslist/fontFamily", String,
-	       "");
+    SETTING_RW(prefs.reslistfontfamily, "/Recoll/prefs/reslist/fontFamily", 
+	       String, "");
     SETTING_RW(prefs.reslistfontsize, "/Recoll/prefs/reslist/fontSize", Int, 
 	       10);
+
     QString rlfDflt = QString::fromAscii(prefs.dfltResListFormat);
-    SETTING_RW(prefs.reslistformat, "/Recoll/prefs/reslist/format", String, rlfDflt);
-    prefs.creslistformat = (const char*)prefs.reslistformat.toUtf8();
-    if (!writing) {
-	// If the current value of the format is the default for the
-	// previous version, replace it with the new default. We
-	// should have a flag to say if it was changed instead
-	if (!prefs.creslistformat.compare(v162reslistformat)) {
-	    LOGDEB(("Replacing old default format\n"));
-	    prefs.reslistformat = rlfDflt;
-	    prefs.creslistformat = (const char*)prefs.reslistformat.toUtf8();
+    if (writing) {
+	if (prefs.reslistformat.compare(rlfDflt)) {
+	    settings.setValue("/Recoll/prefs/reslist/format", 
+			      prefs.reslistformat);
+	} else {
+	    settings.remove("/Recoll/prefs/reslist/format");
 	}
+    } else {
+	prefs.reslistformat = 
+	    settings.value("/Recoll/prefs/reslist/format", rlfDflt).toString();
+	prefs.creslistformat = qs2utf8s(prefs.reslistformat);
     }
-    SETTING_RW(prefs.reslistheadertext, "/Recoll/prefs/reslist/headertext", String, "");
-    SETTING_RW(prefs.stylesheetFile, "/Recoll/prefs/stylesheet", String, "");
+
+    SETTING_RW(prefs.reslistheadertext, "/Recoll/prefs/reslist/headertext", 
+	       String, "");
+    SETTING_RW(prefs.qssFile, "/Recoll/prefs/stylesheet", String, "");
+    SETTING_RW(prefs.snipCssFile, "/Recoll/prefs/snippets/cssfile", String, "");
     SETTING_RW(prefs.queryStemLang, "/Recoll/prefs/query/stemLang", String,
 	       "english");
-    SETTING_RW(prefs.useDesktopOpen, 
-	       "/Recoll/prefs/useDesktopOpen", Bool, true);
+    SETTING_RW(prefs.useDesktopOpen, "/Recoll/prefs/useDesktopOpen", 
+	       Bool, true);
 
     SETTING_RW(prefs.keepSort, 
 	       "/Recoll/prefs/keepSort", Bool, false);
@@ -194,8 +190,7 @@ void rwSettings(bool writing)
 	       Int, 0);
     // This is not really the current program version, just a value to
     // be used in case we have incompatible changes one day
-    SETTING_RW(prefs.rclVersion, "/Recoll/prefs/rclVersion", 
-	       Int, 1009);
+    SETTING_RW(prefs.rclVersion, "/Recoll/prefs/rclVersion", Int, 1009);
 
     // Ssearch combobox history list
     if (writing) {
@@ -295,10 +290,16 @@ void rwSettings(bool writing)
 			      prefs.allExtraDbs.end(), dbdir) != 
 		    prefs.allExtraDbs.end())
 		    continue;
-		if (!Rcl::Db::testDbDir(dbdir)) {
+		bool stripped;
+		if (!Rcl::Db::testDbDir(dbdir, &stripped)) {
 		    LOGERR(("Not a xapian index: [%s]\n", dbdir.c_str()));
 		    continue;
 		}
+		if (stripped != o_index_stripchars) {
+		    LOGERR(("Incompatible character stripping: [%s]\n",
+			    dbdir.c_str()));
+		    continue;
+		}
 		prefs.allExtraDbs.push_back(dbdir);
 	    }
 	}
@@ -310,8 +311,11 @@ void rwSettings(bool writing)
 	// actually there: useful for removable volumes.
 	for (list<string>::iterator it = prefs.activeExtraDbs.begin();
 	     it != prefs.activeExtraDbs.end();) {
-	    if (!Rcl::Db::testDbDir(*it)) {
-		LOGINFO(("Not a xapian index: [%s]\n", it->c_str()));
+	    bool stripped;
+	    if (!Rcl::Db::testDbDir(*it, &stripped) || 
+		stripped != o_index_stripchars) {
+		LOGINFO(("Not a Xapian index or char stripping differs: [%s]\n",
+			 it->c_str()));
 		it = prefs.activeExtraDbs.erase(it);
 	    } else {
 		it++;
@@ -332,8 +336,11 @@ void rwSettings(bool writing)
                               prefs.activeExtraDbs.end(), dbdir) !=
                     prefs.activeExtraDbs.end())
                     continue;
-                if (!Rcl::Db::testDbDir(dbdir)) {
-                    LOGERR(("Not a xapian index: [%s]\n", dbdir.c_str()));
+		bool strpd;
+                if (!Rcl::Db::testDbDir(dbdir, &strpd) || 
+		    strpd != o_index_stripchars) {
+                    LOGERR(("Not a Xapian dir or diff. char stripping: [%s]\n",
+			    dbdir.c_str()));
                     continue;
                 }
                 prefs.activeExtraDbs.push_back(dbdir);
@@ -374,6 +381,18 @@ void rwSettings(bool writing)
     }
 }
 
+string PrefsPack::stemlang()
+{
+    string stemLang = (const char *)prefs.queryStemLang.toAscii();
+    if (stemLang == "ALL") {
+	if (theconfig)
+	    theconfig->getConfParam("indexstemminglanguages", stemLang);
+	else
+	    stemLang = "";
+    }
+    return stemLang;
+}
+
 QString myGetFileName(bool isdir, QString caption, bool filenosave)
 {
     LOGDEB1(("myFileDialog: isdir %d\n", isdir));
