@@ -30,6 +30,7 @@
 #include <qcheckbox.h>
 #include <qcombobox.h>
 #include <QLocale>
+#include <QLibraryInfo>
 
 #include "rcldb.h"
 #include "rclconfig.h"
@@ -49,21 +50,13 @@
 
 #include "recollq.h"
 
-RclConfig *theconfig;
-RclConfig *thestableconfig;
-PTMutexInit thestableconfiglock;
+extern RclConfig *theconfig;
 
 // To avoid writing settings if we stopped before reading them (else
 // some kinds of errors would reset the qt/recoll settings to
 // defaults)
 static bool havereadsettings;
 
-void snapshotConfig()
-{
-    PTMutexLocker locker(thestableconfiglock);
-    thestableconfig = new RclConfig(*theconfig);
-}
-
 PTMutexInit thetempfileslock;
 static vector<TempFile>  o_tempfiles;
 /* Keep an array of temporary files for deletion at exit. It happens that we
@@ -107,6 +100,7 @@ void startManual(const string& helpindex)
 
 bool maybeOpenDb(string &reason, bool force, bool *maindberror)
 {
+    LOGDEB2(("maybeOpenDb: force %d\n", force));
     if (!rcldb) {
 	reason = "Internal error: db not created";
 	return false;
@@ -122,9 +116,10 @@ bool maybeOpenDb(string &reason, bool force, bool *maindberror)
     }
     Rcl::Db::OpenError error;
     if (!rcldb->isopen() && !rcldb->open(Rcl::Db::DbRO, &error)) {
-	reason = "Could not open database in " + 
-	    theconfig->getDbDir() + " wait for indexing to complete?";
+	reason = "Could not open database";
 	if (maindberror) {
+	    reason +=  " in " +  theconfig->getDbDir() + 
+		" wait for indexing to complete?";	    
 	    *maindberror = (error == Rcl::Db::DbOpenMainDb) ? true : false;
 	}
 	return false;
@@ -133,15 +128,29 @@ bool maybeOpenDb(string &reason, bool force, bool *maindberror)
     return true;
 }
 
-bool getStemLangs(list<string>& langs)
+// Retrieve the list currently active stemming languages. We try to
+// get this from the db, as some may have been added from recollindex
+// without changing the config. If this fails, use the config. This is
+// used for setting up choice menus, not updating the configuration.
+bool getStemLangs(vector<string>& vlangs)
 {
+    // Try from db
     string reason;
-    if (!maybeOpenDb(reason)) {
-	LOGERR(("getStemLangs: %s\n", reason.c_str()));
+    if (maybeOpenDb(reason)) {
+	vlangs = rcldb->getStemLangs();
+	LOGDEB0(("getStemLangs: from index: %s\n", 
+		 stringsToString(vlangs).c_str()));
+	return true;
+    } else {
+	// Cant get the langs from the index. Maybe it just does not
+	// exist yet. So get them from the config
+	string slangs;
+	if (theconfig->getConfParam("indexstemminglanguages", slangs)) {
+	    stringToStrings(slangs, vlangs);
+	    return true;
+	}
 	return false;
     }
-    langs = rcldb->getStemLangs();
-    return true;
 }
 
 static void recollCleanup()
@@ -153,7 +162,6 @@ static void recollCleanup()
     LOGDEB2(("recollCleanup: closing database\n"));
     deleteZ(rcldb);
     deleteZ(theconfig);
-//    deleteZ(thestableconfig);
 
     PTMutexLocker locker(thetempfileslock);
     o_tempfiles.clear();
@@ -296,12 +304,6 @@ int main(int argc, char **argv)
 	Usage();
 
 
-    // Translation file for Qt
-    QString slang = QLocale::system().name().left(2);
-    QTranslator qt(0);
-    qt.load(QString("qt_") + slang, "." );
-    app.installTranslator( &qt );
-
     string reason;
     theconfig = recollinit(recollCleanup, sigcleanup, reason, &a_config);
     if (!theconfig || !theconfig->ok()) {
@@ -310,9 +312,15 @@ int main(int argc, char **argv)
 	QMessageBox::critical(0, "Recoll",  msg);
 	exit(1);
     }
-    snapshotConfig();
     //    fprintf(stderr, "recollinit done\n");
 
+    // Translations for Qt standard widgets
+    QString slang = QLocale::system().name().left(2);
+    QTranslator qt_trans(0);
+    qt_trans.load(QString("qt_%1").arg(slang), 
+		  QLibraryInfo::location(QLibraryInfo::TranslationsPath));
+    app.installTranslator(&qt_trans);
+
     // Translations for Recoll
     string translatdir = path_cat(theconfig->getDatadir(), "translations");
     QTranslator translator(0);
@@ -337,15 +345,19 @@ int main(int argc, char **argv)
 	QMessageBox::critical(0, "Recoll",  msg);
 	exit(1);
     }
+    g_advshistory = new AdvSearchHist;
 
     //    fprintf(stderr, "History done\n");
     rwSettings(false);
     havereadsettings = true;
     //    fprintf(stderr, "Settings done\n");
 
-    if (!prefs.stylesheetFile.isEmpty()) {
-	applyStyleSheet(prefs.stylesheetFile);
+    if (!prefs.qssFile.isEmpty()) {
+	applyStyleSheet(prefs.qssFile);
     }
+    QIcon icon;
+    icon.addFile(QString::fromUtf8(":/images/recoll.png"));
+    app.setWindowIcon(icon);
 
     // Create main window and set its size to previous session's
     RclMain w;
