@@ -70,6 +70,7 @@ using std::pair;
 #include "internfile.h"
 #include "docseqdb.h"
 #include "docseqhist.h"
+#include "docseqdocs.h"
 #include "confguiindex.h"
 #include "restable.h"
 #include "listdialog.h"
@@ -80,6 +81,7 @@ using std::pair;
 #include "indexer.h"
 #include "rclzg.h"
 #include "fileudi.h"
+#include "snippets_w.h"
 
 using namespace confgui;
 
@@ -87,9 +89,9 @@ using namespace confgui;
 #include "rclhelp.h"
 #include "moc_rclmain_w.cpp"
 
-extern "C" int XFlush(void *);
 QString g_stringAllStem, g_stringNoStem;
 static const QKeySequence quitKeySeq("Ctrl+q");
+static const QKeySequence closeKeySeq("Ctrl+w");
 
 void RclMain::init()
 {
@@ -135,17 +137,14 @@ void RclMain::init()
     // db not open yet (the case where it does not even exist makes
     // things complicated). So get the languages from the config
     // instead
-    string slangs;
-    list<string> langs;
-    if (theconfig->getConfParam("indexstemminglanguages", slangs)) {
-	stringToStrings(slangs, langs);
-    } else {
+    vector<string> langs;
+    if (!getStemLangs(langs)) {
 	QMessageBox::warning(0, "Recoll", 
 			     tr("error retrieving stemming languages"));
     }
     QAction *curid = prefs.queryStemLang == "ALL" ? m_idAllStem : m_idNoStem;
     QAction *id; 
-    for (list<string>::const_iterator it = langs.begin(); 
+    for (vector<string>::const_iterator it = langs.begin(); 
 	 it != langs.end(); it++) {
 	QString qlang = QString::fromAscii(it->c_str(), it->length());
 	id = preferencesMenu->addAction(qlang);
@@ -178,12 +177,12 @@ void RclMain::init()
     bgrp->addButton(allRDB, bgrpid++);
     connect(bgrp, SIGNAL(buttonClicked(int)), this, SLOT(catgFilter(int)));
     allRDB->setChecked(true);
-    list<string> cats;
+    vector<string> cats;
     theconfig->getGuiFilterNames(cats);
     // Text for button 0 is not used. Next statement just avoids unused
     // variable compiler warning for catg_strings
     m_catgbutvec.push_back(catg_strings[0]);
-    for (list<string>::const_iterator it = cats.begin();
+    for (vector<string>::const_iterator it = cats.begin();
 	 it != cats.end(); it++) {
 	QRadioButton *but = new QRadioButton(catgBGRP);
 	QString catgnm = QString::fromUtf8(it->c_str(), it->length());
@@ -221,13 +220,12 @@ void RclMain::init()
     // A shortcut to get the focus back to the search entry. 
     QKeySequence seq("Ctrl+Shift+s");
     QShortcut *sc = new QShortcut(seq, this);
-    connect(sc, SIGNAL (activated()), 
-	    this, SLOT (focusToSearch()));
+    connect(sc, SIGNAL (activated()), sSearch, SLOT (takeFocus()));
 
     connect(&m_watcher, SIGNAL(fileChanged(QString)), 
 	    this, SLOT(idxStatus()));
-    connect(sSearch, SIGNAL(startSearch(RefCntr<Rcl::SearchData>)), 
-	    this, SLOT(startSearch(RefCntr<Rcl::SearchData>)));
+    connect(sSearch, SIGNAL(startSearch(RefCntr<Rcl::SearchData>, bool)), 
+	    this, SLOT(startSearch(RefCntr<Rcl::SearchData>, bool)));
     connect(sSearch, SIGNAL(clearSearch()), 
 	    this, SLOT(resetSearch()));
 
@@ -274,9 +272,13 @@ void RclMain::init()
     connect(toggleFullScreenAction, SIGNAL(activated()), 
             this, SLOT(toggleFullScreen()));
     connect(actionShowQueryDetails, SIGNAL(activated()),
-	    this, SLOT(showQueryDetails()));
+	    reslist, SLOT(showQueryDetails()));
     connect(periodictimer, SIGNAL(timeout()), 
 	    this, SLOT(periodic100()));
+
+    restable->setRclMain(this, true);
+    connect(actionSaveResultsAsCSV, SIGNAL(activated()), 
+	    restable, SLOT(saveAsCSV()));
     connect(this, SIGNAL(docSourceChanged(RefCntr<DocSequence>)),
 	    restable, SLOT(setDocSource(RefCntr<DocSequence>)));
     connect(this, SIGNAL(searchReset()), 
@@ -288,19 +290,23 @@ void RclMain::init()
 
     connect(restable->getModel(), SIGNAL(sortDataChanged(DocSeqSortSpec)),
 	    this, SLOT(onSortDataChanged(DocSeqSortSpec)));
-    connect(restable, SIGNAL(docEditClicked(Rcl::Doc)), 
-	    this, SLOT(startNativeViewer(Rcl::Doc)));
+
     connect(restable, SIGNAL(docPreviewClicked(int, Rcl::Doc, int)), 
 	    this, SLOT(startPreview(int, Rcl::Doc, int)));
     connect(restable, SIGNAL(docExpand(Rcl::Doc)), 
 	    this, SLOT(docExpand(Rcl::Doc)));
+    connect(restable, SIGNAL(showSubDocs(Rcl::Doc)), 
+	    this, SLOT(showSubDocs(Rcl::Doc)));
     connect(restable, SIGNAL(previewRequested(Rcl::Doc)), 
 	    this, SLOT(startPreview(Rcl::Doc)));
     connect(restable, SIGNAL(editRequested(Rcl::Doc)), 
 	    this, SLOT(startNativeViewer(Rcl::Doc)));
     connect(restable, SIGNAL(docSaveToFileClicked(Rcl::Doc)), 
 	    this, SLOT(saveDocToFile(Rcl::Doc)));
+    connect(restable, SIGNAL(showSnippets(Rcl::Doc)), 
+	    this, SLOT(showSnippets(Rcl::Doc)));
 
+    reslist->setRclMain(this, true);
     connect(this, SIGNAL(docSourceChanged(RefCntr<DocSequence>)),
 	    reslist, SLOT(setDocSource(RefCntr<DocSequence>)));
     connect(firstPageAction, SIGNAL(activated()), 
@@ -316,8 +322,6 @@ void RclMain::init()
 
     connect(reslist, SIGNAL(hasResults(int)), 
 	    this, SLOT(resultCount(int)));
-    connect(reslist, SIGNAL(docExpand(Rcl::Doc)), 
-	    this, SLOT(docExpand(Rcl::Doc)));
     connect(reslist, SIGNAL(wordSelect(QString)),
 	    sSearch, SLOT(addTerm(QString)));
     connect(reslist, SIGNAL(wordReplace(const QString&, const QString&)),
@@ -326,8 +330,13 @@ void RclMain::init()
 	    this, SLOT(enableNextPage(bool)));
     connect(reslist, SIGNAL(prevPageAvailable(bool)), 
 	    this, SLOT(enablePrevPage(bool)));
-    connect(reslist, SIGNAL(docEditClicked(Rcl::Doc)), 
-	    this, SLOT(startNativeViewer(Rcl::Doc)));
+
+    connect(reslist, SIGNAL(docExpand(Rcl::Doc)), 
+	    this, SLOT(docExpand(Rcl::Doc)));
+    connect(reslist, SIGNAL(showSnippets(Rcl::Doc)), 
+	    this, SLOT(showSnippets(Rcl::Doc)));
+    connect(reslist, SIGNAL(showSubDocs(Rcl::Doc)), 
+	    this, SLOT(showSubDocs(Rcl::Doc)));
     connect(reslist, SIGNAL(docSaveToFileClicked(Rcl::Doc)), 
 	    this, SLOT(saveDocToFile(Rcl::Doc)));
     connect(reslist, SIGNAL(editRequested(Rcl::Doc)), 
@@ -336,8 +345,6 @@ void RclMain::init()
 	    this, SLOT(startPreview(int, Rcl::Doc, int)));
     connect(reslist, SIGNAL(previewRequested(Rcl::Doc)), 
 	    this, SLOT(startPreview(Rcl::Doc)));
-    connect(reslist, SIGNAL(headerClicked()), 
-	    this, SLOT(showQueryDetails()));
 
     if (prefs.keepSort && prefs.sortActive) {
 	m_sortspec.field = (const char *)prefs.sortField.toUtf8();
@@ -397,7 +404,7 @@ void RclMain::initDbOpen()
 }
 
 // Start native viewer or preview for input Doc. This is used allow
-// the using Recoll result docs with an ipath in another app.  We act
+// using Recoll result docs with an ipath in another app.  We act
 // as a proxy to extract the data and start a viewer.
 // The Url are encoded as file://path#ipath
 void RclMain::viewUrl()
@@ -420,14 +427,24 @@ void RclMain::viewUrl()
 	     (const char *)qurl.fragment().toLocal8Bit(), udi);
     
     Rcl::Doc doc;
-    if (!rcldb->getDoc(udi, doc) || doc.pc == -1)
+    Rcl::Doc idxdoc; // idxdoc.idxi == 0 -> works with base index only
+    if (!rcldb->getDoc(udi, idxdoc, doc) || doc.pc == -1)
 	return;
 
+    // StartNativeViewer needs a db source to call getEnclosing() on.
+    Rcl::Query *query = new Rcl::Query(rcldb);
+    DocSequenceDb *src = 
+	new DocSequenceDb(RefCntr<Rcl::Query>(query), "", 
+			  RefCntr<Rcl::SearchData>(new Rcl::SearchData));
+    m_source = RefCntr<DocSequence>(src);
+
+
     // Start a native viewer if the mimetype has one defined, else a
     // preview.
     string apptag;
     doc.getmeta(Rcl::Doc::keyapptg, &apptag);
-    string viewer = theconfig->getMimeViewerDef(doc.mimetype, apptag);
+    string viewer = theconfig->getMimeViewerDef(doc.mimetype, apptag, 
+						prefs.useDesktopOpen);
     if (viewer.empty()) {
 	startPreview(doc);
     } else {
@@ -441,17 +458,11 @@ void RclMain::viewUrl()
 	// prevent the temp file deletion completely, leaving it
 	// around forever. Better to let the user save a copy if he
 	// wants I think.
-	sleep(10);
+	sleep(30);
 	fileExit();
     }
 }
 
-void RclMain::focusToSearch()
-{
-    LOGDEB(("Giving focus to sSearch\n"));
-    sSearch->queryText->setFocus(Qt::ShortcutFocusReason);
-}
-
 void RclMain::setStemLang(QAction *id)
 {
     LOGDEB(("RclMain::setStemLang(%p)\n", id));
@@ -611,6 +622,10 @@ void RclMain::periodic100()
 	    if (status) {
 		QMessageBox::warning(0, "Recoll", 
 				     tr("Indexing failed"));
+	    } else {
+		// On the first run, show missing helpers. We only do this once
+		if (m_firstIndexing)
+		    showMissingHelpers();
 	    }
 	    string reason;
 	    maybeOpenDb(reason, 1);
@@ -703,7 +718,13 @@ void RclMain::toggleIndexing()
     break;
     case IXST_NOTRUNNING:
     {
-	list<string> args;
+	// Could also mean that no helpers are missing, but then we
+	// won't try to show a message anyway (which is what
+	// firstIndexing is used for)
+	string mhd;
+	m_firstIndexing = !theconfig->getMissingHelperDesc(mhd);
+
+	vector<string> args;
 	args.push_back("-c");
 	args.push_back(theconfig->getConfDir());
 	m_idxproc = new ExecCmd;
@@ -729,7 +750,12 @@ void RclMain::rebuildIndex()
 					 QMessageBox::Cancel,
 					 QMessageBox::NoButton);
 	if (rep == QMessageBox::Ok) {
-	    list<string> args;
+	    // Could also mean that no helpers are missing, but then we
+	    // won't try to show a message anyway (which is what
+	    // firstIndexing is used for)
+	    string mhd;
+	    m_firstIndexing = !theconfig->getMissingHelperDesc(mhd);
+	    vector<string> args;
 	    args.push_back("-c");
 	    args.push_back(theconfig->getConfDir());
 	    args.push_back("-z");
@@ -742,26 +768,28 @@ void RclMain::rebuildIndex()
 }
 
 // Start a db query and set the reslist docsource
-void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata)
+void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata, bool issimple)
 {
-    LOGDEB(("RclMain::startSearch. Indexing %s\n", m_idxproc?"on":"off"));
-    emit searchReset();
+    LOGDEB(("RclMain::startSearch. Indexing %s Active %d\n", 
+	    m_idxproc?"on":"off", m_queryActive));
+    if (m_queryActive) {
+	LOGDEB(("startSearch: already active\n"));
+	return;
+    }
+    m_queryActive = true;
     m_source = RefCntr<DocSequence>();
 
+    m_searchIsSimple = issimple;
+
     // The db may have been closed at the end of indexing
     string reason;
     // If indexing is being performed, we reopen the db at each query.
     if (!maybeOpenDb(reason, m_idxproc != 0)) {
 	QMessageBox::critical(0, "Recoll", QString(reason.c_str()));
+	m_queryActive = false;
 	return;
     }
 
-    string stemLang = (const char *)prefs.queryStemLang.toAscii();
-    if (stemLang == "ALL") {
-	theconfig->getConfParam("indexstemminglanguages", stemLang);
-    }
-    sdata->setStemlang(stemLang);
-
     Rcl::Query *query = new Rcl::Query(rcldb);
     query->setCollapseDuplicates(prefs.collapseDuplicates);
 
@@ -800,9 +828,8 @@ class QueryThread : public QThread {
 
 void RclMain::initiateQuery()
 {
-    if (m_queryActive || m_source.isNull())
+    if (m_source.isNull())
 	return;
-    m_queryActive = true;
 
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
     QueryThread qthr(m_source);
@@ -829,7 +856,8 @@ void RclMain::initiateQuery()
 	    progress.show();
 	qApp->processEvents();
 	if (progress.wasCanceled()) {
-	    fileExit();
+	    // Just get out of there asap. 
+	    _exit(1);
 	}
 
 	qApp->processEvents();
@@ -864,8 +892,9 @@ void RclMain::showAdvSearchDialog()
 	connect(new QShortcut(quitKeySeq, asearchform), SIGNAL (activated()), 
 		this, SLOT (fileExit()));
 
-	connect(asearchform, SIGNAL(startSearch(RefCntr<Rcl::SearchData>)), 
-		this, SLOT(startSearch(RefCntr<Rcl::SearchData>)));
+	connect(asearchform, 
+		SIGNAL(startSearch(RefCntr<Rcl::SearchData>, bool)), 
+		this, SLOT(startSearch(RefCntr<Rcl::SearchData>, bool)));
 	asearchform->show();
     } else {
 	// Close and reopen, in hope that makes us visible...
@@ -937,8 +966,12 @@ void RclMain::showIndexSched(bool modal)
 	connect(indexSched->cronCLB, SIGNAL(clicked()), 
 		this, SLOT(execCronTool()));
 	if (theconfig && theconfig->isDefaultConfig()) {
+#ifdef RCL_MONITOR
 	    connect(indexSched->rtidxCLB, SIGNAL(clicked()), 
 		    this, SLOT(execRTITool()));
+#else
+	    indexSched->rtidxCLB->setEnabled(false);
+#endif
 	} else {
 	    indexSched->rtidxCLB->setEnabled(false);
 	}
@@ -1049,14 +1082,23 @@ void RclMain::showAboutDialog()
 
 void RclMain::showMissingHelpers()
 {
-    string miss = theconfig->getMissingHelperDesc();
-    QString msg = tr("External applications/commands needed and not found "
-		     "for indexing your file types:\n\n");
+    string miss;
+    if (!theconfig->getMissingHelperDesc(miss)) {
+	QMessageBox::information(this, "", tr("Indexing did not run yet"));
+	return;
+    }
+    QString msg = QString::fromAscii("<p>") +
+	tr("External applications/commands needed for your file types "
+	   "and not found, as stored by the last indexing pass in ");
+    msg += "<i>";
+    msg += QString::fromLocal8Bit(theconfig->getConfDir().c_str());
+    msg += "/missing</i>:<pre>\n";
     if (!miss.empty()) {
 	msg += QString::fromUtf8(miss.c_str());
     } else {
 	msg += tr("No helpers found missing");
     }
+    msg += "</pre>";
     QMessageBox::information(this, tr("Missing helper programs"), msg);
 }
 
@@ -1070,43 +1112,35 @@ void RclMain::showActiveTypes()
 	return;
     }
 
-    // Get list of all mime types in index. For this, we use a
-    // wildcard field search on mtype
-    Rcl::TermMatchResult matches;
-    string prefix;
-    if (!rcldb->termMatch(Rcl::Db::ET_WILD, "", "*", matches, -1, "mtype", 
-			  &prefix)) {
+    // All mime types in index. 
+    vector<string> vdbtypes;
+    if (!rcldb->getAllDbMimeTypes(vdbtypes)) {
 	QMessageBox::warning(0, tr("Error"), 
 			     tr("Index query error"),
 			     QMessageBox::Ok, 
 			     QMessageBox::NoButton);
 	return;
     }
-
-    // Build the set of mtypes, stripping the prefix
     set<string> mtypesfromdb;
-    for (list<Rcl::TermMatchEntry>::const_iterator it = matches.entries.begin(); 
-	 it != matches.entries.end(); it++) {
-	mtypesfromdb.insert(it->term.substr(prefix.size()));
-    }
+    mtypesfromdb.insert(vdbtypes.begin(), vdbtypes.end());
 
     // All types listed in mimeconf:
-    list<string> mtypesfromconfig = theconfig->getAllMimeTypes();
+    vector<string> mtypesfromconfig = theconfig->getAllMimeTypes();
 
     // Intersect file system types with config types (those not in the
     // config can be indexed by name, not by content)
     set<string> mtypesfromdbconf;
-    for (list<string>::const_iterator it = mtypesfromconfig.begin();
+    for (vector<string>::const_iterator it = mtypesfromconfig.begin();
 	 it != mtypesfromconfig.end(); it++) {
 	if (mtypesfromdb.find(*it) != mtypesfromdb.end())
 	    mtypesfromdbconf.insert(*it);
     }
 
-    // Substract the types for missing helpers (the docs are indexed by name only):
-    string miss = theconfig->getMissingHelperDesc();
-    if (!miss.empty()) {
-	FIMissingStore st;
-	FileInterner::getMissingFromDescription(&st, miss);
+    // Substract the types for missing helpers (the docs are indexed
+    // by name only):
+    string miss;
+    if (theconfig->getMissingHelperDesc(miss) && !miss.empty()) {
+	FIMissingStore st(miss);
 	map<string, set<string> >::const_iterator it;
 	for (it = st.m_typesForMissing.begin(); 
 	     it != st.m_typesForMissing.end(); it++) {
@@ -1172,38 +1206,62 @@ void RclMain::startPreview(int docnum, Rcl::Doc doc, int mod)
     // from a compacted mail folder)
     //
     // !! NOTE: there is one case where doing a partial index update
-    // will not worl: if the search result does not exist in the new
+    // will not work: if the search result does not exist in the new
     // version of the file, it won't be purged from the index because
     // a partial index pass does no purge, so its ref date will stay
     // the same and you keep getting the message about the index being
     // out of date. The only way to fix this is to run a normal
-    // indexing pass. 
+    // indexing pass (common case: the mbox was shortened and the
+    // result msgnum is beyond the new end)
     // Also we should re-run the query after updating the index
-    // because the ipaths may be wrong in the current result list
-    if (!doc.ipath.empty()) {
-	string udi, sig;
+    // because the ipaths may be wrong in the current result list We
+    // only do this for the main index, else jump and prey (cant
+    // update anyway, even the makesig() call might not make sense for
+    // our base config)
+    if (!doc.ipath.empty() && rcldb && rcldb->whatDbIdx(doc) == 0) {
+	string udi;
 	doc.getmeta(Rcl::Doc::keyudi, &udi);
-	FileInterner::makesig(doc, sig);
-	if (rcldb && !udi.empty()) {
+	if (!udi.empty()) {
+	    string sig;
+	    if (!FileInterner::makesig(theconfig, doc, sig)) {
+		QMessageBox::warning(0, "Recoll", 
+				     tr("Can't access file: ") + 
+				     QString::fromLocal8Bit(doc.url.c_str()));
+		return;
+	    }
 	    if (rcldb->needUpdate(udi, sig)) {
+		QString msg = 
+		    tr("Index not up to date for this file. "
+		       "Refusing to risk showing the wrong entry. ");
+		bool ixnotact = (m_indexerState == IXST_NOTRUNNING);
+		if (ixnotact) {
+		    msg += tr("Click Ok to update the "
+			      "index for this file, then you will need to "
+			      "re-run the query when indexing is done. ");
+		} else {
+		    msg += tr("The indexer is running so things should "
+			      "improve when it's done. ");
+		} 
+		msg += tr("Click Cancel to return to the list. "
+			  "Click Ignore to show the preview anyway. ");
+		QMessageBox::StandardButtons bts = 
+		    QMessageBox::Ignore | QMessageBox::Cancel;
+		if (ixnotact)
+		    bts |= QMessageBox::Ok;
 		int rep = 
 		    QMessageBox::warning(0, tr("Warning"), 
-				       tr("Index not up to date for this file. "
-					"Refusing to risk showing the wrong "
-					"entry. Click Ok to update the "
-					"index for this file, then re-run the "
-					"query when indexing is done. "
-					"Else, Cancel."),
-					 QMessageBox::Ok,
-					 QMessageBox::Cancel,
+					 msg, bts,
+					 ixnotact? QMessageBox::Cancel : 
 					 QMessageBox::NoButton);
-		if (rep == QMessageBox::Ok) {
+		if (m_indexerState == IXST_NOTRUNNING && 
+		    rep == QMessageBox::Ok) {
 		    LOGDEB(("Requesting index update for %s\n", 
 			    doc.url.c_str()));
 		    vector<Rcl::Doc> docs(1, doc);
 		    updateIdxForDocs(docs);
 		}
-		return;
+		if (rep != QMessageBox::Ignore)
+		    return;
 	    }
 	}
     }
@@ -1215,8 +1273,8 @@ void RclMain::startPreview(int docnum, Rcl::Doc doc, int mod)
 	curPreview = 0;
     }
     if (curPreview == 0) {
-	HiliteData hdata;
-	m_source->getTerms(hdata.terms, hdata.groups, hdata.gslks);
+	HighlightData hdata;
+	m_source->getTerms(hdata);
 	curPreview = new Preview(reslist->listId(), hdata);
 
 	if (curPreview == 0) {
@@ -1258,7 +1316,7 @@ void RclMain::updateIdxForDocs(vector<Rcl::Doc>& docs)
 	
     vector<string> paths;
     if (ConfIndexer::docsToPaths(docs, paths)) {
-	list<string> args;
+	vector<string> args;
 	args.push_back("-c");
 	args.push_back(theconfig->getConfDir());
 	args.push_back("-i");
@@ -1279,7 +1337,7 @@ void RclMain::updateIdxForDocs(vector<Rcl::Doc>& docs)
  */
 void RclMain::startPreview(Rcl::Doc doc)
 {
-    Preview *preview = new Preview(0, HiliteData());
+    Preview *preview = new Preview(0, HighlightData());
     if (preview == 0) {
 	QMessageBox::warning(0, tr("Warning"), 
 			     tr("Can't create preview window"),
@@ -1411,15 +1469,15 @@ void RclMain::on_actionShowResultsAsTable_toggled(bool on)
 {
     LOGDEB(("RclMain::on_actionShowResultsAsTable_toggled(%d)\n", int(on)));
     prefs.showResultsAsTable = on;
+    displayingTable = on;
     restable->setVisible(on);
     reslist->setVisible(!on);
+    actionSaveResultsAsCSV->setEnabled(on);
     if (!on) {
-	displayingTable = false;
 	int docnum = restable->getDetailDocNumOrTopRow();
 	if (docnum >= 0)
 	    reslist->resultPageFor(docnum);
     } else {
-	displayingTable = true;
 	int docnum = reslist->pageFirstDocNum();
 	if (docnum >= 0) {
 	    restable->makeRowVisible(docnum);
@@ -1496,18 +1554,59 @@ static bool lookForHtmlBrowser(string &exefile)
     return false;
 }
 
-void RclMain::startNativeViewer(Rcl::Doc doc)
+void RclMain::showSnippets(Rcl::Doc doc)
 {
-    // Look for appropriate viewer
-    string cmdplusattr;
-    if (prefs.useDesktopOpen) {
-	cmdplusattr = theconfig->getMimeViewerDef("application/x-all", "");
-    } else {
-        string apptag;
-        doc.getmeta(Rcl::Doc::keyapptg, &apptag);
-	cmdplusattr = theconfig->getMimeViewerDef(doc.mimetype, apptag);
-    }
+    SnippetsW *sp = new SnippetsW(doc, m_source);
+    connect(sp, SIGNAL(startNativeViewer(Rcl::Doc, int, QString)),
+	    this, SLOT(startNativeViewer(Rcl::Doc, int, QString)));
+    connect(new QShortcut(quitKeySeq, sp), SIGNAL (activated()), 
+	    this, SLOT (fileExit()));
+    connect(new QShortcut(closeKeySeq, sp), SIGNAL (activated()), 
+	    sp, SLOT (close()));
+    sp->show();
+}
 
+void RclMain::showSubDocs(Rcl::Doc doc)
+{
+    LOGDEB(("RclMain::showSubDocs\n"));
+    string reason;
+    if (!maybeOpenDb(reason)) {
+	QMessageBox::critical(0, "Recoll", QString(reason.c_str()));
+	return;
+    }
+    vector<Rcl::Doc> docs;
+    if (!rcldb->getSubDocs(doc, docs)) {
+	QMessageBox::warning(0, "Recoll", QString("Can't get subdocs"));
+	return;
+    }	
+    DocSequenceDocs *src = 
+	new DocSequenceDocs(rcldb, docs,
+			    qs2utf8s(tr("Sub-documents and attachments")));
+    src->setDescription(qs2utf8s(tr("Sub-documents and attachments")));
+    RefCntr<DocSequence> 
+	source(new DocSource(theconfig, RefCntr<DocSequence>(src)));
+
+    ResTable *res = new ResTable();
+//    ResList *res = new ResList();
+    res->setRclMain(this, false);
+    res->setDocSource(source);
+    res->readDocSource();
+    res->show();
+}
+
+void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString term)
+{
+    string apptag;
+    doc.getmeta(Rcl::Doc::keyapptg, &apptag);
+    LOGDEB(("RclMain::startNativeViewer: mtype [%s] apptag [%s] page %d "
+	    "term [%s] url [%s] ipath [%s]\n", 
+	    doc.mimetype.c_str(), apptag.c_str(), pagenum, 
+	    (const char *)(term.toUtf8()), doc.url.c_str(), doc.ipath.c_str()
+	       ));
+
+    // Look for appropriate viewer
+    string cmdplusattr = theconfig->getMimeViewerDef(doc.mimetype, apptag, 
+						     prefs.useDesktopOpen);
     if (cmdplusattr.empty()) {
 	QMessageBox::warning(0, "Recoll", 
 			     tr("No external viewer configured for mime type [")
@@ -1515,20 +1614,20 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 	return;
     }
 
-    // Extract possible viewer attributes
-    ConfSimple attrs;
+    // Separate command string and viewer attributes (if any)
+    ConfSimple viewerattrs;
     string cmd;
-    theconfig->valueSplitAttributes(cmdplusattr, cmd, attrs);
+    theconfig->valueSplitAttributes(cmdplusattr, cmd, viewerattrs);
     bool ignoreipath = false;
-    if (attrs.get("ignoreipath", cmdplusattr))
+    if (viewerattrs.get("ignoreipath", cmdplusattr))
         ignoreipath = stringToBool(cmdplusattr);
 
     // Split the command line
-    list<string> lcmd;
+    vector<string> lcmd;
     if (!stringToStrings(cmd, lcmd)) {
 	QMessageBox::warning(0, "Recoll", 
 			     tr("Bad viewer command line for %1: [%2]\n"
-				"Please check the mimeconf file")
+				"Please check the mimeview file")
 			     .arg(QString::fromAscii(doc.mimetype.c_str()))
 			     .arg(QString::fromLocal8Bit(cmd.c_str())));
 	return;
@@ -1536,23 +1635,24 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 
     // Look for the command to execute in the exec path and the filters 
     // directory
-    string cmdpath;
-    if (!ExecCmd::which(lcmd.front(), cmdpath)) {
-	cmdpath = theconfig->findFilter(lcmd.front());
+    string execpath;
+    if (!ExecCmd::which(lcmd.front(), execpath)) {
+	execpath = theconfig->findFilter(lcmd.front());
 	// findFilter returns its input param if the filter is not in
 	// the normal places. As we already looked in the path, we
 	// have no use for a simple command name here (as opposed to
 	// mimehandler which will just let execvp do its thing). Erase
-	// cmdpath so that the user dialog will be started further
+	// execpath so that the user dialog will be started further
 	// down.
-	if (!cmdpath.compare(lcmd.front())) 
-	    cmdpath.erase();
+	if (!execpath.compare(lcmd.front())) 
+	    execpath.erase();
 
 	// Specialcase text/html because of the help browser need
-	if (cmdpath.empty() && !doc.mimetype.compare("text/html")) {
-	    if (lookForHtmlBrowser(cmdpath)) {
+	if (execpath.empty() && !doc.mimetype.compare("text/html") && 
+	    apptag.empty()) {
+	    if (lookForHtmlBrowser(execpath)) {
 		lcmd.clear();
-		lcmd.push_back(cmdpath);
+		lcmd.push_back(execpath);
 		lcmd.push_back("%u");
 	    }
 	}
@@ -1560,7 +1660,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 
 
     // Command not found: start the user dialog to help find another one:
-    if (cmdpath.empty()) {
+    if (execpath.empty()) {
 	QString mt = QString::fromAscii(doc.mimetype.c_str());
 	QString message = tr("The viewer specified in mimeview for %1: %2"
 			     " is not found.\nDo you want to start the "
@@ -1581,22 +1681,69 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
         // new command.
 	return;
     }
+    // Get rid of the command name. lcmd is now argv[1...n]
+    lcmd.erase(lcmd.begin());
 
-    // We may need a temp file, or not, depending on the command
-    // arguments and the fact that this is a subdoc or not.
-    bool wantsipath = (cmd.find("%i") != string::npos) || ignoreipath;
-    bool wantsfile = cmd.find("%f") != string::npos;
-    bool istempfile = false;
-    string fn = fileurltolocalpath(doc.url);
-    string orgfn = fn;
+
+    // Process the command arguments to determine if we need to create
+    // a temporary file.
+
+    // If the command has a %i parameter it will manage the
+    // un-embedding. Else if ipath is not empty, we need a temp file.
+    // This can be overridden with the "ignoreipath" attribute
+    bool groksipath = (cmd.find("%i") != string::npos) || ignoreipath;
+
+    // wantsfile: do we actually need a local file ? The only other
+    // case here is an url %u (ie: for web history).
+    bool wantsfile = cmd.find("%f") != string::npos && urlisfileurl(doc.url);
+    bool wantsparentfile = cmd.find("%F") != string::npos && 
+	urlisfileurl(doc.url);
+
+    if (wantsfile && wantsparentfile) {
+	QMessageBox::warning(0, "Recoll", 
+			     tr("Viewer command line for %1 specifies both "
+				"file and parent file value: unsupported")
+			     .arg(QString::fromAscii(doc.mimetype.c_str())));
+	return;
+    }
+	
     string url = doc.url;
+    string fn = fileurltolocalpath(doc.url);
+    Rcl::Doc pdoc;
+    if (wantsparentfile) {
+	// We want the path for the parent document. For example to
+	// open the chm file, not the internal page. Note that we just
+	// override the other file name in this case.
+	if (m_source.isNull() || !m_source->getEnclosing(doc, pdoc)) {
+	    QMessageBox::warning(0, "Recoll",
+				 tr("Cannot find parent document"));
+	    return;
+	}
+	// Override fn with the parent's : 
+	fn = fileurltolocalpath(pdoc.url);
+
+	// If the parent document has an ipath too, we need to create
+	// a temp file even if the command takes an ipath
+	// parameter. We have no viewer which could handle a double
+	// embedding. Will have to change if such a one appears.
+	if (!pdoc.ipath.empty()) {
+	    groksipath = false;
+	}
+    }
+
+    bool istempfile = false;
+
+    LOGDEB(("RclMain::startNV: groksipath %d wantsf %d wantsparentf %d\n", 
+	    groksipath, wantsfile, wantsparentfile));
 
     // If the command wants a file but this is not a file url, or
     // there is an ipath that it won't understand, we need a temp file:
     theconfig->setKeyDir(path_getfather(fn));
-    if ((wantsfile && fn.empty()) || (!wantsipath && !doc.ipath.empty())) {
+    if (((wantsfile || wantsparentfile) && fn.empty()) ||
+	(!groksipath && !doc.ipath.empty())) {
 	TempFile temp;
-	if (!FileInterner::idocToFile(temp, string(), theconfig, doc)) {
+	Rcl::Doc& thedoc = wantsparentfile ? pdoc : doc;
+	if (!FileInterner::idocToFile(temp, string(), theconfig, thedoc)) {
 	    QMessageBox::warning(0, "Recoll",
 				 tr("Cannot extract document or create "
 				    "temporary file"));
@@ -1634,8 +1781,21 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
         }
     }
 
-    // Get rid of the command name. lcmd is now argv[1...n]
-    lcmd.pop_front();
+    // If we are not called with a page number (which would happen for a call
+    // from the snippets window), see if we can compute a page number anyway.
+    if (pagenum == -1) {
+	pagenum = 1;
+	string lterm;
+	if (m_source.isNotNull())
+	    pagenum = m_source->getFirstMatchPage(doc, lterm);
+	if (pagenum == -1)
+	    pagenum = 1;
+	else // We get the match term used to compute the page
+	    term = QString::fromUtf8(lterm.c_str());
+    }
+    char cpagenum[20];
+    sprintf(cpagenum, "%d", pagenum);
+
 
     // Substitute %xx inside arguments
     string efftime;
@@ -1649,9 +1809,11 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
     map<string, string> subs;
     subs["D"] = efftime;
     subs["f"] = fn;
-    subs["F"] = orgfn;
-    subs["i"] = doc.ipath;
+    subs["F"] = fn;
+    subs["i"] = FileInterner::getLastIpathElt(doc.ipath);
     subs["M"] = doc.mimetype;
+    subs["p"] = cpagenum;
+    subs["s"] = (const char*)term.toLocal8Bit();
     subs["U"] = url;
     subs["u"] = url;
     // Let %(xx) access all metadata.
@@ -1660,7 +1822,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
         subs[it->first] = it->second;
     }
     string ncmd;
-    for (list<string>::iterator it = lcmd.begin(); 
+    for (vector<string>::iterator it = lcmd.begin(); 
          it != lcmd.end(); it++) {
         pcSubst(*it, ncmd, subs);
         LOGDEB(("%s->%s\n", it->c_str(), ncmd.c_str()));
@@ -1678,7 +1840,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 	transcode(ncmd, prcmd, fcharset, "UTF-8");
 	QString msg = tr("Executing: [") + 
 	    QString::fromUtf8(prcmd.c_str()) + "]";
-	stb->showMessage(msg, 5000);
+	stb->showMessage(msg, 10000);
     }
 
     if (!istempfile)
@@ -1690,7 +1852,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
     // We keep pushing back and never deleting. This can't be good...
     ExecCmd *ecmd = new ExecCmd;
     m_viewers.push_back(ecmd);
-    ecmd->startExec(cmdpath, lcmd, false, false);
+    ecmd->startExec(execpath, lcmd, false, false);
 }
 
 void RclMain::startManual()
@@ -1757,7 +1919,7 @@ void RclMain::showDocHistory()
     }
     // Construct a bogus SearchData structure
     RefCntr<Rcl::SearchData>searchdata = 
-	RefCntr<Rcl::SearchData>(new Rcl::SearchData(Rcl::SCLT_AND));
+	RefCntr<Rcl::SearchData>(new Rcl::SearchData(Rcl::SCLT_AND, cstr_null));
     searchdata->setDescription((const char *)tr("History data").toUtf8());
 
 
@@ -1790,7 +1952,10 @@ void RclMain::eraseDocHistory()
 void RclMain::eraseSearchHistory()
 {
     prefs.ssearchHistory.clear();
-    sSearch->queryText->clear();
+    if (sSearch)
+	sSearch->queryText->clear();
+    if (g_advshistory)
+	g_advshistory->clear();
 }
 
 // Called when the uiprefs dialog is ok'd
@@ -1800,6 +1965,7 @@ void RclMain::setUIPrefs()
 	return;
     LOGDEB(("Recollmain::setUIPrefs\n"));
     reslist->setFont();
+    sSearch->setPrefs();
 }
 
 void RclMain::enableNextPage(bool yesno)
@@ -1823,19 +1989,6 @@ QString RclMain::getQueryDescription()
     return QString::fromUtf8(m_source->getDescription().c_str());
 }
 
-/** Show detailed expansion of a query */
-void RclMain::showQueryDetails()
-{
-    if (m_source.isNull())
-	return;
-    string oq = breakIntoLines(m_source->getDescription(), 100, 50);
-    QString str;
-    QString desc = tr("Result count (est.)") + ": " + 
-	str.setNum(m_source->getResCnt()) + "<br>";
-    desc += tr("Query details") + ": " + QString::fromUtf8(oq.c_str());
-    QMessageBox::information(this, tr("Query details"), desc);
-}
-
 // User pressed a category button: set filter params in reslist
 void RclMain::catgFilter(int id)
 {
@@ -1882,5 +2035,5 @@ bool RclMain::eventFilter(QObject *, QEvent *event)
 
 void RclMain::applyStyleSheet()
 {
-    ::applyStyleSheet(prefs.stylesheetFile);
+    ::applyStyleSheet(prefs.qssFile);
 }
