@@ -71,7 +71,9 @@ public:
 	  m_idxproc(0),
 	  m_sortspecnochange(false),
 	  m_indexerState(IXST_RUNNINGNOTMINE),
-	  m_queryActive(false)
+	  m_queryActive(false),
+	  m_firstIndexing(false),
+	  m_searchIsSimple(false)
     {
 	setupUi(this);
 	init();
@@ -86,6 +88,11 @@ public:
     /** Same usage: actually display the current urltoview */
     virtual void viewUrl();
 
+    bool lastSearchSimple() 
+    {
+	return m_searchIsSimple;
+    }
+
 public slots:
     virtual bool close();
     virtual void fileExit();
@@ -93,7 +100,7 @@ public slots:
     virtual void periodic100();
     virtual void toggleIndexing();
     virtual void rebuildIndex();
-    virtual void startSearch(RefCntr<Rcl::SearchData> sdata);
+    virtual void startSearch(RefCntr<Rcl::SearchData> sdata, bool issimple);
     virtual void previewClosed(Preview *w);
     virtual void showAdvSearchDialog();
     virtual void showSpellDialog();
@@ -117,9 +124,12 @@ public slots:
     virtual void enableNextPage(bool);
     virtual void enablePrevPage(bool);
     virtual void docExpand(Rcl::Doc);
+    virtual void showSubDocs(Rcl::Doc);
+    virtual void showSnippets(Rcl::Doc);
     virtual void startPreview(int docnum, Rcl::Doc doc, int keymods);
     virtual void startPreview(Rcl::Doc);
-    virtual void startNativeViewer(Rcl::Doc);
+    virtual void startNativeViewer(Rcl::Doc, int pagenum = -1, 
+				   QString term=QString());
     virtual void saveDocToFile(Rcl::Doc);
     virtual void previewNextInTab(Preview *, int sid, int docnum);
     virtual void previewPrevInTab(Preview *, int sid, int docnum);
@@ -132,13 +142,11 @@ public slots:
     virtual void catgFilter(int);
     virtual void initDbOpen();
     virtual void toggleFullScreen();
-    virtual void focusToSearch();
     virtual void on_actionSortByDateAsc_toggled(bool on);
     virtual void on_actionSortByDateDesc_toggled(bool on);
     virtual void on_actionShowResultsAsTable_toggled(bool on);
     virtual void onSortDataChanged(DocSeqSortSpec);
     virtual void resultCount(int);
-    virtual void showQueryDetails();
     virtual void applyStyleSheet();
 
 signals:
@@ -176,6 +184,8 @@ private:
     RefCntr<DocSequence> m_source;
     IndexerState      m_indexerState;
     bool              m_queryActive;
+    bool              m_firstIndexing;
+    bool              m_searchIsSimple; // Last search was started from simple
 
     // If set on init, will be displayed either through ext app, or
     // preview (if no ext app set)
