@@ -31,9 +31,9 @@
 #include <qtimer.h>
 #include <qmessagebox.h>
 #include <qimage.h>
-#include <qclipboard.h>
 #include <qscrollbar.h>
 #include <QTextBlock>
+#include <QShortcut>
 #ifndef __APPLE__
 #include <qx11info_x11.h>
 #endif
@@ -50,17 +50,18 @@
 #include "refcntr.h"
 #include "internfile.h"
 #include "indexer.h"
-
+#include "snippets_w.h"
+#include "listdialog.h"
 #include "reslist.h"
 #include "moc_reslist.cpp"
 #include "rclhelp.h"
 #ifdef RCL_USE_ASPELL
 #include "rclaspell.h"
 #endif
+#include "respopup.h"
 
-#ifndef MIN
-#define MIN(A,B) ((A) < (B) ? (A) : (B))
-#endif
+static const QKeySequence quitKeySeq("Ctrl+q");
+static const QKeySequence closeKeySeq("Ctrl+w");
 
 #ifndef RESLIST_TEXTBROWSER
 #include <QWebFrame>
@@ -71,7 +72,7 @@
 class QtGuiResListPager : public ResListPager {
 public:
     QtGuiResListPager(ResList *p, int ps) 
-	: ResListPager(ps), m_parent(p) 
+	: ResListPager(ps), m_reslist(p) 
     {}
     virtual bool append(const string& data);
     virtual bool append(const string& data, int idx, const Rcl::Doc& doc);
@@ -87,7 +88,7 @@ public:
     virtual string absSep() {return (const char *)(prefs.abssep.toUtf8());}
     virtual string iconUrl(RclConfig *, Rcl::Doc& doc);
 private:
-    ResList *m_parent;
+    ResList *m_reslist;
 };
 
 #if 0
@@ -109,7 +110,7 @@ bool QtGuiResListPager::append(const string& data)
 {
     LOGDEB2(("QtGuiReslistPager::appendString   : %s\n", data.c_str()));
     logdata(data.c_str());
-    m_parent->append(QString::fromUtf8(data.c_str()));
+    m_reslist->append(QString::fromUtf8(data.c_str()));
     return true;
 }
 
@@ -117,24 +118,24 @@ bool QtGuiResListPager::append(const string& data, int docnum,
 			       const Rcl::Doc&)
 {
     LOGDEB2(("QtGuiReslistPager::appendDoc: blockCount %d, %s\n",
-	    m_parent->document()->blockCount(), data.c_str()));
+	    m_reslist->document()->blockCount(), data.c_str()));
     logdata(data.c_str());
 #ifdef RESLIST_TEXTBROWSER
-    int blkcnt0 = m_parent->document()->blockCount();
-    m_parent->moveCursor(QTextCursor::End, QTextCursor::MoveAnchor);
-    m_parent->textCursor().insertBlock();
-    m_parent->insertHtml(QString::fromUtf8(data.c_str()));
-    m_parent->moveCursor(QTextCursor::Start, QTextCursor::MoveAnchor);
-    m_parent->ensureCursorVisible();
-    int blkcnt1 = m_parent->document()->blockCount();
+    int blkcnt0 = m_reslist->document()->blockCount();
+    m_reslist->moveCursor(QTextCursor::End, QTextCursor::MoveAnchor);
+    m_reslist->textCursor().insertBlock();
+    m_reslist->insertHtml(QString::fromUtf8(data.c_str()));
+    m_reslist->moveCursor(QTextCursor::Start, QTextCursor::MoveAnchor);
+    m_reslist->ensureCursorVisible();
+    int blkcnt1 = m_reslist->document()->blockCount();
     for (int block = blkcnt0; block < blkcnt1; block++) {
-	m_parent->m_pageParaToReldocnums[block] = docnum;
+	m_reslist->m_pageParaToReldocnums[block] = docnum;
     }
 #else
     QString sdoc = QString("<div class=\"rclresult\" rcldocnum=\"%1\">").arg(docnum);
-    m_parent->append(sdoc);
-    m_parent->append(QString::fromUtf8(data.c_str()));
-    m_parent->append("</div>");
+    m_reslist->append(sdoc);
+    m_reslist->append(QString::fromUtf8(data.c_str()));
+    m_reslist->append("</div>");
 #endif
     return true;
 }
@@ -189,23 +190,29 @@ void QtGuiResListPager::suggest(const vector<string>uterms,
         LOGERR(("QtGuiResListPager:: aspell not initialized\n"));
         return;
     }
+
+    bool issimple = m_reslist && m_reslist->m_rclmain && 
+	m_reslist->m_rclmain->lastSearchSimple();
+
     for (vector<string>::const_iterator uit = uterms.begin();
          uit != uterms.end(); uit++) {
         list<string> asuggs;
         string reason;
 
-	// If the term is in the index, we don't suggest alternatives. 
-	// Actually, we may want to check the frequencies and propose something
-	// anyway if a possible variation is much more common (as google does)
-        if (aspell->check(*rcldb, *uit, reason))
-            continue;
-        else if (!reason.empty())
-            return;
+	// If the term is in the dictionary, Aspell::suggest won't
+	// list alternatives. In fact we may want to check the
+	// frequencies and propose something anyway if a possible
+	// variation is much more common (as google does) ?
         if (!aspell->suggest(*rcldb, *uit, asuggs, reason)) {
             LOGERR(("QtGuiResListPager::suggest: aspell failed: %s\n", 
                     reason.c_str()));
             continue;
         }
+
+	// We should check that the term stems differently from the
+	// base word (else it's not useful to expand the search). Or
+	// is it ? This should depend if stemming is turned on or not
+
         if (!asuggs.empty()) {
             sugg[*uit] = vector<string>(asuggs.begin(), asuggs.end());
 	    if (sugg[*uit].size() > 5)
@@ -213,8 +220,10 @@ void QtGuiResListPager::suggest(const vector<string>uterms,
 	    // Set up the links as a <href="Sold|new">. 
 	    for (vector<string>::iterator it = sugg[*uit].begin();
 		 it != sugg[*uit].end(); it++) {
-		*it = string("<a href=\"S") + *uit + "|" + *it + "\">" +
-		    *it + "</a>";
+		if (issimple) {
+		    *it = string("<a href=\"S") + *uit + "|" + *it + "\">" +
+			*it + "</a>";
+		}
 	    }
         }
     }
@@ -250,19 +259,30 @@ string QtGuiResListPager::iconUrl(RclConfig *config, Rcl::Doc& doc)
 
 class PlainToRichQtReslist : public PlainToRich {
 public:
-    virtual ~PlainToRichQtReslist() {}
-    virtual string startMatch() {
+    virtual string startMatch(unsigned int idx)
+    {
+	if (m_hdata) {
+	    string s1, s2;
+	    stringsToString<vector<string> >(m_hdata->groups[idx], s1); 
+	    stringsToString<vector<string> >(m_hdata->ugroups[m_hdata->grpsugidx[idx]], s2);
+	    LOGDEB(("Reslist startmatch: group %s user group %s\n", s1.c_str(), s2.c_str()));
+	}
+		
 	return string("<span class='rclmatch' style='color: ")
 	    + string((const char *)prefs.qtermcolor.toAscii()) + string("'>");
     }
-    virtual string endMatch() {return string("</span>");}
+    virtual string endMatch() 
+    {
+	return string("</span>");
+    }
 };
 static PlainToRichQtReslist g_hiliter;
 
 /////////////////////////////////////
 
 ResList::ResList(QWidget* parent, const char* name)
-    : RESLIST_PARENTCLASS(parent)
+    : RESLIST_PARENTCLASS(parent), m_curPvDoc(-1), m_lstClckMod(0), 
+      m_listId(0), m_rclmain(0), m_ismainres(true)
 {
     if (!name)
 	setObjectName("resList");
@@ -285,6 +305,7 @@ ResList::ResList(QWidget* parent, const char* name)
     page()->setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
     settings()->setAttribute(QWebSettings::JavascriptEnabled, true);
 #endif
+
     setFont();
     languageChange();
 
@@ -297,13 +318,11 @@ ResList::ResList(QWidget* parent, const char* name)
     connect(this, SIGNAL(highlighted(const QString &)), 
 	    this, SLOT(highlighted(const QString &)));
 #endif
+
     setContextMenuPolicy(Qt::CustomContextMenu);
     connect(this, SIGNAL(customContextMenuRequested(const QPoint&)),
 	    this, SLOT(createPopupMenu(const QPoint&)));
 
-    m_curPvDoc = -1;
-    m_lstClckMod = 0;
-    m_listId = 0;
     m_pager = new QtGuiResListPager(this, prefs.respagesize);
     m_pager->setHighLighter(&g_hiliter);
 }
@@ -326,9 +345,28 @@ ResList::~ResList()
 	QT_TR_NOOP("Open"),
 	QT_TR_NOOP("(show query)"),
         QT_TR_NOOP("<p><i>Alternate spellings (accents suppressed): </i>"),
+        QT_TR_NOOP("<p><i>Alternate spellings: </i>"),
     };
 }
 
+void ResList::setRclMain(RclMain *m, bool ismain) 
+{
+    m_rclmain = m;
+    m_ismainres = ismain;
+    if (!m_ismainres) {
+	connect(new QShortcut(closeKeySeq, this), SIGNAL (activated()), 
+		this, SLOT (close()));
+	connect(new QShortcut(quitKeySeq, this), SIGNAL (activated()), 
+		m_rclmain, SLOT (fileExit()));
+	connect(this, SIGNAL(previewRequested(Rcl::Doc)), 
+		m_rclmain, SLOT(startPreview(Rcl::Doc)));
+	connect(this, SIGNAL(docSaveToFileClicked(Rcl::Doc)), 
+		m_rclmain, SLOT(saveDocToFile(Rcl::Doc)));
+	connect(this, SIGNAL(editRequested(Rcl::Doc)), 
+		m_rclmain, SLOT(startNativeViewer(Rcl::Doc)));
+    }
+}
+
 void ResList::setFont()
 {
 #ifdef RESLIST_TEXTBROWSER
@@ -495,7 +533,7 @@ bool ResList::getDoc(int docnum, Rcl::Doc &doc)
 
     // Is docnum in current page ? Then all Ok
     if (docnum >= winfirst && docnum <= winlast) {
-	return m_source->getDoc(docnum, doc);
+	return m_pager->getDoc(docnum, doc);
     }
 
     // Else we accept to page down or up but not further
@@ -507,7 +545,7 @@ bool ResList::getDoc(int docnum, Rcl::Doc &doc)
     winfirst = pageFirstDocNum();
     winlast = m_pager->pageLastDocNum();
     if (docnum >= winfirst && docnum <= winlast) {
-	return m_source->getDoc(docnum, doc);
+	return m_pager->getDoc(docnum, doc);
     }
     return false;
 }
@@ -633,9 +671,9 @@ bool ResList::scrollIsAtTop()
 	LOGDEB(("Scrollattop: cur %d min %d\n", cur, min));
 	ret = (cur == min);
     }
-#endif
     LOGDEB2(("scrollIsAtTop: returning %d\n", ret));
     return ret;
+#endif
 }
 
 // Show previous page of results. We just set the current number back
@@ -700,7 +738,6 @@ void ResList::displayPage()
 
     // Possibly color paragraph of current preview if any
     previewExposed(m_curPvDoc);
-
 }
 
 // Color paragraph (if any) of currently visible preview
@@ -786,6 +823,44 @@ void ResList::mouseDoubleClickEvent(QMouseEvent *event)
 #endif
 }
 
+void ResList::newDupsW(const Rcl::Doc&, const vector<Rcl::Doc>& dups)
+{
+    ListDialog dialog;
+    dialog.setWindowTitle(tr("Duplicate documents"));
+
+    dialog.groupBox->setTitle(tr("These Urls ( | ipath) share the same"
+				 " content:"));
+    // We replace the list with an editor so that the user can copy/paste
+    delete dialog.listWidget;
+    QTextEdit *editor = new QTextEdit(dialog.groupBox);
+    editor->setReadOnly(TRUE);
+    dialog.horizontalLayout->addWidget(editor);
+
+    for (vector<Rcl::Doc>::const_iterator it = dups.begin(); 
+	 it != dups.end(); it++) {
+	if (it->ipath.empty()) 
+	    editor->append(QString::fromLocal8Bit(it->url.c_str()));
+	else 
+	    editor->append(QString::fromLocal8Bit(it->url.c_str()) + " | " +
+			   QString::fromUtf8(it->ipath.c_str()));
+    }
+    editor->moveCursor(QTextCursor::Start);
+    editor->ensureCursorVisible();
+    dialog.exec();
+}
+
+void ResList::showQueryDetails()
+{
+    if (m_source.isNull())
+	return;
+    string oq = breakIntoLines(m_source->getDescription(), 100, 50);
+    QString str;
+    QString desc = tr("Result count (est.)") + ": " + 
+	str.setNum(m_source->getResCnt()) + "<br>";
+    desc += tr("Query details") + ": " + QString::fromUtf8(oq.c_str());
+    QMessageBox::information(this, tr("Query details"), desc);
+}
+
 void ResList::linkWasClicked(const QUrl &url)
 {
     string ascurl = (const char *)url.toString().toAscii();;
@@ -793,9 +868,48 @@ void ResList::linkWasClicked(const QUrl &url)
 
     int what = ascurl[0];
     switch (what) {
+
+    // Open abstract/snippets window
+    case 'A':
+    {
+	if (m_source.isNull()) 
+	    return;
+	int i = atoi(ascurl.c_str()+1) - 1;
+	Rcl::Doc doc;
+	if (!getDoc(i, doc)) {
+	    LOGERR(("ResList::linkWasClicked: can't get doc for %d\n", i));
+	    return;
+	}
+	emit(showSnippets(doc));
+    }
+    break;
+
+    // Show duplicates
+    case 'D':
+    {
+	if (m_source.isNull()) 
+	    return;
+	int i = atoi(ascurl.c_str()+1) - 1;
+	Rcl::Doc doc;
+	if (!getDoc(i, doc)) {
+	    LOGERR(("ResList::linkWasClicked: can't get doc for %d\n", i));
+	    return;
+	}
+	vector<Rcl::Doc> dups;
+	if (m_source->docDups(doc, dups)) {
+	    newDupsW(doc, dups);
+	}
+    }
+    break;
+
+    // Show query details
     case 'H': 
-	emit headerClicked(); 
+    {
+	showQueryDetails();
 	break;
+    }
+
+    // Preview and edit
     case 'P': 
     case 'E': 
     {
@@ -805,18 +919,27 @@ void ResList::linkWasClicked(const QUrl &url)
 	    LOGERR(("ResList::linkWasClicked: can't get doc for %d\n", i));
 	    return;
 	}
-	if (what == 'P')
-	    emit docPreviewClicked(i, doc, m_lstClckMod);
-	else
-	    emit docEditClicked(doc);
+	if (what == 'P') {
+	    if (m_ismainres) {
+		emit docPreviewClicked(i, doc, m_lstClckMod);
+	    } else {
+		emit previewRequested(doc);
+	    }
+	} else {
+	    emit editRequested(doc);
+	}
     }
     break;
+
+    // Next/prev page
     case 'n':
 	resultPageNext();
 	break;
     case 'p':
 	resultPageBack();
 	break;
+
+	// Spelling: replacement suggestion clicked
     case 'S':
     {
 	QString s = url.toString();
@@ -830,6 +953,7 @@ void ResList::linkWasClicked(const QUrl &url)
 	}
     }
     break;
+
     default: 
 	LOGERR(("ResList::linkWasClicked: bad link [%s]\n", ascurl.c_str()));
 	break;// ?? 
@@ -859,37 +983,27 @@ void ResList::createPopupMenu(const QPoint& pos)
 
     if (m_popDoc < 0) 
 	return;
-    QMenu *popup = new QMenu(this);
-    popup->addAction(tr("&Preview"), this, SLOT(menuPreview()));
-
     Rcl::Doc doc;
-    bool havedoc = getDoc(m_popDoc, doc);
-    string apptag;
-    if (havedoc)
-	doc.getmeta(Rcl::Doc::keyapptg, &apptag);
+    if (!getDoc(m_popDoc, doc))
+	return;
 
-    if (havedoc && !theconfig->getMimeViewerDef(doc.mimetype, apptag).empty()) {
-	popup->addAction(tr("&Open"), this, SLOT(menuEdit()));
-    }
-    popup->addAction(tr("Copy &File Name"), this, SLOT(menuCopyFN()));
-    popup->addAction(tr("Copy &URL"), this, SLOT(menuCopyURL()));
-    if (havedoc && !doc.ipath.empty()) {
-	popup->addAction(tr("&Write to File"), this, SLOT(menuSaveToFile()));
-    }
-
-    popup->addAction(tr("Find &similar documents"), this, SLOT(menuExpand()));
-    popup->addAction(tr("Preview P&arent document/folder"), 
-		      this, SLOT(menuPreviewParent()));
-    popup->addAction(tr("&Open Parent document/folder"), 
-		     this, SLOT(menuOpenParent()));
+    int options =  ResultPopup::showSaveOne;
+    if (m_ismainres)
+	options |= ResultPopup::isMain;
+    QMenu *popup = ResultPopup::create(this, options, m_source, doc);
     popup->popup(mapToGlobal(pos));
 }
 
 void ResList::menuPreview()
 {
     Rcl::Doc doc;
-    if (getDoc(m_popDoc, doc))
-	emit docPreviewClicked(m_popDoc, doc, 0);
+    if (getDoc(m_popDoc, doc)) {
+	if (m_ismainres) {
+	    emit docPreviewClicked(m_popDoc, doc, 0);
+	} else {
+	    emit previewRequested(doc);
+	}
+    }
 }
 
 void ResList::menuSaveToFile()
@@ -902,74 +1016,56 @@ void ResList::menuSaveToFile()
 void ResList::menuPreviewParent()
 {
     Rcl::Doc doc;
-    if (!getDoc(m_popDoc, doc) || m_source.isNull()) 
-	return;
-    Rcl::Doc pdoc;
-    if (m_source->getEnclosing(doc, pdoc)) {
-	emit previewRequested(pdoc);
-    } else {
-	// No parent doc: show enclosing folder with app configured for
-	// directories
-	pdoc.url = path_getfather(doc.url);
-	pdoc.mimetype = "application/x-fsdirectory";
-	emit editRequested(pdoc);
+    if (getDoc(m_popDoc, doc) && !m_source.isNull())  {
+	Rcl::Doc pdoc = ResultPopup::getParent(m_source, doc);
+	if (pdoc.mimetype == "inode/directory") {
+	    emit editRequested(pdoc);
+	} else {
+	    emit previewRequested(pdoc);
+	}
     }
 }
 
 void ResList::menuOpenParent()
 {
     Rcl::Doc doc;
-    if (!getDoc(m_popDoc, doc) || m_source.isNull()) 
-	return;
-    Rcl::Doc pdoc;
-    if (m_source->getEnclosing(doc, pdoc)) {
-	emit editRequested(pdoc);
-    } else {
-	// No parent doc: show enclosing folder with app configured for
-	// directories
-	pdoc.url = path_getfather(doc.url);
-	pdoc.mimetype = "application/x-fsdirectory";
-	emit editRequested(pdoc);
-    }
+    if (getDoc(m_popDoc, doc) && m_source.isNotNull()) 
+	emit editRequested(ResultPopup::getParent(m_source, doc));
+}
+
+void ResList::menuShowSnippets()
+{
+    Rcl::Doc doc;
+    if (getDoc(m_popDoc, doc))
+	emit showSnippets(doc);
+}
+
+void ResList::menuShowSubDocs()
+{
+    Rcl::Doc doc;
+    if (getDoc(m_popDoc, doc)) 
+	emit showSubDocs(doc);
 }
 
 void ResList::menuEdit()
 {
     Rcl::Doc doc;
     if (getDoc(m_popDoc, doc))
-	emit docEditClicked(doc);
+	emit editRequested(doc);
 }
 
 void ResList::menuCopyFN()
 {
-    LOGDEB(("menuCopyFN\n"));
     Rcl::Doc doc;
-    if (getDoc(m_popDoc, doc)) {
-	LOGDEB(("menuCopyFN: Got doc, fn: [%s]\n", doc.url.c_str()));
-	// Our urls currently always begin with "file://" 
-        //
-        // Problem: setText expects a QString. Passing a (const char*)
-        // as we used to do causes an implicit conversion from
-        // latin1. File are binary and the right approach would be no
-        // conversion, but it's probably better (less worse...) to
-        // make a "best effort" tentative and try to convert from the
-        // locale's charset than accept the default conversion.
-        QString qfn = QString::fromLocal8Bit(doc.url.c_str()+7);
-	QApplication::clipboard()->setText(qfn, QClipboard::Selection);
-	QApplication::clipboard()->setText(qfn, QClipboard::Clipboard);
-    }
+    if (getDoc(m_popDoc, doc))
+	ResultPopup::copyFN(doc);
 }
 
 void ResList::menuCopyURL()
 {
     Rcl::Doc doc;
-    if (getDoc(m_popDoc, doc)) {
-	string url =  url_encode(doc.url, 7);
-	QApplication::clipboard()->setText(url.c_str(), 
-					   QClipboard::Selection);
-	QApplication::clipboard()->setText(url.c_str(), 
-					   QClipboard::Clipboard);
-    }
+    if (getDoc(m_popDoc, doc))
+	ResultPopup::copyURL(doc);
 }
 
 void ResList::menuExpand()
