@@ -41,6 +41,7 @@ using std::pair;
 #include "rcldoc.h"
 #include "reslistpager.h"
 
+class RclMain;
 class QtGuiResListPager;
 
 /**
@@ -66,6 +67,7 @@ class ResList : public RESLIST_PARENTCLASS
     int listId() const {return m_listId;}
     int pageFirstDocNum();
     void setFont();
+    void setRclMain(RclMain *m, bool ismain);
 
  public slots:
     virtual void setDocSource(RefCntr<DocSequence> nsource);
@@ -84,21 +86,24 @@ class ResList : public RESLIST_PARENTCLASS
     virtual void menuExpand();
     virtual void menuPreviewParent();
     virtual void menuOpenParent();
+    virtual void menuShowSnippets();
+    virtual void menuShowSubDocs();
     virtual void previewExposed(int);
     virtual void append(const QString &text);
     virtual void readDocSource();
     virtual void highlighted(const QString& link);
     virtual void createPopupMenu(const QPoint& pos);
+    virtual void showQueryDetails();
 	
  signals:
     void nextPageAvailable(bool);
     void prevPageAvailable(bool);
-    void docEditClicked(Rcl::Doc);
     void docPreviewClicked(int, Rcl::Doc, int);
     void docSaveToFileClicked(Rcl::Doc);
     void previewRequested(Rcl::Doc);
+    void showSnippets(Rcl::Doc);
+    void showSubDocs(Rcl::Doc);
     void editRequested(Rcl::Doc);
-    void headerClicked();
     void docExpand(Rcl::Doc);
     void wordSelect(QString);
     void wordReplace(const QString&, const QString&);
@@ -128,8 +133,12 @@ class ResList : public RESLIST_PARENTCLASS
     virtual int docnumfromparnum(int);
     virtual pair<int,int> parnumfromdocnum(int);
 #else
-    QString    m_text; // webview doesn't take text incrementally, store it.
+    // Webview makes it more difficult to append text incrementally,
+    // so we store the page and display it when done.
+    QString    m_text; 
 #endif
+    RclMain   *m_rclmain;
+    bool m_ismainres;
 
     virtual void displayPage(); // Display current page
     static int newListId();
@@ -137,6 +146,7 @@ class ResList : public RESLIST_PARENTCLASS
     bool scrollIsAtTop();
     bool scrollIsAtBottom();
     void setupArrows();
+    void newDupsW(const Rcl::Doc& doc, const std::vector<Rcl::Doc>& dups);
 };
 
 
