@@ -63,7 +63,8 @@ static const int TEXTINCELLVTRANS = -4;
 class PlainToRichQtReslist : public PlainToRich {
 public:
     virtual ~PlainToRichQtReslist() {}
-    virtual string startMatch() {
+    virtual string startMatch(unsigned int) 
+    {
 	return string("<span style='color: ")
 	    + string((const char *)prefs.qtermcolor.toAscii()) + string("'>");
     }
