@@ -19,6 +19,10 @@
 
 #include <Qt>
 
+#include <string>
+#include <map>
+#include <vector>
+
 #include "ui_restable.h"
 #include "refcntr.h"
 #include "docseq.h"
@@ -26,7 +30,7 @@
 
 class ResTable;
 
-typedef string (FieldGetter)(const string& fldname, const Rcl::Doc& doc);
+typedef std::string (FieldGetter)(const std::string& fldname, const Rcl::Doc& doc);
 
 class RecollModel : public QAbstractTableModel {
 
@@ -49,15 +53,15 @@ public:
     virtual void setDocSource(RefCntr<DocSequence> nsource);
     virtual RefCntr<DocSequence> getDocSource() {return m_source;}
     virtual void deleteColumn(int);
-    virtual const vector<string>& getFields() {return m_fields;}
-    virtual const map<string, QString>& getAllFields() 
+    virtual const std::vector<std::string>& getFields() {return m_fields;}
+    virtual const std::map<std::string, QString>& getAllFields() 
     { 
 	return o_displayableFields;
     }
-    virtual void addColumn(int, const string&);
+    virtual void addColumn(int, const std::string&);
     // Some column name are aliases/translator for base document field 
     // (ie: date, datetime->mtime). Help deal with this:
-    virtual string baseField(const string&);
+    virtual std::string baseField(const std::string&);
 
     // Ignore sort() call because 
     virtual void setIgnoreSort(bool onoff) {m_ignoreSort = onoff;}
@@ -69,12 +73,12 @@ signals:
 
 private:
     mutable RefCntr<DocSequence> m_source;
-    vector<string> m_fields;
-    vector<FieldGetter*> m_getters;
-    static map<string, QString> o_displayableFields;
+    std::vector<std::string> m_fields;
+    std::vector<FieldGetter*> m_getters;
+    static std::map<std::string, QString> o_displayableFields;
     bool m_ignoreSort;
-    FieldGetter* chooseGetter(const string&);
-    HiliteData m_hdata;
+    FieldGetter* chooseGetter(const std::string&);
+    HighlightData m_hdata;
 };
 
 class ResTable;
@@ -96,6 +100,7 @@ private:
 
 class ResTablePager;
 class QUrl;
+class RclMain;
 
 class ResTable : public QWidget, public Ui::ResTable 
 {
@@ -104,7 +109,8 @@ class ResTable : public QWidget, public Ui::ResTable
 public:
     ResTable(QWidget* parent = 0) 
 	: QWidget(parent),
-	  m_model(0), m_pager(0), m_detail(0), m_detaildocnum(-1)
+	  m_model(0), m_pager(0), m_detail(0), m_detaildocnum(-1),
+	  m_rclmain(0), m_ismainres(true)
     {
 	setupUi(this);
 	init();
@@ -115,6 +121,8 @@ public:
     virtual ResTableDetailArea* getDetailArea() {return m_detail;}
     virtual int getDetailDocNumOrTopRow();
 
+    void setRclMain(RclMain *m, bool ismain);
+
 public slots:
     virtual void onTableView_currentChanged(const QModelIndex&);
     virtual void on_tableView_entered(const QModelIndex& index);
@@ -126,12 +134,15 @@ public slots:
     virtual void createPopupMenu(const QPoint& pos);
     virtual void menuPreview();
     virtual void menuSaveToFile();
+    virtual void menuSaveSelection();
     virtual void menuEdit();
     virtual void menuCopyFN();
     virtual void menuCopyURL();
     virtual void menuExpand();
     virtual void menuPreviewParent();
     virtual void menuOpenParent();
+    virtual void menuShowSnippets();
+    virtual void menuShowSubDocs();
     virtual void createHeaderPopupMenu(const QPoint&);
     virtual void deleteColumn();
     virtual void addColumn();
@@ -142,12 +153,13 @@ public slots:
 
 signals:
     void docPreviewClicked(int, Rcl::Doc, int);
-    void docEditClicked(Rcl::Doc);
     void docSaveToFileClicked(Rcl::Doc);
     void previewRequested(Rcl::Doc);
     void editRequested(Rcl::Doc);
     void headerClicked();
     void docExpand(Rcl::Doc);
+    void showSubDocs(Rcl::Doc);
+    void showSnippets(Rcl::Doc);
 
     friend class ResTablePager;
     friend class ResTableDetailArea;
@@ -159,6 +171,8 @@ private:
     int            m_detaildocnum;
     Rcl::Doc       m_detaildoc;
     int            m_popcolumn;
+    RclMain *m_rclmain;
+    bool     m_ismainres;
 };
 
 
