@@ -24,6 +24,8 @@
 #include <qwhatsthis.h>
 #include <qmessagebox.h>
 #include <qevent.h>
+#include <QTimer>
+#include <QCompleter>
 
 #include "debuglog.h"
 #include "guiutils.h"
@@ -34,6 +36,10 @@
 #include "wasatorcl.h"
 #include "rclhelp.h"
 
+// Typing interval after which we consider starting autosearch: no sense to do
+// this is user is typing fast and continuously
+static const int strokeTimeoutMS = 250;
+
 void SSearch::init()
 {
     // See enum above and keep in order !
@@ -58,19 +64,75 @@ void SSearch::init()
     connect(searchTypCMB, SIGNAL(activated(int)), this, SLOT(searchTypeChanged(int)));
 
     queryText->installEventFilter(this);
+    queryText->view()->installEventFilter(this);
+    queryText->setInsertPolicy(QComboBox::NoInsert);
+    // Note: we can't do the obvious and save the completer instead because
+    // the combobox lineedit will delete the completer on setCompleter(0).
+    // But the model does not belong to the completer so it's not deleted...
+    m_savedModel = queryText->completer()->model();
+    if (prefs.ssearchNoComplete)
+	queryText->completer()->setModel(0);
+    // Recoll searches are always case-sensitive because of the use of
+    // capitalization to suppress stemming
+    queryText->completer()->setCaseSensitivity(Qt::CaseSensitive);
+    m_displayingCompletions = false;
     m_escape = false;
+    m_disableAutosearch = true;
+    m_stroketimeout = new QTimer(this);
+    m_stroketimeout->setSingleShot(true);
+    connect(m_stroketimeout, SIGNAL(timeout()), this, SLOT(timerDone()));
+    m_keystroke = false;
+}
+
+void SSearch::takeFocus()
+{
+    LOGDEB2(("SSearch: take focus\n"));
+    queryText->setFocus(Qt::ShortcutFocusReason);
+    // If the focus was already in the search entry, the text is not selected.
+    // Do it for consistency
+    queryText->lineEdit()->selectAll();
+}
+
+void SSearch::timerDone()
+{
+    QString qs = queryText->currentText();
+    LOGDEB1(("SSearch::timerDone: qs [%s]\n", qs2utf8s(qs).c_str()));
+    searchTextChanged(qs);
 }
 
 void SSearch::searchTextChanged(const QString& text)
 {
+    QString qs = queryText->currentText();
+    LOGDEB1(("SSearch::searchTextChanged. ks %d qs [%s]\n", 
+	     m_keystroke, qs2utf8s(text).c_str()));
     if (text.isEmpty()) {
 	searchPB->setEnabled(false);
 	clearqPB->setEnabled(false);
+	queryText->setFocus();
 	emit clearSearch();
     } else {
 	searchPB->setEnabled(true);
 	clearqPB->setEnabled(true);
+	if (m_keystroke) {
+	    m_tstartqs = qs;
+	}
+	if (prefs.ssearchAsYouType && !m_disableAutosearch && 
+	    !m_keystroke && m_tstartqs == qs) {
+	    m_disableAutosearch = true;
+	    string s;
+	    int cs = partialWord(s);
+	    LOGDEB1(("SSearch::searchTextChanged: autosearch. cs %d s [%s]\n", 
+		     cs, s.c_str()));
+	    if (cs < 0) {
+		startSimpleSearch();
+	    } else if (!m_stroketimeout->isActive() && s.size() >= 2) {
+		s = qs2utf8s(queryText->currentText());
+		s += "*";
+		startSimpleSearch(s, 20);
+	    }
+	}
     }
+    m_keystroke = false;
 }
 
 void SSearch::searchTypeChanged(int typ)
@@ -116,36 +178,79 @@ void SSearch::searchTypeChanged(int typ)
 
 void SSearch::startSimpleSearch()
 {
-    if (queryText->currentText().length() == 0)
+    QString qs = queryText->currentText();
+    LOGDEB(("SSearch::startSimpleSearch(): qs [%s]\n", qs2utf8s(qs).c_str()));
+    if (qs.length() == 0)
 	return;
 
     string u8 = (const char *)queryText->currentText().toUtf8();
-    LOGDEB(("SSearch::startSimpleSearch: [%s]\n", u8.c_str()));
 
     trimstring(u8);
     if (u8.length() == 0)
 	return;
 
+    if (!startSimpleSearch(u8))
+	return;
+
+    LOGDEB(("startSimpleSearch: updating history\n"));
+    // Search terms history:
+    // We want to have the new text at the top and any older identical
+    // entry to be erased. There is no standard qt policy to do this ? 
+    // So do it by hand.
+    QString txt = queryText->currentText();
+    QString txtt = txt.trimmed();
+    int index = queryText->findText(txtt);
+    if (index > 0) {
+	queryText->removeItem(index);
+    }
+    if (index != 0) {
+	queryText->insertItem(0, txtt);
+	queryText->setEditText(txt);
+    }
+    m_disableAutosearch = true;
+    m_stroketimeout->stop();
+
+    // Save the current state of the listbox list to the prefs (will
+    // go to disk)
+    prefs.ssearchHistory.clear();
+    for (int index = 0; index < queryText->count(); index++) {
+	prefs.ssearchHistory.push_back(queryText->itemText(index));
+    }
+}
+void SSearch::setPrefs()
+{
+    if (prefs.ssearchNoComplete) {
+	queryText->completer()->setModel(0);
+    } else {
+	queryText->completer()->setModel(m_savedModel);
+    }
+}
+
+bool SSearch::startSimpleSearch(const string& u8, int maxexp)
+{
+    LOGDEB(("SSearch::startSimpleSearch(%s)\n", u8.c_str()));
+    string stemlang = prefs.stemlang();
+
     SSearchType tp = (SSearchType)searchTypCMB->currentIndex();
     Rcl::SearchData *sdata = 0;
 
     if (tp == SST_LANG) {
 	string reason;
         if (prefs.autoSuffsEnable)
-            sdata = wasaStringToRcl(theconfig, u8, reason, 
+            sdata = wasaStringToRcl(theconfig, stemlang, u8, reason, 
 				    (const char *)prefs.autoSuffs.toUtf8());
         else
-            sdata = wasaStringToRcl(theconfig, u8, reason);
+            sdata = wasaStringToRcl(theconfig, stemlang, u8, reason);
 	if (sdata == 0) {
 	    QMessageBox::warning(0, "Recoll", tr("Bad query string") + ": " +
 				 QString::fromAscii(reason.c_str()));
-	    return;
+	    return false;
 	}
     } else {
-	sdata = new Rcl::SearchData(Rcl::SCLT_OR);
+	sdata = new Rcl::SearchData(Rcl::SCLT_OR, stemlang);
 	if (sdata == 0) {
 	    QMessageBox::warning(0, "Recoll", tr("Out of memory"));
-	    return;
+	    return false;
 	}
 	Rcl::SearchDataClause *clp = 0;
 	if (tp == SST_FNM) {
@@ -166,40 +271,21 @@ void SSearch::startSimpleSearch()
     }
 
     if (prefs.ssearchAutoPhrase && rcldb) {
-	string stemLang = (const char *)prefs.queryStemLang.toAscii();
-	if (stemLang == "ALL") {
-	    theconfig->getConfParam("indexstemminglanguages", stemLang);
-	}
-	sdata->setStemlang(stemLang);
 	sdata->maybeAddAutoPhrase(*rcldb, 
 				  prefs.ssearchAutoPhraseThreshPC / 100.0);
     }
-
-    // Search terms history
-
-    // We want to have the new text at the top and any older identical
-    // entry to be erased. There is no standard qt policy to do this ? 
-    // So do it by hand.
-    QString txt = queryText->currentText();
-    int index = queryText->findText(txt);
-    if (index >= 0) 
-	queryText->removeItem(index);
-    queryText->insertItem(0, txt);
-    queryText->setCurrentIndex(0);
-
-    // Save the current state of the listbox list to the prefs (will
-    // go to disk)
-    prefs.ssearchHistory.clear();
-    for (int index = 0; index < queryText->count(); index++) {
-	prefs.ssearchHistory.push_back(queryText->itemText(index));
+    if (maxexp != -1) {
+	sdata->setMaxExpand(maxexp);
     }
-
     RefCntr<Rcl::SearchData> rsdata(sdata);
-    emit startSearch(rsdata);
+    emit startSearch(rsdata, true);
+    return true;
 }
 
 void SSearch::setSearchString(const QString& txt)
 {
+    m_disableAutosearch = true;
+    m_stroketimeout->stop();
     queryText->setEditText(txt);
 }
 
@@ -234,15 +320,24 @@ void SSearch::addTerm(QString term)
     QString text = queryText->currentText();
     text += QString::fromLatin1(" ") + term;
     queryText->setEditText(text);
+    m_disableAutosearch = true;
+    m_stroketimeout->stop();
 }
 
 void SSearch::onWordReplace(const QString& o, const QString& n)
 {
+    LOGDEB(("SSearch::onWordReplace: o [%s] n [%s]\n",
+	    qs2utf8s(o).c_str(), qs2utf8s(n).c_str()));
     QString txt = queryText->currentText();
     QRegExp exp = QRegExp(QString("\\b") + o + QString("\\b"));
     exp.setCaseSensitivity(Qt::CaseInsensitive);
     txt.replace(exp, n);
     queryText->setEditText(txt);
+    m_disableAutosearch = true;
+    m_stroketimeout->stop();
+    Qt::KeyboardModifiers mods = QApplication::keyboardModifiers ();
+    if (mods == Qt::NoModifier)
+	startSimpleSearch();
 }
 
 void SSearch::setAnyTermMode()
@@ -250,9 +345,54 @@ void SSearch::setAnyTermMode()
     searchTypCMB->setCurrentIndex(SST_ANY);
 }
 
+// If text does not end with space, return last (partial) word and >0
+// else return -1
+int SSearch::partialWord(string& s)
+{
+    // Extract last word in text
+    QString txt = queryText->currentText();
+    int cs = txt.lastIndexOf(" ");
+    if (cs == -1)
+	cs = 0;
+    else
+	cs++;
+    if (txt.size() == 0 || cs == txt.size()) {
+	return -1;
+    }
+    s = qs2utf8s(txt.right(txt.size() - cs));
+    return cs;
+}
+
+// Create completion list for term by adding a joker at the end and calling
+// rcldb->termMatch().
+int SSearch::completionList(string s, QStringList& lst, int max)
+{
+    if (!rcldb)
+	return -1;
+    if (s.empty())
+	return 0;
+   // Query database for completions
+    s += "*";
+    Rcl::TermMatchResult tmres;
+    if (!rcldb->termMatch(Rcl::Db::ET_WILD, "", s, tmres, max) || 
+	tmres.entries.size() == 0) {
+	return 0;
+    }
+    for (vector<Rcl::TermMatchEntry>::iterator it = tmres.entries.begin(); 
+	 it != tmres.entries.end(); it++) {
+	lst.push_back(QString::fromUtf8(it->term.c_str()));
+    }
+    return lst.size();
+}
+
 // Complete last word in input by querying db for all possible terms.
 void SSearch::completion()
 {
+    LOGDEB(("SSearch::completion\n"));
+
+    m_disableAutosearch = true;
+    m_stroketimeout->stop();
+
     if (!rcldb)
 	return;
     if (searchTypCMB->currentIndex() == SST_FNM) {
@@ -260,64 +400,77 @@ void SSearch::completion()
 	QApplication::beep();
 	return;
     }
+
     // Extract last word in text
-    string txt = (const char *)queryText->currentText().toUtf8();
-    string::size_type cs = txt.find_last_of(" ");
-    if (cs == string::npos)
-	cs = 0;
-    else
-	cs++;
-    if (txt.size() == 0 || cs == txt.size()) {
+    string s;
+    int cs = partialWord(s);
+    if (cs < 0) {
 	QApplication::beep();
 	return;
     }
-    string s = txt.substr(cs) + "*";
-    LOGDEB(("Completing: [%s]\n", s.c_str()));
 
-    // Query database
-    const int max = 100;
-    Rcl::TermMatchResult tmres;
-    string stemLang = (const char *)prefs.queryStemLang.toAscii();
-    if (stemLang == "ALL") {
-	theconfig->getConfParam("indexstemminglanguages", stemLang);
-    }
-    if (!rcldb->termMatch(Rcl::Db::ET_WILD, stemLang, s, tmres, max) || 
-	tmres.entries.size() == 0) {
+    // Query database for completions
+    QStringList lst;
+    const int maxdpy = 80;
+    const int maxwalked = 10000;
+    if (completionList(s, lst, maxwalked) <= 0) {
 	QApplication::beep();
 	return;
     }
-    if (tmres.entries.size() == (unsigned int)max) {
-	QMessageBox::warning(0, "Recoll", tr("Too many completions"));
-	return;
+    if (lst.size() >= maxdpy) {
+	LOGDEB0(("SSearch::completion(): truncating list\n"));
+	lst = lst.mid(0, maxdpy);
+	lst.append("[...]");
     }
 
-    // If list from db is single word, insert it, else ask user to select
-    QString res;
-    bool ok = false;
-    if (tmres.entries.size() == 1) {
-	res = QString::fromUtf8(tmres.entries.begin()->term.c_str());
-	ok = true;
+    // If list from db is single word, insert it, else popup the listview
+    if (lst.size() == 1) {
+	QString txt = queryText->currentText();
+	txt.truncate(cs);
+	txt.append(lst[0]);
+	queryText->setEditText(txt);
     } else {
-	QStringList lst;
-	for (list<Rcl::TermMatchEntry>::iterator it = tmres.entries.begin(); 
-	     it != tmres.entries.end(); it++) {
-	    lst.push_back(QString::fromUtf8(it->term.c_str()));
-	}
-	res = QInputDialog::getItem (this, tr("Completions"), 
-				     tr("Select an item:"),
-				     lst, 0, false, &ok);
-    }
+	m_savedEditText = queryText->currentText();
+	m_displayingCompletions = true;
+	m_chosenCompletion.clear();
+	m_completedWordStart = cs;
 
-    // Insert result
-    if (ok) {
-	txt.erase(cs);
-	txt.append((const char *)res.toUtf8());
-	queryText->setEditText(QString::fromUtf8(txt.c_str()));
-    } else {
-	return;
+	queryText->clear();
+	queryText->addItems(lst);
+	queryText->showPopup();
+
+	connect(queryText, SIGNAL(activated(const QString&)), this,
+		SLOT(completionTermChosen(const QString&)));
     }
 }
 
+void SSearch::completionTermChosen(const QString& text)
+{
+    if (text != "[...]")
+	m_chosenCompletion = text;
+    else 
+	m_chosenCompletion.clear();
+}
+
+void SSearch::wrapupCompletion()
+{
+    LOGDEB(("SSearch::wrapupCompletion\n"));
+
+    queryText->clear();
+    queryText->addItems(prefs.ssearchHistory);
+    if (!m_chosenCompletion.isEmpty()) {
+	m_savedEditText.truncate(m_completedWordStart);
+	m_savedEditText.append(m_chosenCompletion);
+    }
+    queryText->setEditText(m_savedEditText);
+    m_disableAutosearch = true;
+    m_savedEditText.clear();
+    m_chosenCompletion.clear();
+    m_displayingCompletions = false;
+    disconnect(queryText, SIGNAL(activated(const QString&)), this,
+	       SLOT(completionTermChosen(const QString&)));
+}
+
 #undef SHOWEVENTS
 #if defined(SHOWEVENTS)
 const char *eventTypeToStr(int tp)
@@ -459,7 +612,7 @@ const char *eventTypeToStr(int tp)
 }
 #endif
 
-bool SSearch::eventFilter(QObject *, QEvent *event)
+bool SSearch::eventFilter(QObject *target, QEvent *event)
 {
 #if defined(SHOWEVENTS)
     if (event->type() == QEvent::Timer || 
@@ -467,10 +620,21 @@ bool SSearch::eventFilter(QObject *, QEvent *event)
 	event->type() == QEvent::Paint)
 	return false;
     LOGDEB2(("SSearch::eventFilter: target %p (%p) type %s\n", 
-	    target, queryText->lineEdit(),
-	    eventTypeToStr(event->type())));
+	     target, queryText->lineEdit(),
+	     eventTypeToStr(event->type())));
 #endif
 
+    if (target == queryText->view()) {
+	if (event->type() == QEvent::Hide) {
+	    // List was closed. If we were displaying completions, need
+	    // to reset state.
+	    if (m_displayingCompletions) {
+		QTimer::singleShot(0, this, SLOT(wrapupCompletion()));
+	    }
+	}
+	return false;
+    }
+
     if (event->type() == QEvent::KeyPress)  {
 	QKeyEvent *ke = (QKeyEvent *)event;
 	LOGDEB1(("SSearch::eventFilter: keyPress (m_escape %d) key %d\n", 
@@ -478,18 +642,31 @@ bool SSearch::eventFilter(QObject *, QEvent *event)
 	if (ke->key() == Qt::Key_Escape) {
 	    LOGDEB(("Escape\n"));
 	    m_escape = true;
+	    m_disableAutosearch = true;
+	    m_stroketimeout->stop();
 	    return true;
 	} else if (m_escape && ke->key() == Qt::Key_Space) {
 	    LOGDEB(("Escape space\n"));
 	    ke->accept();
 	    completion();
 	    m_escape = false;
+	    m_disableAutosearch = true;
+	    m_stroketimeout->stop();
 	    return true;
 	} else if (ke->key() == Qt::Key_Space) {
-	    if (prefs.autoSearchOnWS)
+	    if (prefs.ssearchOnWS)
 		startSimpleSearch();
+	} else {
+	    m_escape = false;
+	    m_keystroke = true;
+	    if (prefs.ssearchAsYouType) {
+		m_disableAutosearch = false;
+		QString qs = queryText->currentText();
+		LOGDEB0(("SSearch::eventFilter: start timer, qs [%s]\n", 
+			 qs2utf8s(qs).c_str()));
+		m_stroketimeout->start(strokeTimeoutMS);
+	    }
 	}
-	m_escape = false;
     }
     return false;
 }
