@@ -41,6 +41,7 @@
 #include <qtextedit.h>
 #include <qlist.h>
 #include <QTimer>
+#include <QListWidget>
 
 #include "recoll.h"
 #include "guiutils.h"
@@ -52,6 +53,7 @@
 #include "debuglog.h"
 #include "editdialog.h"
 #include "rclmain_w.h"
+#include "ptrans_w.h"
 
 void UIPrefsDialog::init()
 {
@@ -60,9 +62,16 @@ void UIPrefsDialog::init()
     connect(viewActionPB, SIGNAL(clicked()), this, SLOT(showViewAction()));
     connect(reslistFontPB, SIGNAL(clicked()), this, SLOT(showFontDialog()));
     connect(resetFontPB, SIGNAL(clicked()), this, SLOT(resetReslistFont()));
-    connect(stylesheetPB, SIGNAL(clicked()), this, SLOT(showStylesheetDialog()));
-    connect(resetSSPB, SIGNAL(clicked()), this, SLOT(resetStylesheet()));
 
+    connect(stylesheetPB, SIGNAL(clicked()),this, SLOT(showStylesheetDialog()));
+    connect(resetSSPB, SIGNAL(clicked()), this, SLOT(resetStylesheet()));
+    connect(snipCssPB, SIGNAL(clicked()),this, SLOT(showSnipCssDialog()));
+    connect(resetSnipCssPB, SIGNAL(clicked()), this, SLOT(resetSnipCss()));
+
+    connect(idxLV, SIGNAL(itemSelectionChanged()),
+	    this, SLOT(extradDbSelectChanged()));
+    connect(ptransPB, SIGNAL(clicked()),
+	    this, SLOT(extraDbEditPtrans()));
     connect(addExtraDbPB, SIGNAL(clicked()), 
 	    this, SLOT(addExtraDbPB_clicked()));
     connect(delExtraDbPB, SIGNAL(clicked()), 
@@ -79,8 +88,10 @@ void UIPrefsDialog::init()
     connect(buttonCancel, SIGNAL(clicked()), this, SLOT(reject()));
     connect(buildAbsCB, SIGNAL(toggled(bool)), 
 	    replAbsCB, SLOT(setEnabled(bool)));
-    connect(useDesktopOpenCB, SIGNAL(toggled(bool)), 
-	    viewActionPB, SLOT(setDisabled(bool)));
+    connect(ssAutoAllCB, SIGNAL(toggled(bool)), 
+	    ssAutoSpaceCB, SLOT(setDisabled(bool)));
+    connect(ssAutoAllCB, SIGNAL(toggled(bool)), 
+	    ssAutoSpaceCB, SLOT(setChecked(bool)));
 
     setFromPrefs();
 }
@@ -93,19 +104,28 @@ void UIPrefsDialog::setFromPrefs()
     collapseDupsCB->setChecked(prefs.collapseDuplicates);
     maxHLTSB->setValue(prefs.maxhltextmbs);
     catgToolBarCB->setChecked(prefs.catgToolBar);
-    autoSearchCB->setChecked(prefs.autoSearchOnWS);
+    ssAutoSpaceCB->setChecked(prefs.ssearchOnWS);
+    ssNoCompleteCB->setChecked(prefs.ssearchNoComplete);
+    ssAutoAllCB->setChecked(prefs.ssearchAsYouType);
     syntlenSB->setValue(prefs.syntAbsLen);
     syntctxSB->setValue(prefs.syntAbsCtx);
 
     initStartAdvCB->setChecked(prefs.startWithAdvSearchOpen);
 
-    // External editor. Can use desktop prefs or internal
-    useDesktopOpenCB->setChecked(prefs.useDesktopOpen);
-    viewActionPB->setEnabled(!prefs.useDesktopOpen);
-
     keepSortCB->setChecked(prefs.keepSort);
     previewHtmlCB->setChecked(prefs.previewHtml);
-    previewPlainPreCB->setChecked(prefs.previewPlainPre);
+    switch (prefs.previewPlainPre) {
+    case PrefsPack::PP_BR:
+	plainBRRB->setChecked(1);
+	break;
+    case PrefsPack::PP_PRE:
+	plainPRERB->setChecked(1);
+	break;
+    case PrefsPack::PP_PREWRAP:
+    default:
+	plainPREWRAPRB->setChecked(1);
+	break;
+    }
     // Query terms color
     qtermColorLE->setText(prefs.qtermcolor);
     // Abstract snippet separator string
@@ -125,14 +145,22 @@ void UIPrefsDialog::setFromPrefs()
     }
 
     // Style sheet
-    stylesheetFile = prefs.stylesheetFile;
-    if (stylesheetFile.isEmpty()) {
+    qssFile = prefs.qssFile;
+    if (qssFile.isEmpty()) {
 	stylesheetPB->setText(tr("Choose"));
     } else {
-	string nm = path_getsimple((const char *)stylesheetFile.toLocal8Bit());
+	string nm = path_getsimple((const char *)qssFile.toLocal8Bit());
 	stylesheetPB->setText(QString::fromLocal8Bit(nm.c_str()));
     }
 
+    snipCssFile = prefs.snipCssFile;
+    if (snipCssFile.isEmpty()) {
+	snipCssPB->setText(tr("Choose"));
+    } else {
+	string nm = path_getsimple((const char *)snipCssFile.toLocal8Bit());
+	snipCssPB->setText(QString::fromLocal8Bit(nm.c_str()));
+    }
+
     paraFormat = prefs.reslistformat;
     headerText = prefs.reslistheadertext;
 
@@ -140,13 +168,13 @@ void UIPrefsDialog::setFromPrefs()
     stemLangCMB->clear();
     stemLangCMB->addItem(g_stringNoStem);
     stemLangCMB->addItem(g_stringAllStem);
-    list<string> langs;
+    vector<string> langs;
     if (!getStemLangs(langs)) {
 	QMessageBox::warning(0, "Recoll", 
 			     tr("error retrieving stemming languages"));
     }
     int cur = prefs.queryStemLang == ""  ? 0 : 1;
-    for (list<string>::const_iterator it = langs.begin(); 
+    for (vector<string>::const_iterator it = langs.begin(); 
 	 it != langs.end(); it++) {
 	stemLangCMB->
 	    addItem(QString::fromAscii(it->c_str(), it->length()));
@@ -192,7 +220,10 @@ void UIPrefsDialog::setFromPrefs()
 
 void UIPrefsDialog::accept()
 {
-    prefs.autoSearchOnWS = autoSearchCB->isChecked();
+    prefs.ssearchOnWS = ssAutoSpaceCB->isChecked();
+    prefs.ssearchNoComplete = ssNoCompleteCB->isChecked();
+    prefs.ssearchAsYouType = ssAutoAllCB->isChecked();
+
     prefs.catgToolBar = catgToolBarCB->isChecked();
     prefs.respagesize = pageLenSB->value();
     prefs.collapseDuplicates = collapseDupsCB->isChecked();
@@ -205,8 +236,9 @@ void UIPrefsDialog::accept()
 
     prefs.reslistfontfamily = reslistFontFamily;
     prefs.reslistfontsize = reslistFontSize;
-    prefs.stylesheetFile = stylesheetFile;
+    prefs.qssFile = qssFile;
     QTimer::singleShot(0, m_mainWindow, SLOT(applyStyleSheet()));
+    prefs.snipCssFile = snipCssFile;
     prefs.reslistformat =  paraFormat;
     prefs.reslistheadertext =  headerText;
     if (prefs.reslistformat.trimmed().isEmpty()) {
@@ -230,10 +262,17 @@ void UIPrefsDialog::accept()
 	replAbsCB->isChecked();
 
     prefs.startWithAdvSearchOpen = initStartAdvCB->isChecked();
-    prefs.useDesktopOpen = useDesktopOpenCB->isChecked();
+
     prefs.keepSort = keepSortCB->isChecked();
     prefs.previewHtml = previewHtmlCB->isChecked();
-    prefs.previewPlainPre = previewPlainPreCB->isChecked();
+
+    if (plainBRRB->isChecked()) {
+	prefs.previewPlainPre = PrefsPack::PP_BR;
+    } else if (plainPRERB->isChecked()) {
+	prefs.previewPlainPre = PrefsPack::PP_PRE;
+    } else {
+	prefs.previewPlainPre = PrefsPack::PP_PREWRAP;
+    }
 
     prefs.syntAbsLen = syntlenSB->value();
     prefs.syntAbsCtx = syntctxSB->value();
@@ -265,6 +304,8 @@ void UIPrefsDialog::accept()
 void UIPrefsDialog::editParaFormat()
 {
     EditDialog dialog(this);
+    dialog.setWindowTitle(tr("Result list paragraph format "
+			     "(erase all to reset to default)"));
     dialog.plainTextEdit->setPlainText(paraFormat);
     int result = dialog.exec();
     if (result == QDialog::Accepted)
@@ -273,6 +314,7 @@ void UIPrefsDialog::editParaFormat()
 void UIPrefsDialog::editHeaderText()
 {
     EditDialog dialog(this);
+    dialog.setWindowTitle(tr("Result list header (default is empty)"));
     dialog.plainTextEdit->setPlainText(headerText);
     int result = dialog.exec();
     if (result == QDialog::Accepted)
@@ -334,16 +376,27 @@ void UIPrefsDialog::showFontDialog()
 
 void UIPrefsDialog::showStylesheetDialog()
 {
-    stylesheetFile = myGetFileName(false, "Select stylesheet file", true);
-    string nm = path_getsimple((const char *)stylesheetFile.toLocal8Bit());
+    qssFile = myGetFileName(false, "Select stylesheet file", true);
+    string nm = path_getsimple((const char *)qssFile.toLocal8Bit());
     stylesheetPB->setText(QString::fromLocal8Bit(nm.c_str()));
 }
 
 void UIPrefsDialog::resetStylesheet()
 {
-    stylesheetFile = "";
+    qssFile = "";
     stylesheetPB->setText(tr("Choose"));
 }
+void UIPrefsDialog::showSnipCssDialog()
+{
+    snipCssFile = myGetFileName(false, "Select snippets window CSS file", true);
+    string nm = path_getsimple((const char *)snipCssFile.toLocal8Bit());
+    snipCssPB->setText(QString::fromLocal8Bit(nm.c_str()));
+}
+void UIPrefsDialog::resetSnipCss()
+{
+    snipCssFile = "";
+    snipCssPB->setText(tr("Choose"));
+}
 
 void UIPrefsDialog::resetReslistFont()
 {
@@ -372,6 +425,33 @@ void UIPrefsDialog::showViewAction(const QString& mt)
 ////////////////////////////////////////////
 // External / extra search indexes setup
 
+void UIPrefsDialog::extradDbSelectChanged()
+{
+    if (idxLV->selectedItems().size() <= 1) 
+	ptransPB->setEnabled(true);
+    else
+	ptransPB->setEnabled(false);
+}
+
+void UIPrefsDialog::extraDbEditPtrans()
+{
+    string dbdir;
+    if (idxLV->selectedItems().size() == 0) {
+	dbdir = theconfig->getDbDir();
+    } else if (idxLV->selectedItems().size() == 1) {
+	QListWidgetItem *item = idxLV->selectedItems()[0];
+	QString qd = item->data(Qt::DisplayRole).toString();
+	dbdir = (const char *)qd.toLocal8Bit();
+    } else {
+	QMessageBox::warning(
+	    0, "Recoll", tr("At most one index should be selected"));
+	return;
+    }
+    dbdir = path_canon(dbdir);
+    EditTrans *etrans = new EditTrans(dbdir, this);
+    etrans->show();
+}
+
 void UIPrefsDialog::togExtraDbPB_clicked()
 {
     for (int i = 0; i < idxLV->count(); i++) {
@@ -409,6 +489,19 @@ void UIPrefsDialog::delExtraDbPB_clicked()
     }
 }
 
+static bool samedir(const string& dir1, const string& dir2)
+{
+    struct stat st1, st2;
+    if (stat(dir1.c_str(), &st1))
+	return false;
+    if (stat(dir2.c_str(), &st2))
+	return false;
+    if (st1.st_dev == st2.st_dev && st1.st_ino == st2.st_ino) {
+	return true;
+    }
+    return false;
+}
+
 /** 
  * Browse to add another index.
  * We do a textual comparison to check for duplicates, except for
@@ -417,56 +510,60 @@ void UIPrefsDialog::delExtraDbPB_clicked()
 void UIPrefsDialog::addExtraDbPB_clicked()
 {
     QString input = myGetFileName(true, 
-				  tr("Select xapian index directory "
-				     "(ie: /home/buddy/.recoll/xapiandb)"));
+				  tr("Select recoll config directory or "
+				     "xapian index directory "
+				     "(e.g.: /home/me/.recoll or "
+				     "/home/me/.recoll/xapiandb)"));
 
     if (input.isEmpty())
 	return;
-
     string dbdir = (const char *)input.toLocal8Bit();
+    if (access(path_cat(dbdir, "recoll.conf").c_str(), 0) == 0) {
+	// Chosen dir is config dir.
+	RclConfig conf(&dbdir);
+	dbdir = conf.getDbDir();
+	if (dbdir.empty()) {
+	    QMessageBox::warning(
+		0, "Recoll", tr("The selected directory looks like a Recoll "
+				"configuration directory but the configuration "
+				"could not be read"));
+	    return;
+	}
+    }
+
     LOGDEB(("ExtraDbDial: got: [%s]\n", dbdir.c_str()));
     path_catslash(dbdir);
-    if (!Rcl::Db::testDbDir(dbdir)) {
+    bool stripped;
+    if (!Rcl::Db::testDbDir(dbdir, &stripped)) {
 	QMessageBox::warning(0, "Recoll", 
         tr("The selected directory does not appear to be a Xapian index"));
 	return;
     }
-    struct stat st1, st2;
-    stat(dbdir.c_str(), &st1);
-    string rcldbdir = theconfig->getDbDir();
-    stat(rcldbdir.c_str(), &st2);
-    path_catslash(rcldbdir);
-
-    if (st1.st_dev == st2.st_dev && st1.st_ino == st2.st_ino) {
+    if (o_index_stripchars != stripped) {
+	QMessageBox::warning(0, "Recoll", 
+			     tr("Cant add index with different case/diacritics"
+				" stripping option"));
+	return;
+    }
+    if (samedir(dbdir, theconfig->getDbDir())) {
 	QMessageBox::warning(0, "Recoll", 
 			     tr("This is the main/local index!"));
 	return;
     }
 
-    // For some reason, finditem (which we used to use to detect duplicates 
-    // here) does not work anymore here: qt 4.6.3
-    QList<QListWidgetItem *>items =
-	idxLV->findItems (input, Qt::MatchFixedString|Qt::MatchCaseSensitive);
-    if (!items.empty()) {
-	    QMessageBox::warning(0, "Recoll", 
-		 tr("The selected directory is already in the index list"));
-	    return;
-    }
-#if 0
-    string nv = (const char *)input.toLocal8Bit();
-    QListViewItemIterator it(idxLV);
-    while (it.current()) {
-	QCheckListItem *item = (QCheckListItem *)it.current();
-	string ov = (const char *)item->text().toLocal8Bit();
-	if (!ov.compare(nv)) {
-	    QMessageBox::warning(0, "Recoll", 
-		 tr("The selected directory is already in the index list"));
+    for (int i = 0; i < idxLV->count(); i++) {
+	QListWidgetItem *item = idxLV->item(i);
+	string existingdir = (const char *)item->text().toLocal8Bit();
+	if (samedir(dbdir, existingdir)) {
+	    QMessageBox::warning(
+		0, "Recoll", tr("The selected directory is already in the "
+				"index list"));
 	    return;
 	}
-	++it;
     }
-#endif
-    QListWidgetItem *item = new QListWidgetItem(input, idxLV);
-    item->setCheckState(Qt::Unchecked);
+
+    QListWidgetItem *item = 
+	new QListWidgetItem(QString::fromLocal8Bit(dbdir.c_str()), idxLV);
+    item->setCheckState(Qt::Checked);
     idxLV->sortItems();
 }
