@@ -18,6 +18,7 @@
 #include "filtseq.h"
 #include "sortseq.h"
 #include "debuglog.h"
+#include "internfile.h"
 
 string DocSequence::o_sort_trans;
 string DocSequence::o_filt_trans;
@@ -35,6 +36,21 @@ int DocSequence::getSeqSlice(int offs, int cnt, vector<ResListEntry>& result)
     return ret;
 }
 
+bool DocSequence::getEnclosing(Rcl::Doc& doc, Rcl::Doc& pdoc) 
+{
+    Rcl::Db *db = getDb();
+    if (db == 0) {
+	LOGERR(("DocSequence::getEnclosing: no db\n"));
+	return false;
+    }
+    string udi;
+    if (!FileInterner::getEnclosingUDI(doc, udi))
+        return false;
+
+    bool dbret =  db->getDoc(udi, doc, pdoc);
+    return dbret && pdoc.pc != -1;
+}
+
 // Remove stacked modifying sources (sort, filter) until we get to a real one
 void DocSource::stripStack()
 {
@@ -107,3 +123,4 @@ bool DocSource::setSortSpec(const DocSeqSortSpec &s)
     buildStack();
     return true;
 }
+
