@@ -19,19 +19,19 @@
 #include <string>
 #include <list>
 #include <vector>
-#ifndef NO_NAMESPACES
-using std::string;
-using std::list;
-using std::vector;
-#endif
+
 
 #include "rcldoc.h"
 #include "refcntr.h"
+#include "hldata.h"
+
+// Need this for the "Snippet" class def.
+#include "rclquery.h"
 
 // A result list entry. 
 struct ResListEntry {
     Rcl::Doc doc;
-    string subHeader;
+    std::string subHeader;
 };
 
 /** Sort specification. */
@@ -40,7 +40,7 @@ class DocSeqSortSpec {
     DocSeqSortSpec() : desc(false) {}
     bool isNotNull() const {return !field.empty();}
     void reset() {field.erase();}
-    string field;
+    std::string field;
     bool   desc;
 };
 
@@ -50,12 +50,12 @@ class DocSeqFiltSpec {
  public:
     DocSeqFiltSpec() {}
     enum Crit {DSFS_MIMETYPE, DSFS_QLANG, DSFS_PASSALL};
-    void orCrit(Crit crit, const string& value) {
+    void orCrit(Crit crit, const std::string& value) {
 	crits.push_back(crit);
 	values.push_back(value);
     }
     std::vector<Crit> crits;
-    std::vector<string> values;
+    std::vector<std::string> values;
     void reset() {crits.clear(); values.clear();}
     bool isNotNull() const {return crits.size() != 0;}
 };
@@ -73,7 +73,7 @@ class DocSeqFiltSpec {
 */
 class DocSequence {
  public:
-    DocSequence(const string &t) : m_title(t) {}
+    DocSequence(const std::string &t) : m_title(t) {}
     virtual ~DocSequence() {}
 
     /** Get document at given rank. 
@@ -84,45 +84,69 @@ class DocSequence {
      *           inside history)
      * @return true if ok, false for error or end of data
      */
-    virtual bool getDoc(int num, Rcl::Doc &doc, string *sh = 0) = 0;
+    virtual bool getDoc(int num, Rcl::Doc &doc, std::string *sh = 0) = 0;
 
     /** Get next page of documents. This accumulates entries into the result
      *  list parameter (doesn't reset it). */
-    virtual int getSeqSlice(int offs, int cnt, vector<ResListEntry>& result);
+    virtual int getSeqSlice(int offs, int cnt, 
+			    std::vector<ResListEntry>& result);
 
     /** Get abstract for document. This is special because it may take time.
      *  The default is to return the input doc's abstract fields, but some 
      *  sequences can compute a better value (ie: docseqdb) */
-    virtual bool getAbstract(Rcl::Doc& doc, vector<string>& abs) {
+    virtual bool getAbstract(Rcl::Doc& doc, std::vector<std::string>& abs) {
 	abs.push_back(doc.meta[Rcl::Doc::keyabs]);
 	return true;
     }
-    virtual bool getEnclosing(Rcl::Doc&, Rcl::Doc&) = 0;
+    virtual bool getAbstract(Rcl::Doc& doc, 
+			     std::vector<Rcl::Snippet>& abs) 
+    {
+	abs.push_back(Rcl::Snippet(0, doc.meta[Rcl::Doc::keyabs]));
+	return true;
+    }
+    virtual int getFirstMatchPage(Rcl::Doc&, std::string&) 
+    {
+	return -1;
+    }
+    /** Get duplicates. */
+    virtual bool docDups(const Rcl::Doc&, std::vector<Rcl::Doc>&)
+    {
+	return false;
+    }
+
+    virtual bool getEnclosing(Rcl::Doc&, Rcl::Doc&);
 
     /** Get estimated total count in results */
     virtual int getResCnt() = 0;
 
     /** Get title for result list */
-    virtual string title() {return m_title;}
+    virtual std::string title() 
+    {
+	return m_title;
+    }
 
+    /** Can do snippets ? */
+    virtual bool snippetsCapable()
+    {
+	return false;
+    }
     /** Get description for underlying query */
-    virtual string getDescription() = 0;
+    virtual std::string getDescription() = 0;
 
     /** Get search terms (for highlighting abstracts). Some sequences
      * may have no associated search terms. Implement this for them. */
-    virtual bool getTerms(vector<string>& terms, 
-			  vector<vector<string> >& groups, 
-			  vector<int>& gslks) 
+    virtual void getTerms(HighlightData& hld)			  
     {
-	terms.clear(); groups.clear(); gslks.clear(); return true;
+	hld.clear();
     }
-    /** Get user-input terms (before stemming etc.) */
-    virtual void getUTerms(vector<string>& terms)
+    virtual std::list<std::string> expand(Rcl::Doc &) 
     {
-	terms.clear(); 
+	return std::list<std::string>();
+    }
+    virtual std::string getReason() 
+    {
+	return m_reason;
     }
-    virtual list<string> expand(Rcl::Doc &) {return list<string>();}
-
     /** Optional functionality. */
     virtual bool canFilter() {return false;}
     virtual bool canSort() {return false;}
@@ -130,16 +154,22 @@ class DocSequence {
     virtual bool setSortSpec(const DocSeqSortSpec &) {return false;}
     virtual RefCntr<DocSequence> getSourceSeq() {return RefCntr<DocSequence>();}
 
-    static void set_translations(const string& sort, const string& filt)
+    static void set_translations(const std::string& sort, const std::string& filt)
     {
 	o_sort_trans = sort;
 	o_filt_trans = filt;
     }
+
+    virtual Rcl::Db *getDb() = 0;
+
 protected:
-    static string o_sort_trans;
-    static string o_filt_trans;
+
+    static std::string o_sort_trans;
+    static std::string o_filt_trans;
+    std::string          m_reason;
+
  private:
-    string          m_title;
+    std::string          m_title;
 };
 
 /** A modifier has a child sequence which does the real work and does
@@ -152,25 +182,44 @@ public:
     {}
     virtual ~DocSeqModifier() {}
 
-    virtual bool getAbstract(Rcl::Doc& doc, vector<string>& abs) 
+    virtual bool getAbstract(Rcl::Doc& doc, std::vector<std::string>& abs) 
     {
 	if (m_seq.isNull())
 	    return false;
 	return m_seq->getAbstract(doc, abs);
     }
-    virtual string getDescription() 
+    virtual bool getAbstract(Rcl::Doc& doc, 
+			     std::vector<Rcl::Snippet>& abs) 
+    {
+	if (m_seq.isNull())
+	    return false;
+	return m_seq->getAbstract(doc, abs);
+    }
+    /** Get duplicates. */
+    virtual bool docDups(const Rcl::Doc& doc, std::vector<Rcl::Doc>& dups)
+    {
+	if (m_seq.isNull())
+	    return false;
+	return m_seq->docDups(doc, dups);
+    }
+
+    virtual bool snippetsCapable()
+    {
+	if (m_seq.isNull())
+	    return false;
+	return m_seq->snippetsCapable();
+    }
+    virtual std::string getDescription() 
     {
 	if (m_seq.isNull())
 	    return "";
 	return m_seq->getDescription();
     }
-    virtual bool getTerms(vector<string>& terms, 
-			  vector<vector<string> >& groups, 
-			  vector<int>& gslks) 
+    virtual void getTerms(HighlightData& hld)
     {
 	if (m_seq.isNull())
-	    return false;
-	return m_seq->getTerms(terms, groups, gslks);
+	    return;
+	m_seq->getTerms(hld);
     }
     virtual bool getEnclosing(Rcl::Doc& doc, Rcl::Doc& pdoc) 
     {
@@ -178,20 +227,35 @@ public:
 	    return false;
 	return m_seq->getEnclosing(doc, pdoc);
     }
-    virtual void getUTerms(vector<string>& terms)
+    virtual std::string getReason() 
     {
 	if (m_seq.isNull())
-	    return;
-	m_seq->getUTerms(terms);
+	    return string();
+	return m_seq->getReason();
+    }
+    virtual std::string title() 
+    {
+	return m_seq->title();
+    }
+    virtual RefCntr<DocSequence> getSourceSeq() 
+    {
+	return m_seq;
+    }
+
+    virtual Rcl::Db *getDb()
+    {
+	if (m_seq.isNull())
+	    return 0;
+	return m_seq->getDb();
     }
-    virtual string title() {return m_seq->title();}
-    virtual RefCntr<DocSequence> getSourceSeq() {return m_seq;}
 
 protected:
+
     RefCntr<DocSequence>    m_seq;
 };
 
 class RclConfig;
+
 // A DocSource can juggle docseqs of different kinds to implement
 // sorting and filtering in ways depending on the base seqs capabilities
 class DocSource : public DocSeqModifier {
@@ -203,7 +267,7 @@ public:
     virtual bool canSort() {return true;}
     virtual bool setFiltSpec(const DocSeqFiltSpec &);
     virtual bool setSortSpec(const DocSeqSortSpec &);
-    virtual bool getDoc(int num, Rcl::Doc &doc, string *sh = 0)
+    virtual bool getDoc(int num, Rcl::Doc &doc, std::string *sh = 0)
     {
 	if (m_seq.isNull())
 	    return false;
@@ -215,7 +279,7 @@ public:
 	    return 0;
 	return m_seq->getResCnt();
     }
-    virtual string title();
+    virtual std::string title();
 private:
     bool buildStack();
     void stripStack();
