@@ -30,12 +30,16 @@ class DocSequenceDb : public DocSequence {
     virtual ~DocSequenceDb();
     virtual bool getDoc(int num, Rcl::Doc &doc, string * = 0);
     virtual int getResCnt();
-    virtual bool getTerms(vector<string>& terms, 
-			  vector<vector<string> >& groups, 
-			  vector<int>& gslks);
-    virtual void getUTerms(vector<string>& terms);
+    virtual void getTerms(HighlightData& hld);
+
+    // Called to fill-up the snippets window. Ignoers
+    // buildabstract/replaceabstract and syntabslen
+    virtual bool getAbstract(Rcl::Doc &doc, vector<Rcl::Snippet>&);
+
     virtual bool getAbstract(Rcl::Doc &doc, vector<string>&);
-    virtual bool getEnclosing(Rcl::Doc& doc, Rcl::Doc& pdoc);
+    virtual int getFirstMatchPage(Rcl::Doc&, std::string& term);
+    virtual Rcl::Db *getDb();
+    virtual bool docDups(const Rcl::Doc& doc, std::vector<Rcl::Doc>& dups);
     virtual string getDescription();
     virtual list<string> expand(Rcl::Doc &doc);
     virtual bool canFilter() {return true;}
@@ -47,6 +51,11 @@ class DocSequenceDb : public DocSequence {
         m_queryBuildAbstract = qba;
         m_queryReplaceAbstract = qra;
     }
+
+    virtual bool snippetsCapable()
+    {
+	return true;
+    }
     virtual string title();
 
  private:
@@ -59,6 +68,7 @@ class DocSequenceDb : public DocSequence {
     bool                     m_isFiltered;
     bool                     m_isSorted;
     bool   m_needSetQuery; // search data changed, need to reapply before fetch
+    bool   m_lastSQStatus;
     bool setQuery();
 };
 
