@@ -48,7 +48,7 @@ class DocSequenceHistory : public DocSequence {
     virtual ~DocSequenceHistory() {}
 
     virtual bool getDoc(int num, Rcl::Doc &doc, string *sh = 0);
-    virtual bool getEnclosing(Rcl::Doc& doc, Rcl::Doc& pdoc);
+    virtual Rcl::Db *getDb();
     virtual int getResCnt();
     virtual string getDescription() {return m_description;}
     void setDescription(const string& desc) {m_description = desc;}
