@@ -24,8 +24,10 @@
 #include <time.h>
 
 #include <sstream>
+#include <list>
 using std::ostringstream;
 using std::endl;
+using std::list;
 
 #include "cstr.h"
 #include "reslistpager.h"
@@ -40,9 +42,14 @@ static const string cstr_hlfontcolor("<font color=\"blue\">");
 static const string cstr_hlendfont("</font>");
 class PlainToRichHtReslist : public PlainToRich {
 public:
-    virtual ~PlainToRichHtReslist() {}
-    virtual string startMatch() {return cstr_hlfontcolor;}
-    virtual string endMatch() {return cstr_hlendfont;}
+    virtual string startMatch(unsigned int) 
+    {
+	return cstr_hlfontcolor;
+    }
+    virtual string endMatch() 
+    {
+	return cstr_hlendfont;
+    }
 };
 static PlainToRichHtReslist g_hiliter;
 
@@ -105,6 +112,14 @@ void ResListPager::resultPageNext()
     m_resultsInCurrentPage = pagelen;
     m_respage = npage;
 }
+static string maybeEscapeHtml(const string& fld)
+{
+    if (fld.compare(0, cstr_fldhtm.size(), cstr_fldhtm))
+	return escapeHtml(fld);
+    else
+	return fld.substr(cstr_fldhtm.size());
+}
+
 
 void ResListPager::resultPageFor(int docnum)
 {
@@ -132,9 +147,8 @@ void ResListPager::resultPageFor(int docnum)
     m_respage = npage;
 }
 
-void ResListPager::displayDoc(RclConfig *config,
-			      int i, Rcl::Doc& doc, const HiliteData& hdata,
-			      const string& sh)
+void ResListPager::displayDoc(RclConfig *config, int i, Rcl::Doc& doc, 
+			      const HighlightData& hdata, const string& sh)
 {
     ostringstream chunk;
     int percent;
@@ -175,7 +189,7 @@ void ResListPager::displayDoc(RclConfig *config,
     datebuf[0] = 0;
     if (!doc.dmtime.empty() || !doc.fmtime.empty()) {
 	time_t mtime = doc.dmtime.empty() ?
-	    atol(doc.fmtime.c_str()) : atol(doc.dmtime.c_str());
+	    atoll(doc.fmtime.c_str()) : atoll(doc.dmtime.c_str());
 	struct tm *tm = localtime(&mtime);
 	strftime(datebuf, 99, dateFormat().c_str(), tm);
     }
@@ -183,9 +197,9 @@ void ResListPager::displayDoc(RclConfig *config,
     // Size information. We print both doc and file if they differ a lot
     off_t fsize = -1, dsize = -1;
     if (!doc.dbytes.empty())
-	dsize = atol(doc.dbytes.c_str());
+	dsize = atoll(doc.dbytes.c_str());
     if (!doc.fbytes.empty())
-	fsize = atol(doc.fbytes.c_str());
+	fsize = atoll(doc.fbytes.c_str());
     string sizebuf;
     if (dsize > 0) {
 	sizebuf = displayableBytes(dsize);
@@ -200,14 +214,22 @@ void ResListPager::displayDoc(RclConfig *config,
     if (needabstract && m_docSource.isNotNull()) {
 	vector<string> vabs;
 	m_docSource->getAbstract(doc, vabs);
+	m_hiliter->set_inputhtml(false);
 
 	for (vector<string>::const_iterator it = vabs.begin();
 	     it != vabs.end(); it++) {
 	    if (!it->empty()) {
 		// No need to call escapeHtml(), plaintorich handles it
 		list<string> lr;
-		m_hiliter->set_inputhtml(false);
-		m_hiliter->plaintorich(*it, lr, hdata);
+		// There may be data like page numbers before the snippet text.
+		// will be in brackets.
+		string::size_type bckt = it->find("]");
+		if (bckt == string::npos) {
+		    m_hiliter->plaintorich(*it, lr, hdata);
+		} else {
+		    m_hiliter->plaintorich(it->substr(bckt), lr, hdata);
+		    lr.front() = it->substr(0, bckt) + lr.front();
+		}
 		richabst += lr.front();
 		richabst += absSep();
 	    }
@@ -224,10 +246,25 @@ void ResListPager::displayDoc(RclConfig *config,
     string apptag;
     doc.getmeta(Rcl::Doc::keyapptg, &apptag);
 
-    if (!config->getMimeViewerDef(doc.mimetype, apptag).empty()) {
+    if (!config->getMimeViewerDef(doc.mimetype, apptag, false).empty()) {
 	linksbuf << "<a href=\"E" << docnumforlinks << "\">"  
 		 << trans("Open") << "</a>";
     }
+    ostringstream snipsbuf;
+    if (doc.haspages) {
+	snipsbuf << "<a href=\"A" << docnumforlinks << "\">" 
+		 << trans("Snippets") << "</a>&nbsp;&nbsp;";
+	linksbuf << "&nbsp;&nbsp;" << snipsbuf.str();
+    }
+
+    string collapscnt;
+    if (doc.getmeta(Rcl::Doc::keycc, &collapscnt) && !collapscnt.empty()) {
+	ostringstream collpsbuf;
+	int clc = atoi(collapscnt.c_str()) + 1;
+	collpsbuf << "<a href=\"D" << docnumforlinks << "\">" 
+		  << trans("Dups") << "(" << clc << ")" << "</a>&nbsp;&nbsp;";
+	linksbuf << "&nbsp;&nbsp;" << collpsbuf.str();
+    }
 
     // Build the result list paragraph:
 
@@ -238,24 +275,29 @@ void ResListPager::displayDoc(RclConfig *config,
 	chunk << "<p style='margin: 0px;padding: 0px;clear: both;'>";
 
     // Configurable stuff
-    map<string,string> subs;
+    map<string, string> subs;
     subs["A"] = !richabst.empty() ? richabst : "";
     subs["D"] = datebuf;
+    subs["E"] = snipsbuf.str();
     subs["I"] = iconurl;
     subs["i"] = doc.ipath;
     subs["K"] = !doc.meta[Rcl::Doc::keykw].empty() ? 
-	string("[") + escapeHtml(doc.meta[Rcl::Doc::keykw]) + "]" : "";
-    subs["L"] = linksbuf.rdbuf()->str();
+	string("[") + maybeEscapeHtml(doc.meta[Rcl::Doc::keykw]) + "]" : "";
+    subs["L"] = linksbuf.str();
     subs["N"] = numbuf;
     subs["M"] = doc.mimetype;
     subs["R"] = doc.meta[Rcl::Doc::keyrr];
     subs["S"] = sizebuf;
-    subs["T"] = escapeHtml(titleOrFilename);
-    subs["t"] = escapeHtml(doc.meta[Rcl::Doc::keytt]);
+    subs["T"] = maybeEscapeHtml(titleOrFilename);
+    subs["t"] = maybeEscapeHtml(doc.meta[Rcl::Doc::keytt]);
     subs["U"] = url;
 
-    // Let %(xx) access all metadata.
-    subs.insert(doc.meta.begin(), doc.meta.end());
+    // Let %(xx) access all metadata. HTML-neuter everything:
+    for (map<string,string>::iterator it = doc.meta.begin(); 
+	 it != doc.meta.end(); it++) {
+	if (!it->first.empty()) 
+	    subs[it->first] = maybeEscapeHtml(it->second);
+    }
 
     string formatted;
     pcSubst(parFormat(), formatted, subs);
@@ -271,6 +313,16 @@ void ResListPager::displayDoc(RclConfig *config,
     append(chunk.rdbuf()->str(), i, doc);
 }
 
+bool ResListPager::getDoc(int num, Rcl::Doc& doc)
+{
+    if (m_winfirst < 0 || m_respage.size() == 0)
+	return false;
+    if (num < m_winfirst || num >= m_winfirst + int(m_respage.size()))
+	return false;
+    doc = m_respage[num-m_winfirst].doc;
+    return true;
+}
+
 void ResListPager::displayPage(RclConfig *config)
 {
     LOGDEB(("ResListPager::displayPage\n"));
@@ -305,29 +357,43 @@ void ResListPager::displayPage(RclConfig *config)
 
     if (pageEmpty()) {
 	chunk << trans("<p><b>No results found</b><br>");
-        vector<string>uterms;
-        m_docSource->getUTerms(uterms);
-        if (!uterms.empty()) {
-            map<string, vector<string> > spellings;
-            suggest(uterms, spellings);
-            if (!spellings.empty()) {
-                chunk << 
-                 trans("<p><i>Alternate spellings (accents suppressed): </i>")
-		      << "<br /><blockquote>";
-
-		for (map<string, vector<string> >::const_iterator it0 =
-			 spellings.begin(); it0 != spellings.end(); it0++) {
-		    chunk << "<b>" << it0->first << "</b> : ";
-		    for (vector<string>::const_iterator it = 
-			     it0->second.begin();
-			 it != it0->second.end(); it++) {
-			chunk << *it << " ";
+	string reason = m_docSource->getReason();
+	if (!reason.empty()) {
+	    chunk << "<blockquote>" << escapeHtml(reason) << 
+		"</blockquote></p>";
+	} else {
+	    HighlightData hldata;
+	    m_docSource->getTerms(hldata);
+	    vector<string> uterms(hldata.uterms.begin(), hldata.uterms.end());
+	    if (!uterms.empty()) {
+		map<string, vector<string> > spellings;
+		suggest(uterms, spellings);
+		if (!spellings.empty()) {
+		    if (o_index_stripchars) {
+			chunk << 
+			    trans("<p><i>Alternate spellings (accents suppressed): </i>")
+			      << "<br /><blockquote>";
+		    } else {
+			chunk << 
+			    trans("<p><i>Alternate spellings: </i>")
+			      << "<br /><blockquote>";
+		    
 		    }
-		    chunk << "<br />";
+
+		    for (map<string, vector<string> >::const_iterator it0 =
+			     spellings.begin(); it0 != spellings.end(); it0++) {
+			chunk << "<b>" << it0->first << "</b> : ";
+			for (vector<string>::const_iterator it = 
+				 it0->second.begin();
+			     it != it0->second.end(); it++) {
+			    chunk << *it << " ";
+			}
+			chunk << "<br />";
+		    }
+		    chunk << "</blockquote></p>";
 		}
-                chunk << "</blockquote></p>";
-            }
-        }
+	    }
+	}
     } else {
 	unsigned int resCnt = m_docSource->getResCnt();
 	if (m_winfirst + m_respage.size() < resCnt) {
@@ -362,13 +428,13 @@ void ResListPager::displayPage(RclConfig *config)
     if (pageEmpty())
 	return;
 
-    HiliteData hdata;
-    m_docSource->getTerms(hdata.terms, hdata.groups, hdata.gslks);
+    HighlightData hdata;
+    m_docSource->getTerms(hdata);
 
     // Emit data for result entry paragraph. Do it in chunks that make sense
     // html-wise, else our client may get confused
     for (int i = 0; i < (int)m_respage.size(); i++) {
-	Rcl::Doc &doc(m_respage[i].doc);
+	Rcl::Doc& doc(m_respage[i].doc);
 	string& sh(m_respage[i].subHeader);
 	displayDoc(config, i, doc, hdata, sh);
     }
@@ -406,10 +472,10 @@ string ResListPager::prevUrl()
 
 string ResListPager::iconUrl(RclConfig *config, Rcl::Doc& doc)
 {
-    string iconurl;
-    config->getMimeIconName(doc.mimetype, &iconurl);
-    iconurl = cstr_fileu + iconurl;
-    return iconurl;
+    string apptag;
+    doc.getmeta(Rcl::Doc::keyapptg, &apptag);
+
+    return cstr_fileu + config->getMimeIconPath(doc.mimetype, apptag);
 }
 
 bool ResListPager::append(const string& data)
