@@ -32,7 +32,9 @@ using std::list;
 #include "refcntr.h"
 #include "textsplit.h"
 
-static Rcl::SearchData *wasaQueryToRcl(RclConfig *config, WasaQuery *wasa, 
+static Rcl::SearchData *wasaQueryToRcl(const RclConfig *config, 
+				       const string& stemlang,
+				       WasaQuery *wasa, 
 				       const string& autosuffs, string& reason)
 {
     if (wasa == 0) {
@@ -47,7 +49,7 @@ static Rcl::SearchData *wasaQueryToRcl(RclConfig *config, WasaQuery *wasa,
 
     Rcl::SearchData *sdata = new 
 	Rcl::SearchData(wasa->m_op == WasaQuery::OP_AND ? Rcl::SCLT_AND : 
-			Rcl::SCLT_OR);
+			Rcl::SCLT_OR, stemlang);
     LOGDEB2(("wasaQueryToRcl: %s chain\n", wasa->m_op == WasaQuery::OP_AND ? 
 	     "AND" : "OR"));
 
@@ -62,11 +64,13 @@ static Rcl::SearchData *wasaQueryToRcl(RclConfig *config, WasaQuery *wasa,
 	if (!stringicmp("mime", (*it)->m_fieldspec) ||
 	    !stringicmp("format", (*it)->m_fieldspec)) {
 	    if ((*it)->m_op == WasaQuery::OP_LEAF) {
-		sdata->addFiletype((*it)->m_value);
-	    } else if ((*it)->m_op == WasaQuery::OP_EXCL) {
-		sdata->remFiletype((*it)->m_value);
+		if ((*it)->m_exclude) {
+		    sdata->remFiletype((*it)->m_value);
+		} else {
+		    sdata->addFiletype((*it)->m_value);
+		}
 	    } else {
-		reason = "internal error: mime clause neither leaf not excl??";
+		reason = "internal error: mime clause not leaf??";
 		return 0;
 	    }
 	    continue;
@@ -76,21 +80,20 @@ static Rcl::SearchData *wasaQueryToRcl(RclConfig *config, WasaQuery *wasa,
 	// categories like "audio", "presentation", etc.
 	if (!stringicmp("rclcat", (*it)->m_fieldspec) ||
 	    !stringicmp("type", (*it)->m_fieldspec)) {
-	    if ((*it)->m_op != WasaQuery::OP_LEAF && 
-		(*it)->m_op != WasaQuery::OP_EXCL) {
-		reason = "internal error: rclcat/type clause neither leaf"
-		    "nor excl??";
+	    if ((*it)->m_op != WasaQuery::OP_LEAF) {
+		reason = "internal error: rclcat/type clause not leaf??";
 		return 0;
 	    }
-	    list<string> mtypes;
+	    vector<string> mtypes;
 	    if (config && config->getMimeCatTypes((*it)->m_value, mtypes)
 		&& !mtypes.empty()) {
-		for (list<string>::iterator mit = mtypes.begin();
+		for (vector<string>::iterator mit = mtypes.begin();
 		     mit != mtypes.end(); mit++) {
-		    if ((*it)->m_op == WasaQuery::OP_LEAF)
-			sdata->addFiletype(*mit);
-		    else
+		    if ((*it)->m_exclude) {
 			sdata->remFiletype(*mit);
+		    } else {
+			sdata->addFiletype(*mit);
+		    }
 		}
 	    } else {
 		reason = "Unknown rclcat/type value: no mime types found";
@@ -99,14 +102,6 @@ static Rcl::SearchData *wasaQueryToRcl(RclConfig *config, WasaQuery *wasa,
 	    continue;
 	}
 
-	// Filtering on location
-	if (!stringicmp("dir", (*it)->m_fieldspec)) {
-	    string dir = path_tildexpand((*it)->m_value);
-	    sdata->setTopdir(dir, (*it)->m_op == WasaQuery::OP_EXCL,
-			     (*it)->m_weight);
-	    continue;
-	} 
-
 	// Handle "date" spec
 	if (!stringicmp("date", (*it)->m_fieldspec)) {
 	    if ((*it)->m_op != WasaQuery::OP_LEAF) {
@@ -166,6 +161,10 @@ static Rcl::SearchData *wasaQueryToRcl(RclConfig *config, WasaQuery *wasa,
 	} 
 
 	// "Regular" processing follows:
+	unsigned int mods = (unsigned int)(*it)->m_modifiers;
+	LOGDEB0(("wasaQueryToRcl: clause modifiers 0x%x\n", mods));
+	nclause = 0;
+
 	switch ((*it)->m_op) {
 	case WasaQuery::OP_NULL:
 	case WasaQuery::OP_AND:
@@ -175,9 +174,9 @@ static Rcl::SearchData *wasaQueryToRcl(RclConfig *config, WasaQuery *wasa,
 	    continue;
 
 	case WasaQuery::OP_LEAF: {
-	    LOGDEB(("wasaQueryToRcl: leaf clause [%s]:[%s] slack %d\n", 
-		    (*it)->m_fieldspec.c_str(), (*it)->m_value.c_str(),
-		    (*it)->m_slack));
+	    LOGDEB0(("wasaQueryToRcl: leaf clause [%s:%s] slack %d excl %d\n", 
+		     (*it)->m_fieldspec.c_str(), (*it)->m_value.c_str(),
+		     (*it)->m_slack, (*it)->m_exclude));
 
             // Change terms found in the "autosuffs" list into "ext"
             // field queries
@@ -192,96 +191,79 @@ static Rcl::SearchData *wasaQueryToRcl(RclConfig *config, WasaQuery *wasa,
                 }
             }
 
-	    unsigned int mods = (unsigned int)(*it)->m_modifiers;
-
-	    // I'm not sure I understand the phrase/near detection
-	    // thereafter anymore, maybe it would be better to have an
-	    // explicit flag. Mods can only be set after a double
-	    // quote.
-	    if (TextSplit::hasVisibleWhite((*it)->m_value) || mods) {
-		Rcl::SClType tp = Rcl::SCLT_PHRASE;
-		if (mods & WasaQuery::WQM_PROX) {
-		    tp = Rcl::SCLT_NEAR;
-		}
-		nclause = new Rcl::SearchDataClauseDist(tp, (*it)->m_value,
-							(*it)->m_slack,
-							(*it)->m_fieldspec);
+	    if (!stringicmp("dir", (*it)->m_fieldspec)) {
+		// dir filtering special case
+		nclause = new Rcl::SearchDataClausePath((*it)->m_value, 
+							(*it)->m_exclude);
 	    } else {
-		nclause = new Rcl::SearchDataClauseSimple(Rcl::SCLT_AND, 
-							  (*it)->m_value, 
-							  (*it)->m_fieldspec);
+		if ((*it)->m_exclude && wasa->m_op != WasaQuery::OP_AND) {
+		    LOGERR(("wasaQueryToRcl: excl clause inside OR list!\n"));
+		    continue;
+		}
+		// I'm not sure I understand the phrase/near detection
+		// thereafter anymore, maybe it would be better to have an
+		// explicit flag. Mods can only be set after a double
+		// quote.
+		if (TextSplit::hasVisibleWhite((*it)->m_value) || mods) {
+
+		    Rcl::SClType tp = (mods & WasaQuery::WQM_PROX)  ?
+			Rcl::SCLT_NEAR :
+			Rcl::SCLT_PHRASE;
+		    nclause = new Rcl::SearchDataClauseDist(tp, (*it)->m_value,
+							    (*it)->m_slack,
+							    (*it)->m_fieldspec);
+		} else {
+		    Rcl::SClType tp = (*it)->m_exclude ? 
+			Rcl::SCLT_OR:
+			Rcl::SCLT_AND;
+		    nclause = 
+			new Rcl::SearchDataClauseSimple(tp, (*it)->m_value, 
+							(*it)->m_fieldspec);
+		}
+		nclause->setexclude((*it)->m_exclude);
 	    }
+
 	    if (nclause == 0) {
 		reason = "Out of memory";
 		LOGERR(("wasaQueryToRcl: out of memory\n"));
 		return 0;
 	    }
-	    if (mods & WasaQuery::WQM_NOSTEM) {
-		nclause->addModifier(Rcl::SearchDataClause::SDCM_NOSTEMMING);
-	    }
-	    if ((*it)->m_weight != 1.0)
-		nclause->setWeight((*it)->m_weight);
-	    sdata->addClause(nclause);
 	}
 	    break;
 	    
-	case WasaQuery::OP_EXCL:
-	    LOGDEB2(("wasaQueryToRcl: excl clause [%s]:[%s]\n", 
-                    (*it)->m_fieldspec.c_str(), (*it)->m_value.c_str()));
-	    if (wasa->m_op != WasaQuery::OP_AND) {
-		LOGERR(("wasaQueryToRcl: negative clause inside OR list!\n"));
-		continue;
-	    }
-	    // Note: have to add dquotes which will be translated to
-	    // phrase if there are several words in there. Not pretty
-	    // but should work. If there is actually a single
-	    // word, it will not be taken as a phrase, and
-	    // stem-expansion will work normally
-            // Have to do this because searchdata has nothing like and_not
-	    nclause = new Rcl::SearchDataClauseSimple(Rcl::SCLT_EXCL, 
-						      string("\"") + 
-						      (*it)->m_value + "\"",
-						      (*it)->m_fieldspec);
-	    
-	    if (nclause == 0) {
-		reason = "Out of memory";
-		LOGERR(("wasaQueryToRcl: out of memory\n"));
-		return 0;
-	    }
-	    if ((*it)->m_modifiers & WasaQuery::WQM_NOSTEM)
-		nclause->setModifiers(Rcl::SearchDataClause::SDCM_NOSTEMMING);
-	    if ((*it)->m_weight != 1.0)
-		nclause->setWeight((*it)->m_weight);
-	    sdata->addClause(nclause);
-	    break;
-
 	case WasaQuery::OP_OR:
 	    LOGDEB2(("wasaQueryToRcl: OR clause [%s]:[%s]\n", 
 		     (*it)->m_fieldspec.c_str(), (*it)->m_value.c_str()));
 	    // Create a subquery.
 	    Rcl::SearchData *sub = 
-		wasaQueryToRcl(config, *it, autosuffs, reason);
+		wasaQueryToRcl(config, stemlang, *it, autosuffs, reason);
 	    if (sub == 0) {
 		continue;
 	    }
 	    nclause = 
-		new Rcl::SearchDataClauseSub(Rcl::SCLT_SUB, 
-					     RefCntr<Rcl::SearchData>(sub));
+		new Rcl::SearchDataClauseSub(RefCntr<Rcl::SearchData>(sub));
 	    if (nclause == 0) {
 		LOGERR(("wasaQueryToRcl: out of memory\n"));
 		reason = "Out of memory";
 		return 0;
 	    }
-	    if ((*it)->m_modifiers & WasaQuery::WQM_NOSTEM)
-		nclause->setModifiers(Rcl::SearchDataClause::SDCM_NOSTEMMING);
-	    sdata->addClause(nclause);
 	}
+
+	if (mods & WasaQuery::WQM_NOSTEM)
+	    nclause->addModifier(Rcl::SearchDataClause::SDCM_NOSTEMMING);
+	if (mods & WasaQuery::WQM_DIACSENS)
+	    nclause->addModifier(Rcl::SearchDataClause::SDCM_DIACSENS);
+	if (mods & WasaQuery::WQM_CASESENS)
+	    nclause->addModifier(Rcl::SearchDataClause::SDCM_CASESENS);
+	if ((*it)->m_weight != 1.0)
+	    nclause->setWeight((*it)->m_weight);
+	sdata->addClause(nclause);
     }
 
     return sdata;
 }
 
-Rcl::SearchData *wasaStringToRcl(RclConfig *config, 
+Rcl::SearchData *wasaStringToRcl(const RclConfig *config, const string& stemlang,
 				 const string &qs, string &reason, 
                                  const string& autosuffs)
 {
@@ -289,5 +271,5 @@ Rcl::SearchData *wasaStringToRcl(RclConfig *config,
     WasaQuery *wq = parser.stringToQuery(qs, reason);
     if (wq == 0) 
 	return 0;
-    return wasaQueryToRcl(config, wq, autosuffs, reason);
+    return wasaQueryToRcl(config, stemlang, wq, autosuffs, reason);
 }
