@@ -19,9 +19,9 @@
 #include "autoconfig.h"
 
 #include <stdio.h>
-#include <assert.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <assert.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/uio.h>
@@ -188,15 +188,15 @@ public:
         UdiH h(udi);
 
         LOGDEB2(("Circache::khEnter: h %s offs %lu udi [%s]\n", 
-                h.asHexString().c_str(), (ULONG)ofs, udi.c_str()));
+		 h.asHexString().c_str(), (ULONG)ofs, udi.c_str()));
 
         pair<kh_type::iterator, kh_type::iterator> p = m_ofskh.equal_range(h);
 
         if (p.first != m_ofskh.end() && p.first->first == h) {
             for (kh_type::iterator it = p.first; it != p.second; it++) {
                 LOGDEB2(("Circache::khEnter: col h %s, ofs %lu\n", 
-                        it->first.asHexString().c_str(),
-                        (ULONG)it->second));
+			 it->first.asHexString().c_str(),
+			 (ULONG)it->second));
                 if (it->second == ofs) {
                     // (h,offs) already there. Happens
                     LOGDEB2(("Circache::khEnter: already there\n"));
@@ -217,7 +217,7 @@ public:
         }
     }
 
-    // Return list of candidate offsets for udi (possibly several
+    // Return vector of candidate offsets for udi (possibly several
     // because there may be hash collisions, and also multiple
     // instances).
     bool khFind(const string& udi, vector<off_t>& ofss)
@@ -227,7 +227,7 @@ public:
         UdiH h(udi);
 
         LOGDEB2(("Circache::khFind: h %s udi [%s]\n", 
-                h.asHexString().c_str(), udi.c_str()));
+		 h.asHexString().c_str(), udi.c_str()));
 
         pair<kh_type::iterator, kh_type::iterator> p = m_ofskh.equal_range(h);
 
@@ -262,10 +262,10 @@ public:
         }
         return true;
     }
-    // Clear entries for list of udi/offs
-    bool khClear(const list<pair<string, off_t> >& udis)
+    // Clear entries for vector of udi/offs
+    bool khClear(const vector<pair<string, off_t> >& udis)
     {
-        for (list<pair<string, off_t> >::const_iterator it = udis.begin(); 
+        for (vector<pair<string, off_t> >::const_iterator it = udis.begin(); 
              it != udis.end(); it++)
             khClear(*it);
         return true;
@@ -318,7 +318,10 @@ public:
 
     bool writefirstblock()
     {
-        assert(m_fd >= 0);
+	if (m_fd < 0) {
+	    m_reason << "writefirstblock: not open ";
+	    return false;
+	}
 
         ostringstream s;
         s << 
@@ -344,7 +347,10 @@ public:
 
     bool readfirstblock()
     {
-        assert(m_fd >= 0);
+	if (m_fd < 0) {
+	    m_reason << "readfirstblock: not open ";
+	    return false;
+	}
 
         char bf[CIRCACHE_FIRSTBLOCK_SIZE];
 
@@ -361,22 +367,22 @@ public:
             m_reason << "readfirstblock: conf get maxsize failed";
             return false;
         }
-        m_maxsize = atol(value.c_str());
+        m_maxsize = atoll(value.c_str());
         if (!conf.get("oheadoffs", value, cstr_null)) {
             m_reason << "readfirstblock: conf get oheadoffs failed";
             return false;
         }
-        m_oheadoffs = atol(value.c_str());
+        m_oheadoffs = atoll(value.c_str());
         if (!conf.get("nheadoffs", value, cstr_null)) {
             m_reason << "readfirstblock: conf get nheadoffs failed";
             return false;
         }
-        m_nheadoffs = atol(value.c_str());
+        m_nheadoffs = atoll(value.c_str());
         if (!conf.get("npadsize", value, cstr_null)) {
             m_reason << "readfirstblock: conf get npadsize failed";
             return false;
         }
-        m_npadsize = atol(value.c_str());
+        m_npadsize = atoll(value.c_str());
         if (!conf.get("unient", value, cstr_null)) {
             m_uniquentries = false;
         } else {
@@ -387,6 +393,10 @@ public:
 
     bool writeEntryHeader(off_t offset, const EntryHeaderData& d)
     {
+	if (m_fd < 0) {
+	    m_reason << "writeEntryHeader: not open ";
+	    return false;
+	}
         char bf[CIRCACHE_HEADER_SIZE];
         memset(bf, 0, CIRCACHE_HEADER_SIZE);
         snprintf(bf, CIRCACHE_HEADER_SIZE, 
@@ -405,7 +415,10 @@ public:
 
     CCScanHook::status readEntryHeader(off_t offset, EntryHeaderData& d)
     {
-        assert(m_fd >= 0);
+	if (m_fd < 0) {
+	    m_reason << "readEntryHeader: not open ";
+            return CCScanHook::Error;
+	}
 
         if (lseek(m_fd, offset, 0) != offset) {
             m_reason << "readEntryHeader: lseek(" << offset << 
@@ -438,7 +451,10 @@ public:
     CCScanHook::status scan(off_t startoffset, CCScanHook *user, 
                             bool fold = false)
     {
-        assert(m_fd >= 0);
+	if (m_fd < 0) {
+	    m_reason << "scan: not open ";
+            return CCScanHook::Error;
+	}
 
         off_t so0 = startoffset;
         bool already_folded = false;
@@ -598,31 +614,90 @@ string CirCache::getReason()
     return m_d ? m_d->m_reason.str() : "Not initialized";
 }
 
-bool CirCache::create(off_t m_maxsize, int flags)
+// A scan callback which just records the last header offset and
+// padsize seen. This is used with a scan(nofold) to find the last
+// physical record in the file
+class CCScanHookRecord : public  CCScanHook {
+public:
+    off_t headoffs;
+    off_t padsize;
+    CCScanHookRecord()
+	: headoffs(0), padsize(0)
+    {
+    }
+    virtual status takeone(off_t offs, const string& udi, 
+			   const EntryHeaderData& d)
+    {
+	headoffs = offs;
+	padsize = d.padsize;
+	LOGDEB2(("CCScanHookRecord::takeone: offs %lld padsize %lld\n", 
+		 headoffs, padsize));
+        return Continue;
+    }
+};
+
+bool CirCache::create(off_t maxsize, int flags)
 {
-    LOGDEB(("CirCache::create: [%s] flags 0x%x\n", m_dir.c_str(), flags));
-    assert(m_d != 0);
+    LOGDEB(("CirCache::create: [%s] maxsz %lld flags 0x%x\n", 
+	    m_dir.c_str(), maxsize, flags));
+    if (m_d == 0) {
+	LOGERR(("CirCache::create: null data\n"));
+	return false;
+    }
+
     struct stat st;
     if (stat(m_dir.c_str(), &st) < 0) {
-        if (mkdir(m_dir.c_str(), 0777) < 0) {
-            m_d->m_reason << "CirCache::create: mkdir(" << m_dir << 
-                ") failed" << " errno " << errno;
-            return false;
-        }
+	// Directory does not exist, create it
+	if (mkdir(m_dir.c_str(), 0777) < 0) {
+	    m_d->m_reason << "CirCache::create: mkdir(" << m_dir << 
+		") failed" << " errno " << errno;
+	    return false;
+	}
     } else {
-        if (!(flags & CC_CRTRUNCATE))
-            return open(CC_OPWRITE);
+	// If the file exists too, and truncate is not set, switch
+	// to open-mode. Still may need to update header params.
+	if (access(m_d->datafn(m_dir).c_str(), 0) >= 0 &&
+	    !(flags & CC_CRTRUNCATE)) {
+	    if (!open(CC_OPWRITE)) {
+		return false;
+	    }
+	    if (maxsize == m_d->m_maxsize &&
+		((flags & CC_CRUNIQUE) != 0) == m_d->m_uniquentries) {
+		LOGDEB(("Header unchanged, no rewrite\n"));
+		return true;
+	    }
+	    // If the new maxsize is bigger than current size, we need
+	    // to stop recycling if this is what we are doing.
+	    if (maxsize > m_d->m_maxsize && maxsize > st.st_size) {
+		// Scan the file to find the last physical record. The
+		// ohead is set at physical eof, and nhead is the last
+		// scanned record
+		CCScanHookRecord rec;
+		m_d->scan(CIRCACHE_FIRSTBLOCK_SIZE, &rec, false);
+		m_d->m_oheadoffs = lseek(m_d->m_fd, 0, SEEK_END);
+		m_d->m_nheadoffs = rec.headoffs;
+		m_d->m_npadsize = rec.padsize;
+	    }
+	    m_d->m_maxsize = maxsize;
+	    m_d->m_uniquentries = ((flags & CC_CRUNIQUE) != 0);
+	    LOGDEB(("CirCache::create: rewriting header with "
+		    "maxsize %lld oheadoffs %lld nheadoffs %lld "
+		    "npadsize %d unient %d\n",
+		    m_d->m_maxsize, m_d->m_oheadoffs, m_d->m_nheadoffs,
+		    m_d->m_npadsize, int(m_d->m_uniquentries)));
+	    return m_d->writefirstblock();
+	}
+	// Else fallthrough to create file
     }
 
     if ((m_d->m_fd = ::open(m_d->datafn(m_dir).c_str(), 
-                          O_CREAT | O_RDWR | O_TRUNC, 
-                          0666)) < 0) {
+			    O_CREAT | O_RDWR | O_TRUNC, 0666)) < 0) {
         m_d->m_reason << "CirCache::create: open/creat(" << 
             m_d->datafn(m_dir) << ") failed " << "errno " << errno;
-            return false;
+	return false;
     }
 
-    m_d->m_maxsize = m_maxsize;
+    m_d->m_maxsize = maxsize;
     m_d->m_oheadoffs = CIRCACHE_FIRSTBLOCK_SIZE;
     m_d->m_uniquentries = ((flags & CC_CRUNIQUE) != 0);
 
@@ -639,12 +714,16 @@ bool CirCache::create(off_t m_maxsize, int flags)
 
 bool CirCache::open(OpMode mode)
 {
-    assert(m_d != 0);
+    if (m_d == 0) {
+	LOGERR(("CirCache::open: null data\n"));
+	return false;
+    }
+
     if (m_d->m_fd >= 0)
         ::close(m_d->m_fd);
 
     if ((m_d->m_fd = ::open(m_d->datafn(m_dir).c_str(), 
-                          mode == CC_OPREAD ? O_RDONLY : O_RDWR)) < 0) {
+			    mode == CC_OPREAD ? O_RDONLY : O_RDWR)) < 0) {
         m_d->m_reason << "CirCache::open: open(" << m_d->datafn(m_dir) << 
             ") failed " << "errno " << errno;
         return false;
@@ -726,9 +805,8 @@ public:
 bool CirCache::get(const string& udi, string& dic, string& data, int instance)
 {
     Chrono chron;
-    assert(m_d != 0);
     if (m_d->m_fd < 0) {
-        m_d->m_reason << "CirCache::get: not open";
+        m_d->m_reason << "CirCache::get: no data or not open";
         return false;
     }
 
@@ -754,7 +832,7 @@ bool CirCache::get(const string& udi, string& dic, string& data, int instance)
                 if (!fudi.compare(udi)) {
                     // Found one, memorize offset. Done if instance
                     // matches, else go on. If instance is -1 need to
-                    // go to the end of the list anyway
+                    // go to the end anyway
                     d_good = d;
                     o_good = *it;
                     if (finst == instance) {
@@ -794,9 +872,12 @@ bool CirCache::get(const string& udi, string& dic, string& data, int instance)
 
 bool CirCache::erase(const string& udi)
 {
-    assert(m_d != 0);
+    if (m_d == 0) {
+        LOGERR(("CirCache::erase: null data\n"));
+        return false;
+    }
     if (m_d->m_fd < 0) {
-        m_d->m_reason << "CirCache::erase: not open";
+        m_d->m_reason << "CirCache::erase: no data or not open";
         return false;
     }
 
@@ -849,15 +930,15 @@ class CCScanHookSpacer : public  CCScanHook {
 public:
     UINT sizewanted;
     UINT sizeseen;
-    list<pair<string, off_t> > squashed_udis;
+    vector<pair<string, off_t> > squashed_udis;
     CCScanHookSpacer(int sz)
-        : sizewanted(sz), sizeseen(0) {assert(sz > 0);}
+    : sizewanted(sz), sizeseen(0) {assert(sz > 0);}
 
     virtual status takeone(off_t offs, const string& udi, 
                            const EntryHeaderData& d)
     {
         LOGDEB2(("Circache:ScanSpacer:off %u dcsz %u dtsz %u pdsz %u udi[%s]\n",
-                (UINT)offs, d.dicsize, d.datasize, d.padsize, udi.c_str()));
+		 (UINT)offs, d.dicsize, d.datasize, d.padsize, udi.c_str()));
         sizeseen += CIRCACHE_HEADER_SIZE + d.dicsize + d.datasize + d.padsize;
         squashed_udis.push_back(make_pair(udi, offs));
         if (sizeseen >= sizewanted)
@@ -869,9 +950,12 @@ public:
 bool CirCache::put(const string& udi, const ConfSimple *iconf, 
                    const string& data, unsigned int iflags)
 {
-    assert(m_d != 0);
+    if (m_d == 0) {
+        LOGERR(("CirCache::put: null data\n"));
+        return false;
+    }
     if (m_d->m_fd < 0) {
-        m_d->m_reason << "CirCache::put: not open";
+        m_d->m_reason << "CirCache::put: no data or not open";
         return false;
     }
 
@@ -940,7 +1024,10 @@ bool CirCache::put(const string& udi, const ConfSimple *iconf,
         if (m_d->readEntryHeader(m_d->m_nheadoffs, pd) != CCScanHook::Continue){
             return false;
         }
-        assert(int(pd.padsize) == m_d->m_npadsize);
+	if (int(pd.padsize) != m_d->m_npadsize) {
+	    m_d->m_reason << "CirCache::put: logic error: bad padsize ";
+	    return false;
+	}
         if (pd.dicsize == 0) {
             // erased entry. Also recover the header space, no need to rewrite
             // the header, we're going to write on it.
@@ -1000,7 +1087,7 @@ bool CirCache::put(const string& udi, const ConfSimple *iconf,
     char head[CIRCACHE_HEADER_SIZE];
     memset(head, 0, CIRCACHE_HEADER_SIZE);
     snprintf(head, CIRCACHE_HEADER_SIZE, 
-	    headerformat, dic.size(), datalen, npadsize, flags);
+	     headerformat, dic.size(), datalen, npadsize, flags);
     struct iovec vecs[3];
     vecs[0].iov_base = head;
     vecs[0].iov_len = CIRCACHE_HEADER_SIZE;
@@ -1033,7 +1120,10 @@ bool CirCache::put(const string& udi, const ConfSimple *iconf,
 
 bool CirCache::rewind(bool& eof)
 {
-    assert(m_d != 0);
+    if (m_d == 0) {
+        LOGERR(("CirCache::rewind: null data\n"));
+        return false;
+    }
 
     eof = false;
 
@@ -1054,7 +1144,10 @@ bool CirCache::rewind(bool& eof)
 
 bool CirCache::next(bool& eof)
 {
-    assert(m_d != 0);
+    if (m_d == 0) {
+        LOGERR(("CirCache::next: null data\n"));
+        return false;
+    }
 
     eof = false;
 
@@ -1087,7 +1180,10 @@ bool CirCache::next(bool& eof)
 
 bool CirCache::getCurrentUdi(string& udi)
 {
-    assert(m_d != 0);
+    if (m_d == 0) {
+        LOGERR(("CirCache::getCurrentUdi: null data\n"));
+        return false;
+    }
     
     if (!m_d->readHUdi(m_d->m_itoffs, m_d->m_ithd, udi))
         return false;
@@ -1096,7 +1192,10 @@ bool CirCache::getCurrentUdi(string& udi)
 
 bool CirCache::getCurrent(string& udi, string& dic, string& data)
 {
-    assert(m_d != 0);
+    if (m_d == 0) {
+        LOGERR(("CirCache::getCurrent: null data\n"));
+        return false;
+    }
     if (!m_d->readDicData(m_d->m_itoffs, m_d->m_ithd, dic, &data))
         return false;
 
@@ -1158,7 +1257,7 @@ static bool inflateToDynBuf(void* inp, UINT inlen, void **outpp, UINT *outlenp)
                  d_stream.avail_out, d_stream.total_out));
         if (d_stream.avail_out == 0) {
             if ((outp = (char*)allocmem(outp, inlen, &alloc, 
-                                                 imul, mxinc)) == 0) {
+					imul, mxinc)) == 0) {
                 LOGERR(("Inflate: out of memory, current alloc %d\n", 
                         alloc*inlen));
                 inflateEnd(&d_stream);
@@ -1212,13 +1311,13 @@ using namespace std;
 static char *thisprog;
 
 static char usage [] =
-" -c [-u] <dirname> : create\n"
-" -p <dirname> <apath> [apath ...] : put files\n"
-" -d <dirname> : dump\n"
-" -g [-i instance] [-D] <dirname> <udi>: get\n"
-"   -D: also dump data\n"
-" -e <dirname> <udi> : erase\n"
-;
+    " -c [-u] <dirname> : create\n"
+    " -p <dirname> <apath> [apath ...] : put files\n"
+    " -d <dirname> : dump\n"
+    " -g [-i instance] [-D] <dirname> <udi>: get\n"
+    "   -D: also dump data\n"
+    " -e <dirname> <udi> : erase\n"
+    ;
 static void
 Usage(FILE *fp = stderr)
 {
@@ -1239,121 +1338,121 @@ static int     op_flags;
 
 int main(int argc, char **argv)
 {
-  int instance = -1;
+    int instance = -1;
 
-  thisprog = argv[0];
-  argc--; argv++;
+    thisprog = argv[0];
+    argc--; argv++;
 
-  while (argc > 0 && **argv == '-') {
-    (*argv)++;
-    if (!(**argv))
-      /* Cas du "adb - core" */
-      Usage();
-    while (**argv)
-      switch (*(*argv)++) {
-      case 'c':	op_flags |= OPT_c; break;
-      case 'e':	op_flags |= OPT_e; break;
-      case 'p':	op_flags |= OPT_p; break;
-      case 'g':	op_flags |= OPT_g; break;
-      case 'd':	op_flags |= OPT_d; break;
-      case 'D':	op_flags |= OPT_D; break;
-      case 'u':	op_flags |= OPT_u; break;
-      case 'i':	op_flags |= OPT_i; if (argc < 2)  Usage();
-	if ((sscanf(*(++argv), "%d", &instance)) != 1) 
-	  Usage(); 
-	argc--; 
-	goto b1;
-      default: Usage();	break;
-      }
-  b1: argc--; argv++;
-  }
+    while (argc > 0 && **argv == '-') {
+	(*argv)++;
+	if (!(**argv))
+	    /* Cas du "adb - core" */
+	    Usage();
+	while (**argv)
+	    switch (*(*argv)++) {
+	    case 'c':	op_flags |= OPT_c; break;
+	    case 'e':	op_flags |= OPT_e; break;
+	    case 'p':	op_flags |= OPT_p; break;
+	    case 'g':	op_flags |= OPT_g; break;
+	    case 'd':	op_flags |= OPT_d; break;
+	    case 'D':	op_flags |= OPT_D; break;
+	    case 'u':	op_flags |= OPT_u; break;
+	    case 'i':	op_flags |= OPT_i; if (argc < 2)  Usage();
+		if ((sscanf(*(++argv), "%d", &instance)) != 1) 
+		    Usage(); 
+		argc--; 
+		goto b1;
+	    default: Usage();	break;
+	    }
+    b1: argc--; argv++;
+    }
 
-  DebugLog::getdbl()->setloglevel(DEBDEB1);
-  DebugLog::setfilename("stderr");
+    DebugLog::getdbl()->setloglevel(DEBDEB1);
+    DebugLog::setfilename("stderr");
 
-  if (argc < 1)
-    Usage();
-  string dir = *argv++;argc--;
+    if (argc < 1)
+	Usage();
+    string dir = *argv++;argc--;
 
-  CirCache cc(dir);
+    CirCache cc(dir);
 
-  if (op_flags & OPT_c) {
-      int flags = 0;
-      if (op_flags & OPT_u)
-          flags |= CirCache::CC_CRUNIQUE;
-      if (!cc.create(100*1024, flags)) {
-          cerr << "Create failed:" << cc.getReason() << endl;
-          exit(1);
-      }
-  } else if (op_flags & OPT_p) {
-      if (argc < 1)
-          Usage();
-      if (!cc.open(CirCache::CC_OPWRITE)) {
-          cerr << "Open failed: " << cc.getReason() << endl;
-          exit(1);
-      }
-      while (argc) {
-          string fn = *argv++;argc--;
-          char dic[1000];
-          string data, reason;
-          if (!file_to_string(fn, data, &reason)) {
-              cerr << "File_to_string: " << reason << endl;
-              exit(1);
-          }
-          string udi;
-          make_udi(fn, "", udi);
-          sprintf(dic, "#whatever...\nmimetype = text/plain\nudi=%s\n", 
-                  udi.c_str());
-          string sdic;
-          sdic.assign(dic, strlen(dic));
-          ConfSimple conf(sdic);
+    if (op_flags & OPT_c) {
+	int flags = 0;
+	if (op_flags & OPT_u)
+	    flags |= CirCache::CC_CRUNIQUE;
+	if (!cc.create(100*1024, flags)) {
+	    cerr << "Create failed:" << cc.getReason() << endl;
+	    exit(1);
+	}
+    } else if (op_flags & OPT_p) {
+	if (argc < 1)
+	    Usage();
+	if (!cc.open(CirCache::CC_OPWRITE)) {
+	    cerr << "Open failed: " << cc.getReason() << endl;
+	    exit(1);
+	}
+	while (argc) {
+	    string fn = *argv++;argc--;
+	    char dic[1000];
+	    string data, reason;
+	    if (!file_to_string(fn, data, &reason)) {
+		cerr << "File_to_string: " << reason << endl;
+		exit(1);
+	    }
+	    string udi;
+	    make_udi(fn, "", udi);
+	    sprintf(dic, "#whatever...\nmimetype = text/plain\nudi=%s\n", 
+		    udi.c_str());
+	    string sdic;
+	    sdic.assign(dic, strlen(dic));
+	    ConfSimple conf(sdic);
    
-          if (!cc.put(udi, &conf, data, 0)) {
-              cerr << "Put failed: " << cc.getReason() << endl;
-              cerr << "conf: ["; conf.write(cerr); cerr << "]" << endl;
-              exit(1);
-          }
-      }
-      cc.open(CirCache::CC_OPREAD);
-  } else if (op_flags & OPT_g) {
-      if (!cc.open(CirCache::CC_OPREAD)) {
-          cerr << "Open failed: " << cc.getReason() << endl;
-          exit(1);
-      }
-      while (argc) {
-          string udi = *argv++;argc--;
-          string dic, data;
-          if (!cc.get(udi, dic, data, instance)) {
-              cerr << "Get failed: " << cc.getReason() << endl;
-              exit(1);
-          }
-          cout << "Dict: [" << dic << "]" << endl;
-          if (op_flags & OPT_D)
-              cout << "Data: [" << data << "]" << endl;
-      }
-  } else if (op_flags & OPT_e) {
-      if (!cc.open(CirCache::CC_OPWRITE)) {
-          cerr << "Open failed: " << cc.getReason() << endl;
-          exit(1);
-      }
-      while (argc) {
-          string udi = *argv++;argc--;
-          string dic, data;
-          if (!cc.erase(udi)) {
-              cerr << "Erase failed: " << cc.getReason() << endl;
-              exit(1);
-          }
-      }
-  } else if (op_flags & OPT_d) {
-      if (!cc.open(CirCache::CC_OPREAD)) {
-          cerr << "Open failed: " << cc.getReason() << endl;
-          exit(1);
-      }
-      cc.dump();
-  } else
-      Usage();
+	    if (!cc.put(udi, &conf, data, 0)) {
+		cerr << "Put failed: " << cc.getReason() << endl;
+		cerr << "conf: ["; conf.write(cerr); cerr << "]" << endl;
+		exit(1);
+	    }
+	}
+	cc.open(CirCache::CC_OPREAD);
+    } else if (op_flags & OPT_g) {
+	if (!cc.open(CirCache::CC_OPREAD)) {
+	    cerr << "Open failed: " << cc.getReason() << endl;
+	    exit(1);
+	}
+	while (argc) {
+	    string udi = *argv++;argc--;
+	    string dic, data;
+	    if (!cc.get(udi, dic, data, instance)) {
+		cerr << "Get failed: " << cc.getReason() << endl;
+		exit(1);
+	    }
+	    cout << "Dict: [" << dic << "]" << endl;
+	    if (op_flags & OPT_D)
+		cout << "Data: [" << data << "]" << endl;
+	}
+    } else if (op_flags & OPT_e) {
+	if (!cc.open(CirCache::CC_OPWRITE)) {
+	    cerr << "Open failed: " << cc.getReason() << endl;
+	    exit(1);
+	}
+	while (argc) {
+	    string udi = *argv++;argc--;
+	    string dic, data;
+	    if (!cc.erase(udi)) {
+		cerr << "Erase failed: " << cc.getReason() << endl;
+		exit(1);
+	    }
+	}
+    } else if (op_flags & OPT_d) {
+	if (!cc.open(CirCache::CC_OPREAD)) {
+	    cerr << "Open failed: " << cc.getReason() << endl;
+	    exit(1);
+	}
+	cc.dump();
+    } else
+	Usage();
 
-  exit(0);
+    exit(0);
 }
 
 #endif
