@@ -27,16 +27,13 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <signal.h>
+#include <time.h>
 
-#if !defined(PUTENV_ARG_CONST)
-#include <string.h>
-#endif
-
-#include <list>
 #include <vector>
 #include <string>
 #include <sstream>
 #include <iostream>
+using namespace std;
 
 #include "execmd.h"
 #include "pathut.h"
@@ -44,22 +41,13 @@
 #include "smallut.h"
 #include "netcon.h"
 #include "closefrom.h"
-#include "ptmutex.h"
 
-#ifndef NO_NAMESPACES
-using namespace std;
-#endif /* NO_NAMESPACES */
+extern char **environ;
 
-#ifndef MAX
-#define MAX(A,B) ((A) > (B) ? (A) : (B))
-#endif
-#ifndef MIN
-#define MIN(A,B) ((A) < (B) ? (A) : (B))
-#endif
+bool ExecCmd::o_useVfork = false;
 
 /* From FreeBSD's which command */
-static bool
-exec_is_there(const char *candidate)
+static bool exec_is_there(const char *candidate)
 {
     struct stat fin;
 
@@ -115,6 +103,20 @@ void  ExecCmd::putenv(const string &ea)
     m_env.push_back(ea);
 }
 
+void  ExecCmd::putenv(const string &name, const string& value)
+{
+    string ea = name + "=" + value;
+    putenv(ea);
+}
+
+static void msleep(int millis)
+{
+    struct timespec spec;
+    spec.tv_sec = millis / 1000;
+    spec.tv_nsec = (millis % 1000) * 1000000;
+    nanosleep(&spec, 0);
+}
+
 /** A resource manager to ensure that execcmd cleans up if an exception is 
  *  raised in the callback, or at different places on errors occurring
  *  during method executions */
@@ -136,17 +138,22 @@ public:
 	    close(m_parent->m_pipeout[0]);
 	if (m_parent->m_pipeout[1] >= 0)
 	    close(m_parent->m_pipeout[1]);
-	int status;
-	if (m_parent->m_pid > 0) {
-            pid_t grp = getpgid(m_parent->m_pid);
+
+	// It's apparently possible for m_pid to be > 0 and getpgid to fail. In
+	// this case, we have to conclude that the child process does 
+	// not exist. Not too sure what causes this, but the previous code
+	// definitely tried to call killpg(-1,) from time to time.
+	pid_t grp;
+	if (m_parent->m_pid > 0 && (grp = getpgid(m_parent->m_pid)) > 0) {
 	    LOGDEB(("ExecCmd: killpg(%d, SIGTERM)\n", grp));
             int ret = killpg(grp, SIGTERM);
 	    if (ret == 0) {
 		for (int i = 0; i < 3; i++) {
+		    msleep(i == 0 ? 5 : (i == 1 ? 100 : 2000));
+		    int status;
 		    (void)waitpid(m_parent->m_pid, &status, WNOHANG);
 		    if (kill(m_parent->m_pid, 0) != 0)
 			break;
-		    sleep(1);
 		    if (i == 2) {
 			LOGDEB(("ExecCmd: killpg(%d, SIGKILL)\n", grp));
 			killpg(grp, SIGKILL);
@@ -173,12 +180,86 @@ ExecCmd::~ExecCmd()
     ExecCmdRsrc(this);
 }
 
-int ExecCmd::startExec(const string &cmd, const list<string>& args,
+// In child process. Set up pipes and exec command. 
+// This must not return. _exit() on error.
+// *** This can be called after a vfork, so no modification of the
+//     process memory at all is allowed ***
+// The LOGXX calls should not be there, but they occur only after "impossible"
+// errors, which we would most definitely want to have a hint about
+inline void ExecCmd::dochild(const string &cmd, const char **argv,
+			     const char **envv,
+			     bool has_input, bool has_output)
+{
+    // Start our own process group
+    if (setpgid(0, getpid())) {
+	LOGINFO(("ExecCmd::dochild: setpgid(0, %d) failed: errno %d\n",
+		 getpid(), errno));
+    }
+
+    // Restore SIGTERM to default. Really, signal handling should be
+    // specified when creating the execmd. Help Recoll get rid of its
+    // filter children though. To be fixed one day... Not sure that
+    // all of this is needed. But an ignored sigterm and the masks are
+    // normally inherited.
+    if (signal(SIGTERM, SIG_DFL) == SIG_ERR) {
+	LOGERR(("ExecCmd::dochild: signal() failed, errno %d\n", errno));
+    }
+    sigset_t sset;
+    sigfillset(&sset);
+    pthread_sigmask(SIG_UNBLOCK, &sset, 0);
+    sigprocmask(SIG_UNBLOCK, &sset, 0);
+
+    if (has_input) {
+	close(m_pipein[1]);
+	if (m_pipein[0] != 0) {
+	    dup2(m_pipein[0], 0);
+	    close(m_pipein[0]);
+	}
+    }
+    if (has_output) {
+	close(m_pipeout[0]);
+	if (m_pipeout[1] != 1) {
+	    if (dup2(m_pipeout[1], 1) < 0) {
+		LOGERR(("ExecCmd::doexec: dup2(2) failed. errno %d\n", errno));
+	    }
+	    if (close(m_pipeout[1]) < 0) {
+		LOGERR(("ExecCmd::doexec: close(2) failed. errno %d\n", errno));
+	    }
+	}
+    }
+    // Do we need to redirect stderr ?
+    if (!m_stderrFile.empty()) {
+	int fd = open(m_stderrFile.c_str(), O_WRONLY|O_CREAT
+#ifdef O_APPEND
+		      |O_APPEND
+#endif
+		      , 0600);
+	if (fd < 0) {
+	    close(2);
+	} else {
+	    if (fd != 2) {
+		dup2(fd, 2);
+	    }
+	    lseek(2, 0, 2);
+	}
+    }
+
+    // Close all descriptors except 0,1,2
+    libclf_closefrom(3);
+
+    execve(cmd.c_str(), (char *const*)argv, (char *const*)envv);
+    // Hu ho
+    LOGERR(("ExecCmd::doexec: execve(%s) failed. errno %d\n", cmd.c_str(),
+	    errno));
+    _exit(127);
+}
+
+int ExecCmd::startExec(const string &cmd, const vector<string>& args,
 		       bool has_input, bool has_output)
 {
     { // Debug and logging
 	string command = cmd + " ";
-	for (list<string>::const_iterator it = args.begin();it != args.end();
+	for (vector<string>::const_iterator it = args.begin();it != args.end();
 	     it++) {
 	    command += "{" + *it + "} ";
 	}
@@ -198,19 +279,76 @@ int ExecCmd::startExec(const string &cmd, const list<string>& args,
 	return -1;
     }
 
-    m_pid = fork();
+
+//////////// vfork setup section
+    // We do here things that we could/should do after a fork(), but
+    // not a vfork(). Does no harm to do it here in both cases, except
+    // that it needs cleanup (as compared to doing it just before
+    // exec()).
+
+    // Allocate arg vector (2 more for arg0 + final 0)
+    typedef const char *Ccharp;
+    Ccharp *argv;
+    argv = (Ccharp *)malloc((args.size()+2) * sizeof(char *));
+    if (argv == 0) {
+	LOGERR(("ExecCmd::doexec: malloc() failed. errno %d\n",	errno));
+	exit(1);
+    }
+    // Fill up argv
+    argv[0] = cmd.c_str();
+    int i = 1;
+    vector<string>::const_iterator it;
+    for (it = args.begin(); it != args.end(); it++) {
+	argv[i++] = it->c_str();
+    }
+    argv[i] = 0;
+
+    Ccharp *envv;
+    int envsize;
+    for (envsize = 0; ; envsize++) 
+	if (environ[envsize] == 0)
+	    break;
+    envv = (Ccharp *)malloc((envsize + m_env.size() + 2) * sizeof(char *));
+    int eidx;
+    for (eidx = 0; eidx < envsize; eidx++)
+	envv[eidx] = environ[eidx];
+    for (vector<string>::const_iterator it = m_env.begin(); 
+	 it != m_env.end(); it++) {
+	envv[eidx++] = it->c_str();
+    }
+    envv[eidx] = 0;
+
+    // As we are going to use execve, not execvp, do the PATH
+    // thing. If the command is not found, exe will be empty and the
+    // exec will fail, which is what we want.
+    string exe;
+    which(cmd, exe);
+////////////////////////////////
+
+    if (o_useVfork) {
+	m_pid = vfork();
+    } else {
+	m_pid = fork();
+    }
     if (m_pid < 0) {
 	LOGERR(("ExecCmd::startExec: fork(2) failed. errno %d\n", errno));
 	return -1;
     }
     if (m_pid == 0) {
-	e.inactivate(); // needed ?
-	dochild(cmd, args, has_input, has_output);
+	// e.inactivate() is not needed. As we do not return, the call
+	// stack won't be unwound and destructors of local objects
+	// won't be called.
+	dochild(exe, argv, envv, has_input, has_output);
 	// dochild does not return. Just in case...
 	_exit(1);
     }
 
     // Father process
+////////////////////
+    // Vfork cleanup section
+    free(argv);
+    free(envv);
+///////////////////
 
     // Set the process group for the child. This is also done in the
     // child process see wikipedia(Process_group)
@@ -314,7 +452,7 @@ private:
 };
 
 
-int ExecCmd::doexec(const string &cmd, const list<string>& args,
+int ExecCmd::doexec(const string &cmd, const vector<string>& args,
 		    const string *input, string *output)
 {
 
@@ -405,7 +543,7 @@ int ExecCmd::send(const string& data)
 	    break;
 	int n = con->send(data.c_str() + nwritten, data.length() - nwritten);
 	if (n < 0) {
-	    LOGERR(("ExecCmd::doexec: send failed\n"));
+	    LOGERR(("ExecCmd::send: send failed\n"));
 	    return -1;
 	}
 	nwritten += n;
@@ -503,110 +641,18 @@ bool ExecCmd::maybereap(int *status)
     }
 }
 
-// In child process. Set up pipes, environment, and exec command. 
-// This must not return. exit() on error.
-void ExecCmd::dochild(const string &cmd, const list<string>& args,
-		      bool has_input, bool has_output)
+// Static
+bool ExecCmd::backtick(const std::vector<std::string> cmd, std::string& out)
 {
-    // Start our own process group
-    if (setpgid(0, getpid())) {
-	LOGINFO(("ExecCmd::dochild: setpgid(0, %d) failed: errno %d\n",
-		 getpid(), errno));
-    }
-
-    // Restore SIGTERM to default. Really, signal handling should be
-    // specified when creating the execmd. Help Recoll get rid of its
-    // filter children though. To be fixed one day... Not sure that
-    // all of this is needed. But an ignored sigterm and the masks are
-    // normally inherited.
-    if (signal(SIGTERM, SIG_DFL) == SIG_ERR) {
-	LOGERR(("ExecCmd::dochild: signal() failed, errno %d\n", errno));
-    }
-    sigset_t sset;
-    sigfillset(&sset);
-    pthread_sigmask(SIG_UNBLOCK, &sset, 0);
-    sigprocmask(SIG_UNBLOCK, &sset, 0);
-
-    if (has_input) {
-	close(m_pipein[1]);
-	m_pipein[1] = -1;
-	if (m_pipein[0] != 0) {
-	    dup2(m_pipein[0], 0);
-	    close(m_pipein[0]);
-	    m_pipein[0] = -1;
-	}
-    }
-    if (has_output) {
-	close(m_pipeout[0]);
-	m_pipeout[0] = -1;
-	if (m_pipeout[1] != 1) {
-	    if (dup2(m_pipeout[1], 1) < 0) {
-		LOGERR(("ExecCmd::doexec: dup2(2) failed. errno %d\n", errno));
-	    }
-	    if (close(m_pipeout[1]) < 0) {
-		LOGERR(("ExecCmd::doexec: close(2) failed. errno %d\n", errno));
-	    }
-	    m_pipeout[1] = -1;
-	}
-    }
-    // Do we need to redirect stderr ?
-    if (!m_stderrFile.empty()) {
-	int fd = open(m_stderrFile.c_str(), O_WRONLY|O_CREAT
-#ifdef O_APPEND
-		      |O_APPEND
-#endif
-		      , 0600);
-	if (fd < 0) {
-	    close(2);
-	} else {
-	    if (fd != 2) {
-		dup2(fd, 2);
-	    }
-	    lseek(2, 0, 2);
-	}
-    }
-
-    // Close all descriptors except 0,1,2
-    libclf_closefrom(3);
-
-    // Allocate arg vector (2 more for arg0 + final 0)
-    typedef const char *Ccharp;
-    Ccharp *argv;
-    argv = (Ccharp *)malloc((args.size()+2) * sizeof(char *));
-    if (argv == 0) {
-	LOGERR(("ExecCmd::doexec: malloc() failed. errno %d\n",	errno));
-	exit(1);
-    }
-	
-    // Fill up argv
-    argv[0] = cmd.c_str();
-    int i = 1;
-    list<string>::const_iterator it;
-    for (it = args.begin(); it != args.end(); it++) {
-	argv[i++] = it->c_str();
-    }
-    argv[i] = 0;
-
-#if 0
-    {int i = 0;cerr << "cmd: " << cmd << endl << "ARGS: " << endl; 
-	while (argv[i]) cerr << argv[i++] << endl;}
-#endif
-
-    for (vector<string>::const_iterator it = m_env.begin(); 
-	 it != m_env.end(); it++) {
-#ifdef PUTENV_ARG_CONST
-	::putenv(it->c_str());
-#else
-	::putenv(strdup(it->c_str()));
-#endif
-    }
-    execvp(cmd.c_str(), (char *const*)argv);
-    // Hu ho
-    LOGERR(("ExecCmd::doexec: execvp(%s) failed. errno %d\n", cmd.c_str(),
-	    errno));
-    _exit(127);
+    vector<string>::const_iterator it = cmd.begin();
+    it++;
+    vector<string> args(it, cmd.end());
+    ExecCmd mexec;
+    int status = mexec.doexec(*cmd.begin(), args, 0, &out);
+    return status == 0;
 }
 
+/// ReExec class methods ///////////////////////////////////////////////////
 ReExec::ReExec(int argc, char *args[])
 {
     init(argc, args);
@@ -624,6 +670,48 @@ void ReExec::init(int argc, char *args[])
     free(cd);
 }
 
+void ReExec::insertArgs(const vector<string>& args, int idx)
+{
+    vector<string>::iterator it, cit;
+    unsigned int cmpoffset = (unsigned int)-1;
+
+    if (idx == -1 || string::size_type(idx) >= m_argv.size()) {
+	it = m_argv.end();
+	if (m_argv.size() >= args.size()) {
+	    cmpoffset = m_argv.size() - args.size();
+	}
+    } else {
+	it = m_argv.begin() + idx;
+	if (idx + args.size() <= m_argv.size()) {
+	    cmpoffset = idx;
+	}
+    }
+
+    // Check that the option is not already there
+    if (cmpoffset != (unsigned int)-1) {
+	bool allsame = true;
+	for (unsigned int i = 0; i < args.size(); i++) {
+	    if (m_argv[cmpoffset + i] != args[i]) {
+		allsame = false;
+		break;
+	    }
+	}
+	if (allsame)
+	    return;
+    }
+
+    m_argv.insert(it, args.begin(), args.end());
+}
+
+void ReExec::removeArg(const string& arg)
+{
+    for (vector<string>::iterator it = m_argv.begin(); 
+	 it != m_argv.end(); it++) {
+	if (*it == arg)
+	    it = m_argv.erase(it);
+    }
+}
+
 // Reexecute myself, as close as possible to the initial exec
 void ReExec::reexec()
 {
@@ -678,18 +766,21 @@ void ReExec::reexec()
     execvp(m_argv[0].c_str(), (char *const*)argv);
 }
 
+
 ////////////////////////////////////////////////////////////////////
 #else // TEST
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
+
 #include <string>
 #include <iostream>
-#include <list>
-#include "debuglog.h"
-#include "cancelcheck.h"
-
+#include <vector>
 using namespace std;
 
+#include "debuglog.h"
+#include "cancelcheck.h"
 #include "execmd.h"
 
 static int     op_flags;
@@ -759,6 +850,17 @@ ReExec reexec;
 int main(int argc, char *argv[])
 {
     reexec.init(argc, argv);
+
+    if (0) {
+	vector<string> newargs;
+	newargs.push_back("newarg");
+	newargs.push_back("newarg1");
+	newargs.push_back("newarg2");
+	newargs.push_back("newarg3");
+	newargs.push_back("newarg4");
+	reexec.insertArgs(newargs, 2);
+    }
+
     thisprog = argv[0];
     argc--; argv++;
 
@@ -781,7 +883,7 @@ int main(int argc, char *argv[])
 	Usage();
 
     string cmd = *argv++; argc--;
-    list<string> l;
+    vector<string> l;
     while (argc > 0) {
 	l.push_back(*argv++); argc--;
     }
